#include "MixAnalyzer.h"

#include "../exceptions/MarkerNotFoundException.h"

namespace dma
{
	namespace core
	{

		MixAnalyzer::MixAnalyzer(HbMapPtr hbmin) :
			personNumberAnalyzer_(new PersonNumberAnalyzer(hbmin)), configurationGenerator_(new ConfigurationGenerator(hbmin)), profilesFilter_(new ProfilesFilter()), hbMin_(hbmin),
					mixProportions_(new MixProportionPtrMap()), maxMixProportionDeviation_(0.35)
		{

		}

		MixAnalyzer::~MixAnalyzer()
		{
		}

		unsigned int MixAnalyzer::getNumberOfPersonsInvolved(SamplePtr sample)
		{
			return personNumberAnalyzer_->getMinimumForSample(sample);
		}

		SampleConfigurationPtr MixAnalyzer::performDeconvolution(SamplePtr sample,
				MixProportionPtrMapPtr mixProportions,
				double maxMixProportionDeviation,
				unsigned int maxPersonsNumber,
				CONFIGURATION_GENERATION_METHOD configurationGenerationMethod)
		{

			// Setting mixture proportion
			mixProportions_ = mixProportions;

			// Creating container for marker configurations
			SampleConfigurationPtr output(new SampleConfiguration(sample->getName()));

			// Getting Minimum Number of Persons Involved
			unsigned int mnpi = getNumberOfPersonsInvolved(sample);

			// Generating for all markers possible combinations
			unsigned int i = 0;
			unsigned int size = sample->getMarkerNames()->size();
			for (i = 0; i < size; i++)
			{
				// Fetching marker
				MarkerPtr marker = sample->getMarker(sample->getMarkerNames()->at(i));

				// Generating combinations for marker
				shared_ptr<std::vector<MarkerConfigurationPtr> > markerConfigurations = configurationGenerator_->generate(marker,
						mnpi,
						maxPersonsNumber,
						mixProportions_,
						maxMixProportionDeviation_
						);

				// Adding configuration
				output->addMarkerConfigurations(marker->getName(), markerConfigurations);
			}

			return output;
		}

		void MixAnalyzer::filterSampleByKnownProfile(SampleConfigurationPtr unfiltred, ProfilePtr knownProfile)
		{
			profilesFilter_->filterSampleByKnownProfile(unfiltred, knownProfile);
		}

		double MixAnalyzer::getHBmin(std::string markerName)
		{

			std::map<std::string, double>::iterator it1;
			for (it1 = hbMin_->begin(); it1 != hbMin_->end(); it1++)
			{
				if ((*it1).first.compare(markerName) == 0)
				{
					return (*it1).second;
				}
			}

			// If marker with given name has not been found exception is throwned
			throw MarkerNotFoundException(markerName, "MixAnalyzer::getHBmin");
		}

		void MixAnalyzer::setHBmin(std::string markerName, double hbmin)
		{

			bool flagFound = false;

			std::map<std::string, double>::iterator it1;
			for (it1 = hbMin_->begin(); it1 != hbMin_->end(); ++it1)
			{
				if (it1->first.compare(markerName) == 0)
				{
					it1->second = hbmin;
					flagFound = true;
				}
			}

			// Inserting new entry
			if (flagFound == false)
			{
				hbMin_->insert(make_pair(markerName, hbmin));
			}
		}
	}
}
