/*
 * PeakAllotter.cpp
 *
 *  Created on: Nov 13, 2008
 *      Author: mariusz
 */

#include "PeakAllotter.h"

namespace dma
{
	namespace core
	{

		PeakAllotter::PeakAllotter()
		{

		}

		PeakAllotter::~PeakAllotter()
		{

		}

		void PeakAllotter::performDeconvolution(MarkerConfigurationPtr configuration,
				MarkerPtr marker,
				MixtureProportionPtr mixPropotion)
		{

			double d1, sumOfAllShares, alleleValue;
			unsigned int ui1;

			// Loop control variables
			unsigned int i, j, allelesNumber, lociGenotypeNumber, cwgaSize;

			// Vector containing indexes of Contributors With Given Allele
			shared_ptr<std::vector<unsigned int> > cwga(new std::vector<unsigned int>());

			// Vector containing Mixture Proportion Between Contributors
			shared_ptr<std::vector<double> > mpbc(new std::vector<double>());

			// Vector containing Peak Share For Contributor
			shared_ptr<std::vector<double> > psfc(new std::vector<double>());

			// Getting number of contributors from configuration
			lociGenotypeNumber = configuration->getLociNumber();

			// Iterating through all alleles on mixture
			allelesNumber = marker->getAllelesNumber();
			for (i = 0; i < allelesNumber; i++)
			{

				cwga->clear();
				mpbc->clear();
				psfc->clear();

				// Fetching allele from mixture
				AlleleBasePtr alleleInMixture = marker->getAllele(i);
				alleleValue = alleleInMixture->getAlleleValue();

				// Searching for contributors which genotype includes given allele
				for (j = 0; j < lociGenotypeNumber; j++)
				{
					if (configuration->getLocus(j)->isAllelePresent(alleleValue))
					{
						cwga->push_back(j);
					}
				}

				// Getting number of contributors sharing allele
				cwgaSize = cwga->size();

				// Checking whether allele from mixture should be allotted
				if (cwgaSize == 0)
				{

					// Allele is not present on any of contributors genotypes. Move to next allele in mixture
					continue;
				}

				else
				{

					// Calculating mixture proportion between contributors
					mpbc = mixPropotion->getContributorsShares(cwga);

					// Calculating sum of all shares with respect to proportion and number of alleles of given value on locus
					sumOfAllShares = 0.0;
					for (j = 0; j < cwgaSize; j++)
					{

						ui1 = configuration->getLocus(cwga->at(j))->getZygosityAsNumber();
						sumOfAllShares = sumOfAllShares + mpbc->at(j) * ui1;
					}

					// Calculating share for contributor
					for (j = 0; j < cwgaSize; j++)
					{
						d1 = mpbc->at(j) / sumOfAllShares;
						psfc->push_back(d1);
					}

					assert((cwga->size()==psfc->size()));

					// Distributing peak areas/heights across alleles
					for (j = 0; j < cwgaSize; j++)
					{

						assignPeakAreaHeight(configuration->getLocus(cwga->at(j)), alleleInMixture, psfc->at(j));
					}
				}
			}
		}

		void PeakAllotter::assignPeakAreaHeight(LocusPtr locus, AlleleBasePtr alleleInMixture, double psfc)
		{
			double dPeakArea, dPeakHeight;

			dPeakArea = psfc * alleleInMixture->getPeakArea();
			dPeakHeight = psfc * alleleInMixture->getPeakHeight();

			if ( locus->getAllele(0)->getAlleleValue() == alleleInMixture->getAlleleValue() )
			{
				locus->getAllele(0)->setPeakArea(dPeakArea);
				locus->getAllele(0)->setPeakHeight(dPeakHeight);
			}
			if ( locus->getAllele(1)->getAlleleValue() == alleleInMixture->getAlleleValue() )
			{
				locus->getAllele(1)->setPeakArea(dPeakArea);
				locus->getAllele(1)->setPeakHeight(dPeakHeight);
			}
		}
	}
}
