/*
 * PeakAllotter.h
 *
 *  Created on: Nov 13, 2008
 *      Author: mariusz
 */

#ifndef PEAKALLOTTER_H_
#define PEAKALLOTTER_H_

#include "../models/Marker.h"
#include "../models/MarkerConfiguration.h"
#include "../models/MixtureProportion.h"

namespace dma
{
	namespace core
	{

		/**
		 * Class which is responsible for allottion of peak found in mixture into contributors profiles
		 */
		class PeakAllotter
		{
		public:
			PeakAllotter();
			virtual ~PeakAllotter();

			/**
			 * Method deconvulates peaks for given marker found in mixture sample into profiles of contributors.
			 *
			 *
			 * @param configuration Marker configuration - set loci genotypes for contributors
			 * @param marker Marker from mixture - including peak area and height
			 * @param mixPropotion Mixture proportion
			 */
			void performDeconvolution(MarkerConfigurationPtr configuration, MarkerPtr marker, MixtureProportionPtr mixPropotion);

		private:

			/**
			 * Method set for allele in contributors genotype peak area/height according to peak area/height of allele in mixture, and is's share ratio
			 *
			 * @param Locus Genotype of contributor from configuration
			 * @param alleleInMixture Allele from mixture
			 * @param psfc Peak share for contributor
			 */
			void assignPeakAreaHeight(LocusPtr locus, AlleleBasePtr alleleInMixture, double psfc);
		};
	}
}

#endif /* PEAKALLOTTER_H_ */
