/*
 * PeakRestrictionFilter.h
 *
 *  Created on: Nov 12, 2008
 *      Author: mariusz
 */

#ifndef PEAKRESTRICTIONFILTER_H_
#define PEAKRESTRICTIONFILTER_H_

#include <vector>
#include <boost/shared_ptr.hpp>
#include "Strategies.h"
#include "../models/Marker.h"
#include "../models/Locus.h"
#include "../models/MarkerConfiguration.h"
#include "../models/MixtureProportion.h"
#include "../exceptions/MarkerNotFoundException.h"
#include "../models/statuses/MCStatusExcHeterozygoticImbalance.h"
#include "../models/statuses/MCStatusExcMixtureProportion.h"
#include "../TypeDefinitions.h"

using namespace boost;

namespace dma
{
	namespace core
	{

		class PeakRestrictionFilter;
		typedef boost::shared_ptr<PeakRestrictionFilter> PeakRestrictionFilterPtr;

		/**
		 * Filter used for filtering out combinations which are excluded due to heterozygotic balance and mix proporiton constrains
		 */
		class PeakRestrictionFilter
		{
		private:

			/**
			 * Generation strategy
			 */
			StrategiesPtr strategies_;

			/**
			 * Variable indicating which peak property will be used for calculations - area or height
			 */
			AlleleBase::PEAK_PROPERTY peakProperty_;

			/**
			 * Heterozygotic minimas for markers
			 */
			HbMapPtr hbMin_;

		public:

			PeakRestrictionFilter(StrategiesPtr strategies, HbMapPtr hbMin);

			PeakRestrictionFilter(StrategiesPtr strategies);

			virtual ~PeakRestrictionFilter();

			/**
			 * Method filters out genotype combinations which does not met heterozygotic balance
			 *
			 * @param markerName
			 * @param configurations
			 * @return
			 *
			 */
			MarkerConfigurationPtrVectorPtr filterByHb(std::string markerName, MarkerConfigurationPtrVectorPtr configurations);

			/**
			 * Method filters out genotype combinations which does does not met mixture proportion
			 *
			 * @param configurations
			 * @param mps Mixture proportions
			 * @param mpMaxDev Mixture Proportion max deviation
			 * @return
			 */
			MarkerConfigurationPtrVectorPtr filterByMp( MarkerConfigurationPtrVectorPtr configurations,
					MixProportionPtrMapPtr mpMap,
					double mpMaxDev);

			/**
			 * Retrieve used strategies
			 *
			 * @return
			 */
			StrategiesPtr getStrategies();

		protected:

			/**
			 * Method returns heterozygote balance minimum for given marker. If marker with given name is not found
			 * @class MarkerNotFoundException is generated
			 *
			 * @param markerName
			 * @return
			 * @exception MarkerNotFoundException
			 */
			double getHBMin(std::string markerName);



		};
	}
}

#endif /* PEAKRESTRICTIONFILTER_H_ */
