#include "PersonNumberAnalyzer.h"

namespace dma
{
	namespace core
	{

		PersonNumberAnalyzer::PersonNumberAnalyzer(HbMapPtr hbMin) :
			hbMin_(hbMin)
		{
		}

		PersonNumberAnalyzer::PersonNumberAnalyzer() :
			hbMin_(new HbMap())
		{
		}

		PersonNumberAnalyzer::~PersonNumberAnalyzer()
		{
		}

		unsigned int PersonNumberAnalyzer::getMinimumForSample(SamplePtr sample)
		{

			// Control variables
			unsigned int i;
			MarkerPtr currentMarker;

			// Current minimum number of persons
			int currentMinimum = 0;
			int newMinimum = 0;

			unsigned int size = sample->getMarkerNames()->size();
			for (i = 0; i < size; i++)
			{

				// Getting currently analyzed marker
				currentMarker = sample->getMarker(sample->getMarkerNames()->at(i));
				newMinimum = 0;

				// Getting minimum number of persons for given marker
				newMinimum = getMinimumForMarker(currentMarker);

				if (newMinimum > currentMinimum)
				{
					currentMinimum = newMinimum;
				}
			}

			return currentMinimum;
		}

		unsigned int PersonNumberAnalyzer::getMinimumForMarker(MarkerPtr marker)
		{

			double d1;
			unsigned int allelesNumber = 0;
			int newMinimum = 0;

			// Analyzing only numeric alleles
			if (marker->getType() == AlleleBase::NUMERIC)
			{

				// Getting number of alleles present in given marker
				allelesNumber = marker->getAllelesNumber();

				if (allelesNumber == 1)
				{

					newMinimum = 1;
				}
				else if (allelesNumber == 2)
				{

					std::string markerName = marker->getName();

					// Getting heterozygote balance for given marker
					double hbmin = getHBMin(marker->getName());

					// Checking whether heterozygote balance is met
					double hb = marker->calculateHb(0, 1, AlleleBase::PEAK_PROPERTY_AREA);

					if (hb >= hbmin)
					{

						newMinimum = 1;
					} else {

						newMinimum = 2;
					}
				}
				else if (allelesNumber > 2)
				{

					// Calculating number of persons involved
					d1 = ceil((double) marker->getAllelesNumber() / 2);
					newMinimum = (int) d1;
				}

				return newMinimum;
			}
			else
			{

				return 1;
			}
		}

		double PersonNumberAnalyzer::getHBMin(std::string markerName)
		{

			std::map<std::string, double>::iterator it1;
			for (it1 = hbMin_->begin(); it1 != hbMin_->end(); it1++)
			{
				if ((*it1).first.compare(markerName) == 0)
				{
					return (*it1).second;
				}
			}

			// If marker with given name has not been found exception is throwned
			throw MarkerNotFoundException(markerName, "PersonNumberAnalyzer::getHBmin");
		}
	}
}
