#ifndef PERSONNUMBERANALYZER_H_
#define PERSONNUMBERANALYZER_H_

#include "../TypeDefinitions.h"
#include "../models/Sample.h"
#include <boost/shared_ptr.hpp>
#include <cmath>

namespace dma
{
	namespace core
	{


		class PersonNumberAnalyzer;
		typedef boost::shared_ptr<PersonNumberAnalyzer> PersonNumberAnalyzerPtr;

		/**
		 * Interface for classes used for calculating minimum number of persons which can be involved in mixture
		 */
		class PersonNumberAnalyzer
		{

		protected:
			/**
			 * Analyzed sample
			 */
			SamplePtr sample_;

			/**
			 * Minimum values of HB - heterozygotic balance - for each of the markers from set
			 */
			HbMapPtr hbMin_;

		public:

			PersonNumberAnalyzer(HbMapPtr hbMin);

			virtual ~PersonNumberAnalyzer();

			/**
			 * Method returns minimum number of persons, which are necessary to explain mixture for given sample
			 */
			unsigned int getMinimumForSample(SamplePtr sample);

			/**
			 * Method returns minimum number of persons, which are necessary to explain mixture for given marker
			 */
			unsigned int getMinimumForMarker(MarkerPtr marker);

		protected:

			PersonNumberAnalyzer();

			/**
			 * Method returns heterozygote balance minimum for given marker. If marker with given name is not found
			 * @class MarkerNotFoundException is generated
			 *
			 * @param markerName
			 * @return
			 * @exception MarkerNotFoundException
			 */
			double getHBMin(std::string markerName);
		};
	}
}

#endif /*PERSONNUMBERANALYZER_H_*/
