/*
 * PopulationDataAnalyzer.cpp
 *
 *  Created on: Feb 1, 2009
 *      Author: mariusz
 */

#include "PopulationDataAnalyzer.h"

namespace dma
{
	namespace core
	{

		PopulationDataAnalyzer::PopulationDataAnalyzer(MarkerPtrVectorPtr populationData) :
			populationData_(new MarkerPtrMap())
		{

			// Inserting data into map
			MarkerPtrVector::const_iterator it1;
			for (it1 = populationData->begin(); it1 != populationData->end(); it1++)
			{

				populationData_->insert(make_pair((*it1)->getName(), *it1));
			}
		}

		PopulationDataAnalyzer::~PopulationDataAnalyzer()
		{

		}

		MarkerPtrVectorPtr PopulationDataAnalyzer::verifyPopulationData(SampleConfigurationPtr sample,
				FILTER_MARKERS filterMarkers)
		{

			// Creating output container
			MarkerPtrVectorPtr output(new MarkerPtrVector());

			// Iterating throught markers within the configuration
			shared_ptr<std::vector<std::string> > markerNames = sample->getMarkerNames();
			std::vector<std::string>::const_iterator it1;

			for (it1 = markerNames->begin(); it1 != markerNames->end(); it1++)
			{

				// Getting configurations for marker and iterating through them
				MarkerConfigurationPtrVectorPtr mcs = sample->getMarkerConfigurations(*it1);
				MarkerConfigurationPtrVector::const_iterator it2;

				for (it2 = mcs->begin(); it2 != mcs->end(); it2++)
				{

					// Checking whether to filtering consider marker
					if (filterMarkers == FILTER_MARKERS_YES && (*it2)->getType() != AlleleBase::NUMERIC)
					{
						break;
					}

					// Getting alleles present in configuration
					shared_ptr<std::vector<AlleleBasePtr> > allelesInMc = (*it2)->getUniqueAlleles();

					// Checking whether alleles exists in population data
					std::vector<AlleleBasePtr>::const_iterator it3;
					for (it3 = allelesInMc->begin(); it3 != allelesInMc->end(); it3++)
					{

						bool flag = isAlleleInPopulationData(*it1, *it3);
						if (flag == false)
						{

							addAllele(output, *it1, *it3);
						}
					}
				}
			}

			return output;
		}

		bool PopulationDataAnalyzer::isAlleleInPopulationData(std::string markerName, AlleleBasePtr allele)
		{

			// Checking whether marker with given name exists
			MarkerPtrMap::iterator it1 = populationData_->find(markerName);

			if (it1 != populationData_->end())
			{

				return (*it1).second->isAllelePresent(allele->getAlleleValue());

			}
			else
			{

				// MARKER IS NOT PRESENT IN POPULATION DATA
				return false;
			}
		}

		void PopulationDataAnalyzer::addAllele(MarkerPtrVectorPtr container, std::string markerName, AlleleBasePtr allele)
		{

			MarkerPtr marker(new Marker(""));

			// Checking whether marker exists in container
			MarkerPtrVector::const_iterator it1;
			for (it1 = container->begin(); it1 != container->end(); it1++)
			{

				if ((*it1)->getName().compare(markerName) == 0)
				{

					marker = *it1;
				}
			}

			// If marker has not been found creating one and adding it to container
			if (marker->getName().compare("") == 0)
			{

				marker.reset(new Marker(markerName));
				container->push_back(marker);
			}

			// Adding allele to marker
			if (marker->isAllelePresent(allele->getAlleleValue()) == false)
			{

				try
				{
					marker->addAllele(allele);
				} catch (AlleleTypeConflictException e)
				{
					throw AlleleTypeConflictException(e.getOldAlleleType(),
							e.getNewAlleleType(),
							e.getMarkerName(),
							"PopulationDataAnalyzer::addAllele");
				}
			}
		}
	}
}
