/*
 * PopulationDataAnalyzer.h
 *
 *  Created on: Feb 1, 2009
 *      Author: mariusz
 */

#ifndef POPULATIONDATAANALYZER_H_
#define POPULATIONDATAANALYZER_H_

#include <map>
#include <string>
#include <boost/shared_ptr.hpp>

#include "../models/Marker.h"
#include "../models/SampleConfiguration.h"

using namespace boost;

namespace dma
{
	namespace core
	{

		/**
		 * Class used for analysis of population data. Role of the class is to check whether all allele frequencies from given mixture sample
		 * is given population data
		 */
		class PopulationDataAnalyzer
		{
		public:

			enum FILTER_MARKERS
			{

				FILTER_MARKERS_NO, FILTER_MARKERS_YES
			};

		private:

			/**
			 * Population data
			 */
			MarkerPtrMapPtr populationData_;

		public:

			/**
			 * Population data analyzer
			 * @return
			 */
			PopulationDataAnalyzer(MarkerPtrVectorPtr populationData);

			virtual ~PopulationDataAnalyzer();

			/**
			 * Method checks whether for alleles which are present in configurations from sample their frequencies in population is given.
			 * Alleles for which frequency is not given are given in result.
			 *
			 * @param sample
			 * @param filterMarkers Variable indicating whether character alleles should be filtered out
			 * @return
			 */
			MarkerPtrVectorPtr verifyPopulationData(SampleConfigurationPtr sample, FILTER_MARKERS filterMarkers =
					FILTER_MARKERS_YES);

		private:

			/**
			 * Method checks whether allele given on marker is present in population data
			 *
			 * @param markerName
			 * @param allele
			 * @return
			 */
			bool isAlleleInPopulationData(std::string markerName, AlleleBasePtr allele);

			/**
			 * Add given allele to container. Method checks whether given allele did not already existed. If so nothing is done
			 *
			 * @param container
			 * @param allele
			 */
			void addAllele(MarkerPtrVectorPtr container, std::string markerName, AlleleBasePtr allele);

		};
	}
}

#endif /* POPULATIONDATAANALYZER_H_ */
