/*
 * ProfilesFilter.cpp
 *
 *  Created on: Dec 11, 2008
 *      Author: mariusz
 */

#include "ProfilesFilter.h"

namespace dma
{
	namespace core
	{

		ProfilesFilter::ProfilesFilter() :
			strategies_(new Strategies())
		{

		}

		ProfilesFilter::~ProfilesFilter()
		{

		}

		StrategiesPtr ProfilesFilter::getStrategies() const
		{

			return strategies_;
		}

		void ProfilesFilter::filterSampleByKnownProfile(SampleConfigurationPtr unfiltred, ProfilePtr knownProfile)
		{

			std::string markerName;

			// Getting names of markers within the known profile
			shared_ptr<std::vector<std::string> > markerNames = knownProfile->getMarkerNames();

			// Getting marker names and iterating through all markers
			std::vector<std::string>::iterator it1;

			for (it1 = markerNames->begin(); it1 != markerNames->end(); it1++)
			{

				// Getting marker name
				markerName = *it1;

				// Filtering configurations
				MarkerConfigurationPtrVectorPtr markerConfigurations = unfiltred->getMarkerConfigurations(markerName);
				filterMarkerConfigurationsByKnownProfile(markerConfigurations, markerName, knownProfile);
			}
		}

		void ProfilesFilter::filterMarkerConfigurationsByKnownProfile(MarkerConfigurationPtrVectorPtr unfiltred,
				std::string markerName,
				ProfilePtr knownProfile)
		{

			LocusPtr locus;
			MarkerConfigurationPtrVector::iterator it1;

			// Checking whether profile includes given marker
			bool markerFound = false;
			shared_ptr<std::vector<std::string> > markerNamesInProfile = knownProfile->getMarkerNames();
			std::vector<std::string>::const_iterator it2;
			for (it2 = markerNamesInProfile->begin(); it2 != markerNamesInProfile->end(); it2++)
			{

				if (markerName.compare(*it2) == 0)
				{
					markerFound = true;
					break;
				}
			}

			// If marker is not within marker in known profile end filtering
			if (markerFound == false)
			{
				return;
			}

			// Getting locus for given marker in known profile
			locus = knownProfile->getMarker(markerName);

			// Filtering configurations
			for (it1 = unfiltred->begin(); it1 != unfiltred->end(); it1++)
			{

				// Analyzing only configurations with status valid
				if ((*it1)->isValid() )
				{

					if ((*it1)->isLocusPresent(locus) == false)
					{

						// Checking whether to exclude configuration or only mark it as invalid
						if (strategies_->getExcludedConfigurationsStrategy() == Strategies::ECS_ASSIGN_STATUS)
						{

							MCStatusExcKnownProfilePtr es(new MCStatusExcKnownProfile(locus));
							(*it1)->setStatus(es);
						}
						else
						{

							unfiltred->erase(it1);
							if (it1 == unfiltred->end())
							{
								break;
							}
						}
					}
				}
			}
		}
	}
}
