/*
 * AlleleNotFoundException.h
 *
 *  Created on: Nov 13, 2008
 *      Author: mariusz
 */

#ifndef ALLELENOTFOUNDEXCEPTION_H_
#define ALLELENOTFOUNDEXCEPTION_H_

#include "DmaException.h"
#include <boost/lexical_cast.hpp>

namespace dma
{
	namespace core
	{

		/**
		 *	Exception is throwned when search for allele with given value fails
		 */
		class AlleleNotFoundException: public DmaException
		{
		private:

			/**
			 * Value of searched allele
			 */
			double alleleValue_;

			/**
			 * Name of marker on which allele has been searched
			 */
			std::string markerName_;

			/**
			 * Operation during which exception has been reported
			 */
			std::string operation_;

		public:

			/**
			 * Exception related to unavailability of allele
			 *
			 * @param markerName Name of marker on which allele should be present
			 * @param alleleValue Value of allele which has been searched
			 * @param operation Operation during which exception has been reported
			 * @return
			 */
			AlleleNotFoundException(const std::string& markerName, double alleleValue, const std::string& operation) :
				DmaException(), alleleValue_(alleleValue), markerName_(markerName), operation_(operation)
			{

			}

			~AlleleNotFoundException() throw()
			{

			}

			const char* what() const throw()
			{

				std::string s = "Exception - Allele not found";
				return s.c_str();
			}

			const char* getDescription() const throw()
			{

				std::string s = "";
				s = s + "Error has been reported while performing operation '" + operation_ + "': ";
				s = s + "Couldn't find allele with value ";
				s = s + boost::lexical_cast<std::string>(alleleValue_);
				s = s + " on marker '" + markerName_ + "'";
				return s.c_str();
			}
		};
	}
}

#endif /* ALLELENOTFOUNDEXCEPTION_H_ */
