/*
 * AlleleTypeConflictException.h
 *
 *  Created on: Oct 19, 2008
 *      Author: mariusz
 */

#ifndef ALLELETYPECONFLICTEXCEPTION_H_
#define ALLELETYPECONFLICTEXCEPTION_H_

#include "DmaException.h"
#include "../models/AlleleBase.h"

namespace dma
{
	namespace core
	{

		/**
		 * Exception is throwned when allele to-be-added to locus/marker is of different
		 * than alleles already present of locus/marker
		 */
		class AlleleTypeConflictException: public DmaException
		{
		protected:

			/**
			 * Type of alleles present already on locus/marker
			 */
			AlleleBase::ALLELE_TYPE oldAlleleType_;

			/**
			 * Type of allele to be added
			 */
			AlleleBase::ALLELE_TYPE newAlleleType_;

			/**
			 * Marker on which exception has been reported
			 */
			std::string markerName_;

			/**
			 * Operation during which exception has been reported
			 */
			std::string operation_;
		public:

			/**
			 *
			 * @param oldAlleleType Type of alleles present already on locus/marker
			 * @param newAlleleType Type of allele to be added
			 * @param operation Operation during which exception has been reported
			 * @return
			 */
			AlleleTypeConflictException(AlleleBase::ALLELE_TYPE oldAlleleType,
					AlleleBase::ALLELE_TYPE newAlleleType,
					const std::string& markerName,
					const std::string& operation) throw() :
				DmaException(), oldAlleleType_(oldAlleleType), newAlleleType_(newAlleleType), markerName_(markerName),
						operation_(operation)
			{

			}

			~AlleleTypeConflictException() throw()
			{

			}

			/**
			 *
			 * @return Type of alleles present already on locus/marker
			 */
			AlleleBase::ALLELE_TYPE getOldAlleleType() const
			{
				return oldAlleleType_;
			}

			/**
			 *
			 * @return Type of allele to be added
			 */
			AlleleBase::ALLELE_TYPE getNewAlleleType() const
			{
				return newAlleleType_;
			}

			/**
			 *
			 * @return Name of marker on which operation has been performed
			 */
			std::string getMarkerName() const
			{

				return markerName_;
			}

			/**
			 *
			 * @return Name of operation during which exception has been reported
			 */
			std::string gerOperation() const
			{

				return operation_;
			}

			const char* what() const throw()
			{

				std::string s = "Exception - Allele type conflict";
				return s.c_str();
			}

			const char* getDescription() const throw()
			{

				std::string s = "";
				s = s + "Error has been reported while performing operation '" + operation_ + "': ";
				s = s + "Allele conflict has been has been reported. Attempt to add allele of type ";
				s = s + AlleleBase::getTypeAsChar(newAlleleType_);
				s = s + " to locus/marker which contained alleles of type ";
				s = s + AlleleBase::getTypeAsChar(oldAlleleType_);
				return s.c_str();
			}
		};
	}
}

#endif /* ALLELETYPECONFLICTEXCEPTION_H_ */
