/*
 * MarkerNotFoundException.h
 *
 *  Created on: 2008-10-02
 *      Author: mariusz
 */

#ifndef MARKERNOTFOUNDEXCEPTION_H_
#define MARKERNOTFOUNDEXCEPTION_H_

#include <iostream>
#include <string>
#include "DmaException.h"

namespace dma
{
	namespace core
	{

		/**
		 *	Exception is throwned when search for marker with given name fails
		 */
		class MarkerNotFoundException: public DmaException
		{
		private:

			/**
			 * Name of searched marker
			 */
			std::string markerName_;

			/**
			 * Operation during which exception has been reported
			 */
			std::string operation_;

		public:
			/**
			 * Constructor
			 *
			 * @param markerName Name of marker searched
			 * @param operation Operation during which exception has been reported
			 * @return
			 */
			MarkerNotFoundException(const std::string& markerName, const std::string& operation) throw() :
				DmaException(), markerName_(markerName), operation_(operation)
			{

			}

			~MarkerNotFoundException() throw()
			{

			}

			std::string getMarkerName() const
			{

				return markerName_;
			}

			const char* what() const throw()
			{

				std::string s = "Exception - Marker not found";
				return s.c_str();
			}

			const char* getDescription() const throw()
			{

				std::string s = "";
				s = s + "Error has been reported while performing operation '" + operation_ + "': ";
				s = s + "Couldn't find marker with name '" + markerName_ + "'";
				return s.c_str();
			}
		};
	}
}

#endif /* MARKERNOTFOUNDEXCEPTION_H_ */
