/*
 * OutOfBoundsException.h
 *
 *  Created on: Oct 12, 2008
 *      Author: mariusz
 */

#ifndef OUTOFBOUNDSEXCEPTION_H_
#define OUTOFBOUNDSEXCEPTION_H_

#include "DmaException.h"
#include <boost/lexical_cast.hpp>

namespace dma
{
	namespace core
	{

		/**
		 * Exception is generated when attempt is made to access element from container which index exceeds container size
		 */
		class OutOfBoundsException: public DmaException
		{
		private:

			/**
			 *	Accessed index
			 */
			unsigned int index_;

			/**
			 * Size of container
			 */
			unsigned int size_;

			/**
			 * Name of container accessed
			 */
			std::string containerName_;

		public:

			/**
			 * Constructor
			 *
			 * @param index Index of element to be accessed
			 * @param size Size of container
			 * @param containerName Name of container accessed
			 * @return
			 */
			OutOfBoundsException(unsigned int index, unsigned int size, const std::string& containerName = "") :
				index_(index), size_(size), containerName_(containerName)
			{

			}

			~OutOfBoundsException() throw()
			{

			}

			/**
			 *
			 * @return Index of element to-be-accessed
			 */
			unsigned int getIndex() const
			{
				return index_;
			}

			/**
			 *
			 * @return Size of container
			 */
			unsigned int getSize() const
			{
				return size_;
			}

			/**
			 *
			 * @return Name of container accessed
			 */
			std::string getContainerName() const
			{

				return containerName_;
			}

			const char* what() const throw()
			{

				std::string s = "Exception - Out of bounds";
				return s.c_str();
			}

			const char* getDescription() const throw()
			{

				std::string s = "";
				s = s + "Error has been reported while accessing container '" + containerName_ + "': ";
				s = s + "Couldn't get element with index " + boost::lexical_cast<std::string>(index_)
						+ " from container with size " + boost::lexical_cast<std::string>(size_);
				return s.c_str();
			}
		};
	}
}

#endif /* OUTOFBOUNDSEXCEPTION_H_ */
