/*
 * PopulationDataException.h
 *
 *  Created on: Feb 1, 2009
 *      Author: mariusz
 */

#ifndef POPULATIONDATAEXCEPTION_H_
#define POPULATIONDATAEXCEPTION_H_

#include <string>
#include "DmaException.h"
#include "../models/Marker.h"

namespace dma
{
	namespace core
	{

		/**
		 * Exception is throwned when for an allele from sample it's frequency is not given in population data
		 */
		class PopulationDataException: public DmaException
		{
		private:

			/**
			 * Vector containing markers with alleles for which frequency is not given
			 */
			MarkerPtrVectorPtr missingAlleles_;

		public:

			PopulationDataException(MarkerPtrVectorPtr missingAlleles) throw() :
				missingAlleles_(missingAlleles)
			{

			}

			~PopulationDataException() throw()
			{

			}

			const char* what() const throw()
			{

				std::string s = "Exception - Allele frequencies not given in population data";
				return s.c_str();
			}

			const char* getDescription() const throw()
			{

				unsigned int i, j, size1, size2;

				std::string s = "";
				s = s + "Frequency of allele is not given in population data for following alleles: ";

				size1 = missingAlleles_->size();
				for (i = 0; i < size1; i++)
				{

					s = s + "Marker ";
					s = s + missingAlleles_->at(i)->getName();
					s = s + " ( ";

					// Start of allele list
					size2 = missingAlleles_->at(i)->getAllelesNumber();
					for (j = 0; j < size2; j++)
					{

						s = s + missingAlleles_->at(i)->getAllele(j)->getAlleleValueAsString();

						if (j != (size2 - 1))
						{
							s = s + ", ";
						}
					}
					// Enf of allele list

					if (i != (size1 - 1))
					{
						s = s + " ), ";
					}
					else
					{
						s = s + " ) ";
					}
				}

				return s.c_str();
			}
		};
	}
}

#endif /* POPULATIONDATAEXCEPTION_H_ */
