#include "AlleleBase.h"

namespace dma
{
	namespace core
	{

		AlleleBase::AlleleBase(double alleleValue,
				double alleleSize,
				double peakHeight,
				double peakArea,
				AlleleBase::ALLELE_TYPE type) :
			alleleValue_(alleleValue), alleleSize_(alleleSize), peakHeight_(peakHeight), peakArea_(peakArea), type_(type),
					frequency_(0), markerName_("")
		{

		}

		AlleleBase::~AlleleBase()
		{
		}

		double AlleleBase::getAlleleValue() const
		{
			return alleleValue_;
		}

		double AlleleBase::getAlleleSize() const
		{
			return alleleSize_;
		}

		double AlleleBase::getPeakHeight() const
		{
			return peakHeight_;
		}

		double AlleleBase::getPeakArea() const
		{
			return peakArea_;
		}

		void AlleleBase::setPeakHeight(double peakHeight)
		{

			peakHeight_ = peakHeight;
		}

		void AlleleBase::setPeakArea(double peakArea)
		{

			peakArea_ = peakArea;
		}

		AlleleBase::ALLELE_TYPE AlleleBase::getType() const
		{
			return type_;
		}

		char AlleleBase::getTypeAsChar() const
		{
			return AlleleBase::getTypeAsChar(type_);
		}

		char AlleleBase::getTypeAsChar(ALLELE_TYPE alleleType)
		{
			switch (alleleType)
			{
			case AlleleBase::NUMERIC:
				return AlleleBase::ALLELE_TYPE_NUMERIC;
			case AlleleBase::CHARACTER:
				return AlleleBase::ALLELE_TYPE_CHARACTER;
			default:
				return AlleleBase::ALLELE_TYPE_UNDEFINED;
			}
		}

		double AlleleBase::getFrequency() const
		{
			return frequency_;
		}

		AlgValuePtr AlleleBase::getFrequencyAsAlgValue() const
		{
			std::string s = "p_";
			s = s + getAlleleValueAsString();

			AlgValuePtr rv(new AlgValue(s, frequency_));
			return rv;
		}

		void AlleleBase::setFrequency(double frequency)
		{
			frequency_ = frequency;
		}

		std::string AlleleBase::getMarkerName() const
		{

			return markerName_;
		}

		void AlleleBase::setMarkerName(std::string markerName)
		{
			markerName_ = markerName;
		}
	}
}
