#ifndef CHARACTERALLELE_H_
#define CHARACTERALLELE_H_

#include <utility>
#include <cmath>
#include "AlleleBase.h"
#include <boost/shared_ptr.hpp>

namespace dma
{
	namespace core
	{

		class CharacterAllele;
		typedef shared_ptr<CharacterAllele> CharacterAllelePtr;

		typedef std::pair<CharacterAllelePtr, CharacterAllelePtr> CharacterAllelePtrPair;

		/**
		 * Allele which value is represented by character i.e. 'X' and 'Y' for AMEL marker
		 */
		class CharacterAllele: public AlleleBase
		{
		public:

			/**
			 * Constructor for allele which value is represented as character
			 *
			 * @param alleleValue Character representing allele
			 * @param peakHeight Peak height given in RFU
			 * @param peakSize Peak size given in RFU
			 * @return
			 */
			CharacterAllele(char alleleValue, double peakHeight = 0.0, double peakArea = 0.0);

			virtual ~CharacterAllele();

			/**
			 * @see AlleleBase::cloneToSmartPtr()
			 */
			AlleleBasePtr cloneToPtr() const;

			/**
			 * @see AlleleBase::getAlleleValueAsString()
			 */
			std::string getAlleleValueAsString() const;

			/**
			 * @see AlleleBase::toString()
			 */
			std::string toString() const;

			/**
			 * Returns allele vale as character
			 * @return
			 */
			char getAlleleValueAsChar() const;
		};
	}
}
#endif /*CHARACTERALLEL_H_*/
