#ifndef HYPOTHESIS_H_
#define HYPOTHESIS_H_

#include <string>
#include <vector>
#include "Person.h"
#include <boost/shared_ptr.hpp>
#include "../exceptions/OutOfBoundsException.h"

using namespace boost;

namespace dma
{
	namespace core
	{

		class Hypothesis;
		typedef shared_ptr<Hypothesis> HypothesisPtr;

		class Hypothesis
		{
		public:

			/**
			 * Type of hypothesis
			 */
			enum TYPE
			{
				PROSECUTOR, DEFENCE
			};
		private:

			/**
			 * Hypothesis name - code
			 */
			std::string name_;

			/**
			 * Type of hypothesis
			 */
			TYPE type_;

			/**
			 * Persons to-be-involved in mixture
			 */
			shared_ptr<std::vector<PersonPtr> > contributors_;

		public:

			/**
			 * Constructor
			 * @param name Hypothesis name
			 * @param type Type of hypothesis
			 * @return
			 */
			Hypothesis(std::string name, TYPE type);

			virtual ~Hypothesis();

			/**
			 * Method returns type hypothesis
			 * @return
			 */
			TYPE getType() const;

			/**
			 * Add additional person as a contributor to hypothesis
			 * @param person
			 */
			void addContributor(PersonPtr contributor);

			/**
			 * Method returns number of contributors to the mixture under hypothesis
			 * @return
			 */
			unsigned int getNumberOfContributors() const;

			/**
			 * Method returns given contributor
			 * @param index
			 * @return
			 * @exception OutOfBoundsException
			 */
			PersonPtr getContributor(unsigned int index) const;
		};
	}
}

#endif /*HYPOTHESIS_H_*/
