/*
 * LikelihoodRatio.cpp
 *
 *  Created on: Dec 22, 2008
 *      Author: mariusz
 */

#include "LikelihoodRatio.h"

namespace dma
{
	namespace core
	{

		LikelihoodRatio::LikelihoodRatio() :
			lrForMarkers_(new std::map<std::string, AlgExpPtr>())
		{

		}

		LikelihoodRatio::~LikelihoodRatio()
		{

		}

		void LikelihoodRatio::addLRForMarker(std::string markerName, AlgExpPtr lr)
		{

			// If marker has already existed - deleting the old one
			std::map<std::string, AlgExpPtr>::iterator it = lrForMarkers_->find( markerName );
			if (it != lrForMarkers_->end())
			{

				lrForMarkers_->erase(it);
			}

			lrForMarkers_->insert(make_pair(markerName, lr));
		}

		shared_ptr<std::vector<std::string> > LikelihoodRatio::getMarkerNames() const
		{

			shared_ptr<std::vector<std::string> > markerNames(new std::vector<std::string>());

			std::map<std::string, AlgExpPtr>::iterator it1;
			;
			for (it1 = lrForMarkers_->begin(); it1 != lrForMarkers_->end(); it1++)
			{

				markerNames->push_back((*it1).first);
			}

			return markerNames;
		}

		AlgExpPtr LikelihoodRatio::getLikelihoodRatio(std::string markerName) const
		{

			std::map<std::string, AlgExpPtr>::iterator it1;
			for (it1 = lrForMarkers_->begin(); it1 != lrForMarkers_->end(); it1++)
			{

				if ((*it1).first.compare(markerName) == 0)
				{
					return (*it1).second;
				}
			}

			throw MarkerNotFoundException(markerName, "LikelihoodRatio::getLikelihoodRatio()");
		}

		AlgExpPtr LikelihoodRatio::calculateTotalLr() const
		{

			// Checking whether any likelihood ratio's exists
			if (lrForMarkers_->size() == 0)
			{

				AlgValuePtr rv(new AlgValue("0", 0.0));
				return rv;
			}
			else if (lrForMarkers_->size() == 1)
			{

				return lrForMarkers_->begin()->second;
			}
			else if (lrForMarkers_->size() == 2)
			{

				std::map<std::string, AlgExpPtr>::iterator it1;
				it1 = lrForMarkers_->begin();
				AlgOperMultiPtr rv(new AlgOperMulti((it1++)->second, it1->second));
				return rv;
			}
			else
			{

				std::map<std::string, AlgExpPtr>::iterator it1;
				it1 = lrForMarkers_->begin();
				AlgOperMultiPtr rv(new AlgOperMulti((it1++)->second, (it1++)->second));
				for (; it1 != lrForMarkers_->end(); it1++)
				{
					rv->addExp(it1->second);
				}

				return rv;
			}
		}

	}
}
