/*
 * LikelihoodRatio.h
 *
 *  Created on: Dec 22, 2008
 *      Author: mariusz
 */

#ifndef LIKELIHOODRATIO_H_
#define LIKELIHOODRATIO_H_

#include <map>
#include <boost/shared_ptr.hpp>
#include "../exceptions/MarkerNotFoundException.h"
#include "../models/algebra/AlgValue.h"
#include "../models/algebra/AlgOperMulti.h"

using namespace boost;

namespace dma
{
	namespace core
	{

		class LikelihoodRatio;
		typedef shared_ptr<LikelihoodRatio> LikelihoodRatioPtr;

		/**
		 * Class encapsulates likelihood ratio for every marker form given set
		 *
		 * Internal storage is realized by map.
		 */
		class LikelihoodRatio
		{
		private:

			/**
			 * Likelihood ratio for every marker
			 */
			shared_ptr<std::map<std::string, AlgExpPtr> > lrForMarkers_;

		public:

			LikelihoodRatio();

			virtual ~LikelihoodRatio();

			/**
			 * Add likelihood ratio for given marker. If marker already exists value of lr is replaced by new one
			 *
			 * @param markerName
			 * @param lr
			 */
			void addLRForMarker(std::string markerName, AlgExpPtr lr);

			/**
			 * Generate algebraic expression representing total likelihood ratio for whole genotype.
			 * Calculations are based on product rule of probability.
			 *
			 * @return
			 */
			AlgExpPtr calculateTotalLr() const;

			/**
			 * Get likelihood ratio for given marker. If marker is not found @class MarkerNotFoundException is throwned
			 *
			 * @param markerName
			 * @return
			 * @exception MarkerNotFoundException
			 */
			AlgExpPtr getLikelihoodRatio(std::string markerName) const;

			/**
			 * Get markers names
			 * @return
			 */
			shared_ptr<std::vector<std::string> > getMarkerNames() const;

		};
	}
}

#endif /* LIKELIHOODRATIO_H_ */
