/*
 * MixtureProportion.cpp
 *
 *  Created on: Nov 13, 2008
 *      Author: mariusz
 */

#include "MixtureProportion.h"

namespace dma
{
	namespace core
	{

		MixtureProportion::MixtureProportion() :
			proportions_(new std::vector<double>())
		{

		}

		MixtureProportion::~MixtureProportion()
		{

		}

		void MixtureProportion::addContributor(double contributorShare)
		{

			proportions_->push_back(contributorShare);
		}

		double MixtureProportion::getContributorShare(unsigned int index) const
		{

			if (index < proportions_->size())
			{

				// Calculating sum of all contributors
				std::vector<double>::iterator it1;
				double sum = 0;
				for (it1 = proportions_->begin(); it1 != proportions_->end(); it1++)
				{
					sum = sum + (*it1);
				}

				// Performing normalization - contributors proportion to total sum of shares
				return proportions_->at(index) / sum;
			}
			else
			{

				throw OutOfBoundsException(index, proportions_->size(), "MixtureProportion::getContributorShare()::proportions_");
			}
		}

		unsigned int MixtureProportion::getContributorsNumber() const
		{
			return proportions_->size();
		}

		shared_ptr<std::vector<double> > MixtureProportion::getContributorsShares(const shared_ptr<std::vector<unsigned int> > indexes) const
		{

			unsigned int i;
			unsigned int size = indexes->size();
			double d1;

			// Checking whether all indexes are valid
			for (i = 0; i < size; i++)
			{
				if (indexes->at(i) >= proportions_->size())
				{

					throw OutOfBoundsException(indexes->at(i),
							proportions_->size(),
							"MixtureProportion::getContributorShares()::proportions_");
				}
			}

			// Creating output array
			shared_ptr<std::vector<double> > output(new std::vector<double>());

			// Calculating sum of all contributors shares
			double sum = 0.0;
			for (i = 0; i < size; i++)
			{
				sum = sum + proportions_->at(indexes->at(i));
			}

			// Calculating relative proportions
			for (i = 0; i < size; i++)
			{
				d1 = proportions_->at(indexes->at(i)) / sum;
				output->push_back(d1);
			}

			return output;
		}

		std::string MixtureProportion::toString()
		{

			std::string buffer = "Mixture proportion: ";
			unsigned int i;
			unsigned int size = getContributorsNumber();
			for (i = 0; i < size; i++)
			{
				buffer = buffer + lexical_cast<std::string> (getContributorShare(i)) + "\t";
			}

			return buffer;
		}

		MixtureProportionPtr MixtureProportion::parse(std::string input) throw (std::runtime_error)
		{

			MixtureProportionPtr output(new MixtureProportion());
			double d1;

			// Creating tokens container
			std::vector<std::string> s_proportions;
			std::vector<double> d_proportions;

			// Defining tokenizer
			typedef boost::tokenizer<boost::char_separator<char> > char_tokenizer;

			// Creating tokenizer
			boost::char_separator<char> sep(":", "", boost::keep_empty_tokens);
			char_tokenizer tokens(input, sep);

			// Extracting data
			for (char_tokenizer::iterator tok_iter = tokens.begin(); tok_iter != tokens.end(); ++tok_iter)
			{
				try
				{

					// Parsing
					d1 = lexical_cast<double> (*tok_iter);
					d_proportions.push_back(d1);
				} catch (bad_lexical_cast& e)
				{

					// Generating error message and exception
					std::string errMsg = "Unable to parse mix proportion with value '" + (*tok_iter) + "'";
					throw std::runtime_error(errMsg);
				}
			}

			// Creating mixture proportion
			std::vector<double>::iterator it1;
			for (it1 = d_proportions.begin(); it1 != d_proportions.end(); it1++)
			{

				output->addContributor(*it1);
			}

			// Checking whether and contributors has been added
			if (output->getContributorsNumber() == 0)
			{

				// Generating error message and exception
				std::string errorMsg = "Mixture proportion parsing failed - no contributors where detected ( string empty? )";
				throw std::runtime_error(errorMsg);
			}

			return output;
		}
	}
}
