#ifndef NUMERICALLELE_H_
#define NUMERICALLELE_H_

#include <utility>
#include <string>
#include <sstream>
#include "AlleleBase.h"
#include <boost/shared_ptr.hpp>

namespace dma
{
	namespace core
	{

		class NumericAllele;
		typedef shared_ptr<NumericAllele> NumericAllelePtr;

		typedef std::pair<NumericAllelePtr, NumericAllelePtr> NumericAllelePtrPair;

		/**
		 * Allele which value represents number of tandem repeats
		 */
		class NumericAllele: public AlleleBase
		{
		public:

			/**
			 * Constructor which values represents number of tandem repeats
			 *
			 * @param allelValue Allele value - number of tandem repeats
			 * @param alleleSize Allele size - number of base pairs
			 * @param peakHeight Peak height given in RFU
			 * @param peakArea Peak area given in RFU
			 * @return
			 */
			NumericAllele(double allelValue, double alleleSize = 0.0, double peakHeight = 0.0, double peakArea = 0.0);

			virtual ~NumericAllele();

			/**
			 * @see AlleleBase::cloneToSmartPtr()
			 */
			AlleleBasePtr cloneToPtr() const;

			/**
			 * @see AlleleBase::getAlleleValueAsString()
			 */
			std::string getAlleleValueAsString() const;

			/**
			 * @see AlleleBase::toString()
			 */
			std::string toString() const;
		};
	}
}

#endif /*NUMERICALLELE_H_*/
