#ifndef PERSON_H_
#define PERSON_H_

#include <stdexcept>
#include <map>
#include <string>
#include <boost/shared_ptr.hpp>
#include "Profile.h"

using namespace boost;

namespace dma
{
	namespace core
	{

		class Person;
		typedef shared_ptr<Person> PersonPtr;

		/**
		 *	Person potentially involved in mixture explanation
		 */
		class Person
		{
		public:
			/**
			 * Type of person
			 *
			 * 	- Suspect - Person is considered as suspect in a trial
			 * 	- Victim - Person is a victim
			 *  - Contaminator - Person, which DNA information should not be given in mixture
			 * 	- Role of the person in mixture is not yet given
			 */
			enum TYPE
			{
				SUSPECT, VICTIM, CONTAMINATOR, UNKNOWN
			};
		private:

			/**
			 * Persons identifier
			 */
			std::string personId_;

			/**
			 * Type of a person
			 */
			TYPE type_;

			/**
			 * Persons genotype
			 */
			ProfilePtr profile_;

		public:

			/**
			 * Constructor
			 *
			 * @param personId Identifier of the person
			 * @param type Role of the person in mixture
			 * @return
			 */
			Person(std::string personId, TYPE type);

			/**
			 * Constructor
			 *
			 * @param personId Identifier of the person
			 * @param type Role of the person in mixture
			 * @return
			 */
			Person(std::string personId, TYPE type, ProfilePtr profile);

			virtual ~Person();

			/**
			 *
			 * @return Method returns role the person in mixture
			 */
			TYPE getType() const;

			/**
			 *
			 * @return Method returns person's identifier
			 */
			std::string getPersonId() const;

			/**s
			 * Get profile associated with given person
			 *
			 * @return
			 */
			ProfilePtr getProfile() const;

			/**
			 * Method returns information whether given person has profile associated with him
			 * @return
			 */
			bool hasProfile() const;
		};
	}
}
#endif /*PERSON_H_*/
