/*
 * Profile.cpp
 *
 *  Created on: Dec 11, 2008
 *      Author: mariusz
 */

#include "Profile.h"

namespace dma
{
	namespace core
	{

		Profile::Profile() :
			markers_(new LocusPtrMap())
		{

		}

		Profile::~Profile()
		{

		}

		Profile::Profile(const Profile& copy)
		{

			// Reseting container
			markers_.reset(new LocusPtrMap());

			// Getting marker names on other side
			shared_ptr<std::vector<std::string> > markerNames = copy.getMarkerNames();
			std::vector<std::string>::const_iterator it1;
			for (it1 = markerNames->begin(); it1 != markerNames->end(); it1++)
			{

				markers_->insert(std::make_pair(*it1, copy.getMarker(*it1)));
			}
		}

		Profile& Profile::operator =(const Profile& rhs)
		{

			if (this != &rhs)
			{

				// Reseting container
				markers_.reset(new LocusPtrMap());

				// Getting marker names on other side
				shared_ptr<std::vector<std::string> > markerNames = rhs.getMarkerNames();
				std::vector<std::string>::const_iterator it1;
				for (it1 = markerNames->begin(); it1 != markerNames->end(); it1++)
				{

					markers_->insert(std::make_pair(*it1, rhs.getMarker(*it1)));
				}

			}

			return *this;
		}

		shared_ptr<std::vector<std::string> > Profile::getMarkerNames() const
		{

			shared_ptr<std::vector<std::string> > markerNames(new std::vector<std::string>());

			LocusPtrMap::const_iterator it1;
			for (it1 = markers_->begin(); it1 != markers_->end(); it1++)
			{

				markerNames->push_back(it1->first);
			}

			return markerNames;
		}

		LocusPtr Profile::getMarker(std::string markerName) const
		{

			LocusPtrMap::iterator it1 = markers_->find(markerName);

			if (it1 != markers_->end())
			{

				return it1->second;
			}
			else
			{

				throw MarkerNotFoundException(markerName, "Profile::getMarker()");
			}
		}

		void Profile::addMarker(std::string markerName, LocusPtr locus)
		{

			LocusPtrMap::iterator it1 = markers_->find(markerName);

			if (it1 != markers_->end())
			{
				// Marker already exists within profile. Deleting old marker
				markers_->erase(it1);
			}

			// Setting marker name for locus
			locus->setMarkerName(markerName);

			// Adding marker to container
			markers_->insert(std::make_pair(markerName, locus));
		}
	}
}
