/*
 * Profile.h
 *
 *  Created on: Dec 11, 2008
 *      Author: mariusz
 */

#ifndef PROFILE_H_
#define PROFILE_H_

#include "../models/Locus.h"
#include "../exceptions/MarkerNotFoundException.h"
#include <vector>
#include <boost/shared_ptr.hpp>

namespace dma
{
	namespace core
	{

		class Profile;

		/**
		 * Smart pointer for profile class
		 */
		typedef shared_ptr<Profile> ProfilePtr;

		/**
		 * Map of objects of class Profile
		 */
		typedef std::map<std::string, ProfilePtr> ProfilePtrMap;

		/**
		 * Smart pointer for map of objects of class Profile
		 */
		typedef shared_ptr<ProfilePtrMap> ProfilePtrMapPtr;



		/**
		 * Class encapsulates information regarding single DNA profile - alleles on each of markers
		 */
		class Profile
		{
		private:

			/**
			 * Markers/Loci found within profile
			 */
			LocusPtrMapPtr markers_;

		public:

			/**
			 * Constructor
			 *
			 * @return
			 */
			Profile();

			/**
			 * Copy constructor
			 * @param copy copied object
			 * @return
			 */
			Profile(const Profile& copy);

			/**
			 * Assignment operator overloading method
			 *
			 * @param rhs
			 * @return
			 */
			Profile& operator =(const Profile& rhs);

			virtual ~Profile();

			/**
			 * Get names of markers present in profile
			 * @return
			 */
			shared_ptr<std::vector<std::string> > getMarkerNames() const;

			/**
			 * Get marker by name. If marker is not found @class MarkerNotFoundException exception is generated
			 *
			 * @param name
			 * @return
			 * @exception MarkerNotFoundException
			 */
			LocusPtr getMarker(std::string markerName) const;

			/**
			 * Method add marker to profile. If marker already existed within profile the old one is substituted by new one
			 *
			 * @param markerName
			 * @param locus
			 */
			void addMarker(std::string markerName, LocusPtr locus);

		};
	}
}

#endif /* PROFILE_H_ */
