#include "Sample.h"

namespace dma
{
	namespace core
	{

		Sample::Sample(std::string name, std::string panel) :
			name_(name), panel_(panel), markers_(new std::map<std::string, MarkerPtr>())
		{

		}

		Sample::Sample(const Sample& copy)
		{

			// Copying class attributes
			name_ = copy.name_;
			panel_ = copy.panel_;

			// Deleting old data
			markers_.reset(new std::map<std::string, MarkerPtr>());

			// Copying markers
			std::map<std::string, MarkerPtr>::iterator it1;
			for (it1 = copy.markers_->begin(); it1 != copy.markers_->end(); it1++)
			{
				markers_->insert(make_pair(it1->first, it1->second));
			}
		}

		Sample& Sample::operator=(const Sample& rhs)
		{

			if (this != &rhs)
			{

				// Copying class attributes
				name_ = rhs.name_;
				panel_ = rhs.panel_;

				// Deleting old data
				markers_.reset(new std::map<std::string, MarkerPtr>());

				// Copying markers
				std::map<std::string, MarkerPtr>::iterator it1;
				for (it1 = rhs.markers_->begin(); it1 != rhs.markers_->end(); it1++)
				{
					markers_->insert(make_pair(it1->first, it1->second));
				}
			}

			return *this;
		}

		Sample::~Sample()
		{
			//std::cout << "Deleting sample " << this->name_ << std::endl;
		}

		std::string Sample::getName() const
		{
			return name_;
		}

		std::string Sample::getPanel() const
		{
			return panel_;
		}

		unsigned int Sample::getMarkersNumber() const
		{
			return markers_->size();
		}

		void Sample::addMarker(MarkerPtr marker)
		{

			// If marker has already existed - deleting the old one
			std::map<std::string, MarkerPtr>::iterator it = markers_->find(marker->getName());
			if (it != markers_->end())
			{

				markers_->erase(it);
			}

			markers_->insert(make_pair(marker->getName(), marker));
		}

		MarkerPtr Sample::getMarker(const std::string markerName) const
		{

			if (markers_->count(markerName) > 0)
			{

				return markers_->find(markerName)->second;
			}
			else
			{

				throw MarkerNotFoundException(markerName, "Sample::getMarker()");
			}
		}

		shared_ptr<std::vector<std::string> > Sample::getMarkerNames() const
		{

			shared_ptr<std::vector<std::string> > output(new std::vector<std::string>());

			std::map<std::string, MarkerPtr>::iterator it1;
			for (it1 = markers_->begin(); it1 != markers_->end(); it1++)
			{
				output->push_back(it1->first);
			}

			return output;
		}

		std::string Sample::toString() const
		{

			std::string buffer = "Sample " + getName() + ":\n\n";

			unsigned int i;
			unsigned int size = getMarkerNames()->size();
			for (i = 0; i < size; i++)
			{

				buffer = buffer + "--------------------------------------------------------------------\n";
				buffer = buffer + getMarker(getMarkerNames()->at(i))->toString() + "\n";
			}

			return buffer;
		}
	}
}
