/*
 * SampleConfiguration.cpp
 *
 *  Created on: Nov 11, 2008
 *      Author: mariusz
 */

#include "SampleConfiguration.h"

namespace dma
{

	namespace core
	{

		SampleConfiguration::SampleConfiguration(std::string name) :
			markers_(new std::map<std::string, MarkerConfigurationPtrVectorPtr>()), name_(name)
		{

		}

		SampleConfiguration::~SampleConfiguration()
		{

		}

		SampleConfiguration::SampleConfiguration(const SampleConfiguration& copy)

		{

			// Setting basic attributes
			name_ = copy.getName();

			// Deleting old data
			markers_.reset( (new std::map<std::string, MarkerConfigurationPtrVectorPtr >()) );

			// Copying markers
			std::map<std::string, MarkerConfigurationPtrVectorPtr >::iterator it1;
			for (it1 = copy.markers_->begin(); it1 != copy.markers_->end(); it1++)
			{
				markers_->insert(make_pair(it1->first, it1->second));
			}
		}

		SampleConfiguration& SampleConfiguration::operator=(const SampleConfiguration& rhs)
		{
			if (this != &rhs)
			{

				// Setting basic attributes
				name_ = rhs.getName();

				// Deleting old data
				markers_.reset( (new std::map<std::string, MarkerConfigurationPtrVectorPtr >()) );

				// Copying markers
				std::map<std::string, shared_ptr<std::vector<MarkerConfigurationPtr> > >::iterator it1;
				for (it1 = rhs.markers_->begin(); it1 != rhs.markers_->end(); it1++)
				{
					markers_->insert(make_pair(it1->first, it1->second));
				}
			}

			return *this;
		}

		SampleConfigurationPtr SampleConfiguration::cloneToSmartPtr() const
		{

			SampleConfigurationPtr clone(new SampleConfiguration(getName()));

			// Cloning marker configurations
			shared_ptr<std::vector<std::string> > markerNames = getMarkerNames();
			std::vector<std::string>::iterator it1;
			for (it1 = markerNames->begin(); it1 != markerNames->end(); it1++)
			{

				shared_ptr<std::vector<MarkerConfigurationPtr> > markerConfigurations = getMarkerConfigurations(*it1);
				std::vector<MarkerConfigurationPtr>::iterator it2;

				for (it2 = markerConfigurations->begin(); it2 != markerConfigurations->end(); it2++)
				{
					clone->addMarkerConfiguration(*it1, (*it2)->cloneToSmartPtr());
				}
			}

			return clone;
		}

		void SampleConfiguration::addMarkerConfiguration(std::string markerName, MarkerConfigurationPtr markerConfiguration)
		{

			// Checking whether marker with given name exists - if not creating one
			std::map<std::string, MarkerConfigurationPtrVectorPtr>::iterator it1 = markers_->find(markerName);

			if (it1 == markers_->end())
			{

				// Creating container for configurations and adding to marker container
				MarkerConfigurationPtrVectorPtr configurationsContainer(new MarkerConfigurationPtrVector());
				markers_->insert(make_pair(markerName, configurationsContainer));

				// Performing search one again - result should be returned
				it1 = markers_->find(markerName);
			}

			if (it1 != markers_->end())
			{

				// Container for marker found - adding marker
				it1->second->push_back(markerConfiguration);
			}
			else
			{

				// Container for marker not found - generating exceptions
				throw MarkerNotFoundException(markerName, "SampleConfiguration::addMarkerConfiguration()");
			}
		}

		void SampleConfiguration::addMarkerConfigurations(std::string markerName,
				MarkerConfigurationPtrVectorPtr markerConfigurations)
		{

			MarkerConfigurationPtrVector::const_iterator it1;

			// Adding configuration one by one
			for (it1 = markerConfigurations->begin(); it1 != markerConfigurations->end(); it1++)
			{

				addMarkerConfiguration(markerName, *it1);
			}
		}

		MarkerConfigurationPtrVectorPtr SampleConfiguration::getMarkerConfigurations(std::string markerName) const
		{

			std::map<std::string, MarkerConfigurationPtrVectorPtr>::const_iterator it1 = markers_->find(markerName);

			if (it1 != markers_->end())
			{

				return it1->second;
			}
			else
			{

				throw MarkerNotFoundException(markerName, "SampleConfiguration::getMarkerConfigurations()");
			}
		}

		shared_ptr<std::vector<std::string> > SampleConfiguration::getMarkerNames() const
		{

			// Creating output vector
			shared_ptr<std::vector<std::string> > output(new std::vector<std::string>());

			// Copying marker names
			std::map<std::string, shared_ptr<std::vector<MarkerConfigurationPtr> > >::const_iterator it1;
			for (it1 = markers_->begin(); it1 != markers_->end(); it1++)
			{
				output->push_back(it1->first);
			}

			return output;
		}

		unsigned int SampleConfiguration::getMaximumLociNumber(bool countValidOnly) const
		{

			unsigned int maximumLociNumber = 0;
			unsigned int lociNumber = 0;
			MarkerConfigurationPtrVectorPtr markerConfigurations;

			// Getting marker names
			shared_ptr<std::vector<std::string> > markerNames = getMarkerNames();

			// Iterating through all markers
			std::vector<std::string>::const_iterator it1;
			for (it1 = markerNames->begin(); it1 != markerNames->end(); it1++)
			{
				markerConfigurations = getMarkerConfigurations(*it1);

				// Getting maximum number of loci in marker configuration
				std::vector<MarkerConfigurationPtr>::iterator it2;
				for (it2 = markerConfigurations->begin(); it2 != markerConfigurations->end(); it2++)
				{

					MarkerConfigurationPtr mcObj = *it2;

					// If valid only configurations are counted checking whether configuration is valid
					if (countValidOnly == true)
					{

						if (mcObj->isValid() == false)
						{
							continue;
						}
					}

					// Getting number of loci ( contributors ) present in configuration
					lociNumber = mcObj->getLociNumber();
					if (lociNumber > maximumLociNumber)
					{
						maximumLociNumber = lociNumber;
					}
				}
			}

			return maximumLociNumber;
		}

		std::string SampleConfiguration::getName() const
		{

			return name_;
		}

		void SampleConfiguration::setName(std::string name)
		{

			name_ = name;
		}

		std::string SampleConfiguration::toString() const
		{

			// Getting namesa of markers which are present in sample
			shared_ptr<std::vector<std::string> > markerNames = getMarkerNames();

			std::string buffer = "";
			unsigned int i, j, size2;
			unsigned int size = markerNames->size();
			for (i = 0; i < size; i++)
			{

				buffer = buffer + "------------------------------------------------------------------\n";
				buffer = buffer + "MARKER " + markerNames->at(i) + "\n";

				// Displaying possible combinations
				size2 = getMarkerConfigurations(markerNames->at(i))->size();
				for (j = 0; j < size2; j++)
				{
					buffer = buffer + "Configuration " + lexical_cast<std::string> (j + 1) + ": "
							+ getMarkerConfigurations(markerNames->at(i))->at(j)->toString() + "\n";
				}
				buffer = buffer + "\n";
			}

			return buffer;
		}
	}
}
