/*
 * MCStatus.h
 *
 *  Created on: Jan 6, 2009
 *      Author: mariusz
 */

#ifndef MCSTATUS_H_
#define MCSTATUS_H_

#include <string>
#include <boost/shared_ptr.hpp>

using namespace boost;

namespace dma
{

	namespace core
	{

		class MCStatus;

		/**
		 * Smart pointer to object of class MCStatus
		 */
		typedef shared_ptr<MCStatus> MCStatusPtr;

		/**
		 * Abstract class encapsulating information regarding extended marker configuration status
		 */
		class MCStatus
		{
		private:

			/**
			 * Unique status name
			 */
			std::string statusName_;

			/**
			 * Flag indicating whether configuration is valid
			 */
			bool isValid_;

		public:

			/**
			 * Constructor
			 *
			 * @param statusName Short name of the status
			 * @param isValid Flag indicating whether configuration is valid
			 * @return
			 */
			MCStatus(std::string statusName, bool isValid);

			virtual ~MCStatus();

			/**
			 * Clone object instance to smart pointer
			 * @return
			 */
			virtual MCStatusPtr cloneToPtr() const = 0;

			/**
			 * Get name of the status. i.e. VALID, EXCLUDED_DUPLICATION
			 * @return
			 */
			std::string getName() const;

			/**
			 * Flag indicating whether configuration is valid
			 * @return
			 */
			bool isValid() const;

			/**
			 * Get full information regarding current configuration status as string
			 * @return
			 */
			virtual std::string toString() const = 0;


		};

	}
}

#endif /* MCSTATUS_H_ */
