/*
 * MCStatusExcAlleleMissing.h
 *
 *  Created on: Jan 6, 2009
 *      Author: mariusz
 */

#ifndef MCSTATUSEXCALLELEMISSING_H_
#define MCSTATUSEXCALLELEMISSING_H_

#include <string>
#include "MCStatus.h"
#include "../AlleleBase.h"
#include "../NumericAllele.h"

using namespace boost;

namespace dma
{

	namespace core
	{

		class MCStatusExcAlleleMissing;

		/**
		 * Smart pointer to object of class MCStatusExcAlleleMissing
		 */
		typedef shared_ptr<MCStatusExcAlleleMissing> MCStatusExcAlleleMissingPtr;

		/**
		 * Marker configuration status indicating that one of the alleles which is given in mixture sample is not given in configuration
		 */
		class MCStatusExcAlleleMissing: public MCStatus
		{
		private:

			/**
			 * Allele from mixture sample which is missing in marker configuration
			 */
			AlleleBasePtr alleleMissing_;

		public:

			/**
			 * Allele is initalized with value 0
			 * @return
			 */
			MCStatusExcAlleleMissing();

			/**
			 *
			 * @param missingAllele Allele which is given in mixture sample, but is not given in configuration
			 * @return
			 */
			MCStatusExcAlleleMissing(AlleleBasePtr alleleMissing);

			virtual ~MCStatusExcAlleleMissing();

			/**
			 * Retrieve mising allele
			 * @return
			 */
			AlleleBasePtr getAllele() const;

			/**
			 * @see MCStatus::cloneToPtr()
			 * @return
			 */
			MCStatusPtr cloneToPtr() const;

			/**
			 * @see MCStatus::toString()
			 * @return
			 */
			std::string toString() const;

			/**
			 * Get textual name of status
			 * @return
			 */
			static std::string getStatusName();
		};

	}
}

#endif /* MCSTATUSEXCALLELEMISSING_H_ */
