/*
 * MCStatusExcHeterozygoticImbalance.h
 *
 *  Created on: Jan 6, 2009
 *      Author: mariusz
 */

#ifndef MCSTATUSEXCHETEROZYGOTICIMBALANCE_H_
#define MCSTATUSEXCHETEROZYGOTICIMBALANCE_H_

#include <string>
#include "MCStatus.h"
#include "../Locus.h"

using namespace boost;

namespace dma
{

	namespace core
	{

		class MCStatusExcHeterozygoticImbalance;

		/**
		 * Smart pointer to object of class MCStatusExcHeterozygoticImbalance
		 */
		typedef shared_ptr<MCStatusExcHeterozygoticImbalance> MCStatusExcHeterozygoticImbalancePtr;

		/**
		 * Status indicating that heterozygotic balance is not met for given locus
		 */
		class MCStatusExcHeterozygoticImbalance: public MCStatus
		{
		private:

			/**
			 * Locus for which heterozygotic balance is not met
			 */
			LocusPtr locus_;

			/**
			 * Value of heterozygotic balance for given locus
			 */
			double hb_;

		public:

			/**
			 * Empty constructor. Locus in initialized with two numerical alleles of value {0,0} and HB = 0
			 *
			 * @return
			 */
			MCStatusExcHeterozygoticImbalance();

			/**
			 *
			 * @param locus Locus for which heterozygotic balance is not met
			 * @param hb Value of heterozygotic balance
			 * @return
			 */
			MCStatusExcHeterozygoticImbalance(LocusPtr locus, double hb);

			virtual ~MCStatusExcHeterozygoticImbalance();

			/**
			 * @see MCStatus::cloneToPtr()
			 * @return
			 */
			MCStatusPtr cloneToPtr() const;

			/**
			 * Get locus for which heterozygotic balance is not met
			 * @return
			 */
			LocusPtr getLocus() const;

			/**
			 * Get value of heterozygotic balance
			 * @return
			 */
			double getHb() const;

			/**
			 * @see MCStatus::toString()
			 * @return
			 */
			std::string toString() const;

			/**
			 * Get textual name of status
			 * @return
			 */
			static std::string getStatusName();
		};

	}
}

#endif /* MCSTATUSEXCHETEROZYGOTICIMBALANCE_H_ */
