/*
 * MCStatusExcKnownProfile.h
 *
 *  Created on: Jan 6, 2009
 *      Author: mariusz
 */

#ifndef MCSTATUSEXCKNOWNPROFILE_H_
#define MCSTATUSEXCKNOWNPROFILE_H_

#include <string>
#include "MCStatus.h"
#include "../Locus.h"

using namespace boost;

namespace dma
{

	namespace core
	{

		class MCStatusExcKnownProfile;

		/**
		 * Smart pointer to object of class MCStatusExcKnownProfile
		 */
		typedef shared_ptr<MCStatusExcKnownProfile> MCStatusExcKnownProfilePtr;

		/**
		 * Status indicating that configuration is excluded due to fact that locus from known known profile is not given in configuration
		 */
		class MCStatusExcKnownProfile: public MCStatus
		{
		private:

			/**
			 * Missing locus from known profile
			 */
			LocusPtr missingLocus_;

		public:

			/**
			 * Constructor
			 *
			 * @return
			 */
			MCStatusExcKnownProfile();

			/**
			 * Copy constructor
			 *
			 * @param missingLocus
			 * @return
			 */
			MCStatusExcKnownProfile(LocusPtr missingLocus);

			virtual ~MCStatusExcKnownProfile();

			/**
			 * @see MCStatus::cloneToPtr()
			 * @return
			 */
			MCStatusPtr cloneToPtr() const;

			/**
			 * Get missing locus
			 * @return
			 */
			LocusPtr getMissingLocus() const;

			/**
			 * @see MCStatus::toString()
			 * @return
			 */
			std::string toString() const;

			/**
			 * Get textual name of status
			 * @return
			 */
			static std::string getStatusName();
		};

	}
}

#endif /* MCSTATUSEXCKNOWNPROFILE_H_ */
