/*
 * MCStatusExcMixtureProportion.cpp
 *
 *  Created on: Jun 23, 2009
 *      Author: mariusz
 */

#include "MCStatusExcMixtureProportion.h"

namespace dma
{

	namespace core
	{

		MCStatusExcMixtureProportion::MCStatusExcMixtureProportion() :
			MCStatus( MCStatusExcMixtureProportion::getStatusName(), false), expectedMP_(0), actualMP_(0)
		{

			NumericAllelePtr na0(new NumericAllele(0));
			locus_.reset(new Locus(na0->cloneToPtr(), na0->cloneToPtr()));
		}

		MCStatusExcMixtureProportion::MCStatusExcMixtureProportion(LocusPtr locus, double expectedMP, double actualMP) :
			MCStatus(MCStatusExcMixtureProportion::getStatusName(), false), locus_(locus), expectedMP_(expectedMP), actualMP_(actualMP)
		{

		}

		MCStatusExcMixtureProportion::~MCStatusExcMixtureProportion()
		{

		}

		MCStatusPtr MCStatusExcMixtureProportion::cloneToPtr() const
		{

			MCStatusExcMixtureProportionPtr output(new MCStatusExcMixtureProportion(locus_->cloneToSmartPtr(),
					expectedMP_,
					actualMP_));
			return output;
		}

		std::string MCStatusExcMixtureProportion::toString() const
		{

			std::string buffer =
					"Configuration is excluded since actual mixture proportion variation is greater for locus than expected one: ";
			buffer = buffer + locus_->toString() + ". ";
			buffer = buffer + "Expected MP: " + lexical_cast<std::string> (expectedMP_) + ".";
			buffer = buffer + "Actual MP: " + lexical_cast<std::string> (actualMP_);

			return buffer;
		}

		LocusPtr MCStatusExcMixtureProportion::getLocus() const
		{

			return locus_;
		}

		double MCStatusExcMixtureProportion::getExpectedMP() const
		{

			return expectedMP_;
		}

		double MCStatusExcMixtureProportion::getActualMP() const
		{

			return actualMP_;
		}

		std::string MCStatusExcMixtureProportion::getStatusName()
		{

			return "EXCLUDED_MIXTURE_PROPORTION";
		}
	}

}
