/*
 * MCStatusExcMixtureProportion.h
 *
 *  Created on: Jun 23, 2009
 *      Author: mariusz
 */

#ifndef MCSTATUSEXCMIXTUREPROPORTION_H_
#define MCSTATUSEXCMIXTUREPROPORTION_H_

#include <string>
#include "MCStatus.h"
#include "../Locus.h"

namespace dma
{

	namespace core
	{

		class MCStatusExcMixtureProportion;

		typedef shared_ptr<MCStatusExcMixtureProportion> MCStatusExcMixtureProportionPtr;

		class MCStatusExcMixtureProportion: public MCStatus
		{
		private:

			/**
			 * Locus for which mixture proportion is incorrect
			 *
			 */
			LocusPtr locus_;

			/**
			 * Expected mixture proportion
			 */
			double expectedMP_;

			/**
			 * Actual mixture proportion
			 */
			double actualMP_;

		public:

			MCStatusExcMixtureProportion();

			MCStatusExcMixtureProportion(LocusPtr locus, double expectedMP, double actualMP);

			virtual ~MCStatusExcMixtureProportion();

			/**
			 * @see MCStatus::cloneToPtr()
			 * @return
			 */
			MCStatusPtr cloneToPtr() const;

			/**
			 * @see MCStatus::toString()
			 * @return
			 */
			std::string toString() const;

			/**
			 * Get locus for which mixture proportion is not preserved
			 * @return
			 */
			LocusPtr getLocus() const;

			/**
			 * Get expected mixture proportion for given locus
			 * @return
			 */
			double getExpectedMP() const;

			/**
			 * Get actual mixture proportion for given locus
			 * @return
			 */
			double getActualMP() const;

			/**
			 * Get textual name of status
			 * @return
			 */
			static std::string getStatusName();
		};

	}

}

#endif /* MCSTATUSEXCMIXTUREPROPORTION_H_ */
