/*
 * MixtureDataPrinter.cpp
 *
 *  Created on: Oct 28, 2008
 *      Author: mariusz
 */

#include "MixtureDataPrinters.h"

namespace dma
{

	namespace utils
	{

		std::string getTextMarkerFrequency(shared_ptr<std::vector<MarkerPtr> > data)
		{
			std::string buffer = "";

			unsigned int i;
			unsigned int size = data->size();
			MarkerPtr current;
			for (i = 0; i < size; i++)
			{
				current = data->at(i);
				buffer = buffer + getTextMarker(current);
			}

			return buffer;
		}

		std::string getTextAllele(AlleleBasePtr allele)
		{

			std::string buffer;
			std::string buffer2;

			if (allele->getType() == AlleleBase::CHARACTER)
			{

				CharacterAllelePtr cAllele = dynamic_pointer_cast<CharacterAllele> (allele);
				buffer = cAllele->toString() + "\n";
			}
			else if (allele->getType() == AlleleBase::NUMERIC)
			{

				NumericAllelePtr nAllele = dynamic_pointer_cast<NumericAllele> (allele);
				buffer = nAllele->toString() + "\n";
			}
			else
			{

				buffer = "U Allele\n";
			}

			return buffer;
		}

		std::string getTextMarker(MarkerPtr marker)
		{
			std::string buffer;

			buffer = "--- Marker name: " + marker->getName() + "\n";
			buffer = buffer + "Allel number: " + lexical_cast<std::string> (marker->getAllelesNumber()) + "\n";
			unsigned int i;
			for (i = 0; i < marker->getAllelesNumber(); i++)
			{
				buffer = buffer + getTextAllele(marker->getAllele(i));
			}

			return buffer;
		}

		std::string getTextSample(SamplePtr sample)
		{
			std::string buffer;

			buffer = "-----------------------------------------------\n";
			buffer = buffer + "Sample name: " + sample->getName() + "\n";
			buffer = buffer + "Panel name: " + sample->getPanel() + "\n";
			buffer = buffer + "Number of markers: " + lexical_cast<std::string> (sample->getMarkersNumber()) + "\n";
			unsigned int i;
			for (i = 0; i < sample->getMarkerNames()->size(); i++)
			{
				buffer = buffer + getTextMarker(sample->getMarker(sample->getMarkerNames()->at(i))) + "\n";
			}

			return buffer;
		}

		std::string getTextSamples(shared_ptr<std::vector<SamplePtr> > data)
		{
			std::string buffer = "***********************			DATA DUMP			*********************************";

			buffer = "Number of samples found: " + lexical_cast<std::string> (data->size()) + "\n";
			unsigned int i;
			for (i = 0; i < data->size(); i++)
			{
				buffer = buffer + getTextSample(data->at(i));
			}

			buffer = buffer + "*********************************************************************************";
			return buffer;
		}

		void printMarkerFrequency(shared_ptr<std::vector<MarkerPtr> > data)
		{
			std::cout << getTextMarkerFrequency(data) << std::endl;
		}

		void printAllel(AlleleBasePtr allele)
		{
			std::cout << getTextAllele(allele) << std::endl;
		}

		void printMarker(MarkerPtr marker)
		{
			std::cout << getTextMarker(marker) << std::endl;
		}

		void printSample(SamplePtr sample)
		{
			std::cout << getTextSample(sample) << std::endl;
		}

		void printSamplesData(shared_ptr<std::vector<SamplePtr> > data)
		{
			std::cout << getTextSamples(data) << std::endl;
		}

		void printMarkerConfiguration(MarkerConfigurationPtr configuration)
		{
			std::cout << configuration->toString() << std::endl;
		}

		void printMarkerConfigurations(MarkerConfigurationPtrVectorPtr configurations)
		{

			unsigned int i, size1;
			size1 = configurations->size();
			for (i = 0; i < size1; i++)
			{
				std::cout << "Configuration  " << (i + 1) << "\t";
				printMarkerConfiguration(configurations->at(i));
			}
		}
	}
}
