/*
 * DeconvolutionController.cpp
 *
 *  Created on: Jul 30, 2009
 *      Author: mariusz
 */

#include "DeconvolutionController.h"

namespace dma
{

	namespace external
	{

		DeconvolutionController::DeconvolutionController() :
			externalDataParser_(new ExternalDataParser())
		{

		}

		DeconvolutionController::~DeconvolutionController()
		{

		}

		SampleConfigurationPtr DeconvolutionController::doPerform(std::string mdFilename,
				ExternalDataParser::MIXTURE_DATA_FILE_FORMAT mixtureDataFileFormat,
				std::string hbdFilename,
				MixProportionPtrMapPtr mixturePropotions,
				double maxMixtureProportionDev) throw (std::runtime_error, ParserException)
		{

			// Performing external data parsing
			SamplePtr mixtureSample = externalDataParser_->parseMixtureDataFile(mdFilename, mixtureDataFileFormat);
			HbMapPtr hbMin = externalDataParser_->parseHbDataFile(hbdFilename);

			// Creating mixture analyzer
			MixAnalyzerPtr mixAnalyzer(new MixAnalyzer(hbMin));

			// Finding maximum number of persons which have been specified in mixture proportions
			MixProportionPtrMap::const_iterator mpIterator;
			unsigned int maxPersonNumber = 0;
			for (mpIterator = mixturePropotions->begin(); mpIterator != mixturePropotions->end(); mpIterator++)
			{

				if ((*mpIterator).first > maxPersonNumber)
				{
					maxPersonNumber = (*mpIterator).first;
				}
			}

			// Performing deconvolution using mixture analyzer
			SampleConfigurationPtr configurations = mixAnalyzer->performDeconvolution(mixtureSample,
					mixturePropotions,
					maxMixtureProportionDev,
					maxPersonNumber,
					MixAnalyzer::RESTRICTED);

			return configurations;
		}

	}

}
