/*
 * DeconvolutionController.h
 *
 *  Created on: Jul 30, 2009
 *      Author: mariusz
 */

#ifndef DECONVOLUTIONCONTROLLER_H_
#define DECONVOLUTIONCONTROLLER_H_

#include <string>
#include <stdexcept>
#include "boost/shared_ptr.hpp"
#include "analysis/MixAnalyzer.h"
#include "models/SampleConfiguration.h"
#include "models/MixtureProportion.h"
#include "../parsers/ExternalDataParser.h"

using namespace dma::core;

namespace dma
{

	namespace external
	{

		class DeconvolutionController;
		typedef boost::shared_ptr<DeconvolutionController> DeconvolutionControllerPtr;

		/**
		 * Class responsible for realization of deconvolution based on external data files
		 */
		class DeconvolutionController
		{
		private:

			/**
			 * Object responsible for parsing external data
			 */
			ExternalDataParserPtr externalDataParser_;

		public:
			DeconvolutionController();

			virtual ~DeconvolutionController();

			/**
			 * Do perform mixture deconvolution basing of data given in data files and mixture proportion.
			 * Method returns set of configurations with assigned status regarding their validity
			 *
			 * @param mdFilename			Name of file containing mixture sample
			 * @param mixtureDataFileFormat Type of parsed file
			 * @param hbdFilename			Name of file containing heterozygotic balance minimal values
			 * @param mixturePropotions		Mixture proportions
			 * @return
			 * @exception std::runtime_error
			 * @exception ParserException
			 */
			SampleConfigurationPtr doPerform(std::string mdFilename,
					ExternalDataParser::MIXTURE_DATA_FILE_FORMAT mixtureDataFileFormat,
					std::string hbdFilename,
					MixProportionPtrMapPtr mixturePropotions,
					double maxMixtureProportionDev) throw( std::runtime_error, ParserException);


		};

	}

}

#endif /* DECONVOLUTIONCONTROLLER_H_ */
