/*
 * LrCalculationController.cpp
 *
 *  Created on: Jul 30, 2009
 *      Author: mariusz
 */

#include "LrCalculationController.h"

namespace dma
{

	namespace external
	{

		LrCalculationController::LrCalculationController() :
			externalDataParser_(new ExternalDataParser())
		{

		}

		LrCalculationController::~LrCalculationController()
		{

		}

		LikelihoodRatioPtr LrCalculationController::doPerform(std::string reportFilename,
				std::string pdFilename,
				HypothesisPtr hp,
				HypothesisPtr hd) throw ( PopulationDataException, ParserException, std::runtime_error )
		{

			// Performing file parsing
			SampleConfigurationPtr configurations = externalDataParser_->parseOutputDataFile(reportFilename);
			MarkerPtrVectorPtr populationData = externalDataParser_->parsePopulationDataFile(pdFilename);

			// Performing verification of population data - all alleles from configurations are in population
			PopulationDataAnalyzer populationDataAnalyzer(populationData);
			MarkerPtrVectorPtr missingAlleles = populationDataAnalyzer.verifyPopulationData(configurations);
			if (missingAlleles->size() > 0)
			{
				throw PopulationDataException(missingAlleles);
			}

			// Performing LR calculation
			LrCalculatorPtr lrCalculator(new LrCalculator(populationData));
			LikelihoodRatioPtr lr = lrCalculator->calculateLr(configurations, hp, hd);

			return lr;
		}

	}

}
