/*
 * LrCalculationController.h
 *
 *  Created on: Jul 30, 2009
 *      Author: mariusz
 */

#ifndef LRCALCULATIONCONTROLLER_H_
#define LRCALCULATIONCONTROLLER_H_

#include "boost/shared_ptr.hpp"
#include "../parsers/ExternalDataParser.h"
#include "models/Hypothesis.h"
#include "analysis/LrCalculator.h"
#include "analysis/PopulationDataAnalyzer.h"
#include "exceptions/PopulationDataException.h"

namespace dma
{

	namespace external
	{

		class LrCalculationController;
		typedef boost::shared_ptr<LrCalculationController> LrCalculationControllerPtr;

		/**
		 * Class responsible for realization of LR calculations based on external data files
		 */
		class LrCalculationController
		{
		private:

			/**
			 * Object responsible for parsing external data
			 */
			ExternalDataParserPtr externalDataParser_;

		public:

			LrCalculationController();
			virtual ~LrCalculationController();

			/**
			 * Perform likelihood ratio calculation basing on data files and supplied hypothesis
			 *
			 * @param reportFilename File path of report
			 * @param pdFilename File path to population data
			 * @param hp Prosector's hypothesis
			 * @param hd Defence's hypothesis
			 * @return
			 * @exception PopulationDataException
			 * @exception ParserException
			 * @exception std::runtime_error
			 */
			LikelihoodRatioPtr doPerform(std::string reportFilename, std::string pdFilename, HypothesisPtr hp, HypothesisPtr hd)
					throw (PopulationDataException, ParserException, std::runtime_error);

		};
	}
}
#endif /* LRCALCULATIONCONTROLLER_H_ */
