#include "CsvFileReader.h"

namespace dma
{
	namespace external
	{

		CsvFileReader::CsvFileReader(std::string filename, std::string separator) :
			FileReader(filename), possibleSeparators_(new std::vector<std::string>()), separator_(separator)
		{
			// Creating possible separators list
			possibleSeparators_->push_back(",");
			possibleSeparators_->push_back(";");
			possibleSeparators_->push_back("\t");

			// Checking whether separator has been given
			if (separator.length() == 0)
			{

				// If no separator has been given determining it from file contents
				separator_ = determineSeparator();
			}

		}

		CsvFileReader::~CsvFileReader()
		{

		}

		void CsvFileReader::setSeparator(std::string separator)
		{
			separator_ = separator;
		}

		std::string CsvFileReader::getSeparator() const
		{

			return separator_;
		}

		std::string CsvFileReader::determineSeparator()
		{
			// Separator most probably used
			std::string separator = "";
			unsigned int counterCurrent = 0;
			unsigned int counterOld = 0;

			// Defining tokenizer
			typedef boost::tokenizer<boost::char_separator<char> > char_tokenizer;

			// Getting first line
			std::string* line = this->getLine();

			if (line != NULL)
			{

				// Iterating through possible separators
				std::vector<std::string>::iterator psIt;

				for (psIt = possibleSeparators_->begin(); psIt != possibleSeparators_->end(); psIt++)
				{
					// Reseting counter
					counterCurrent = 0;

					// Creating tokenizer
					boost::char_separator<char> sep((*psIt).c_str(), "", boost::keep_empty_tokens);
					char_tokenizer tokens(*line, sep);

					// Getting number of tokens found for given separator
					for (char_tokenizer::iterator tok_iter = tokens.begin(); tok_iter != tokens.end(); ++tok_iter)
					{
						counterCurrent++;
					}

					// Checking whether number of tokens found is greater than for the last time
					if (counterCurrent > counterOld)
					{

						counterOld = counterCurrent;
						separator = *psIt;
					}
				}
			}

			this->resetInputStream();

			return separator;
		}

		shared_ptr<std::vector<std::string> > CsvFileReader::getLineAndExplode()
		{

			std::string* buffer;
			shared_ptr<std::vector<std::string> > out(new std::vector<std::string>());

			// Defining tokenizer
			typedef boost::tokenizer<boost::char_separator<char> > char_tokenizer;

			// Reading line by line
			if ((buffer = this->getLine()) != NULL)
			{

				boost::char_separator<char> sep(separator_.c_str(), "", boost::keep_empty_tokens);
				char_tokenizer tokens(*buffer, sep);

				// Extracting data from line
				for (char_tokenizer::iterator tok_iter = tokens.begin(); tok_iter != tokens.end(); ++tok_iter)
				{
					out->push_back(*tok_iter);
				}
			}
			else
			{
				return out;
			}

			return out;
		}
	}
}
