#ifndef CSVFILEREADER_H_
#define CSVFILEREADER_H_

#include <string>
#include <iostream>
#include <fstream>
#include <vector>
#include <boost/tokenizer.hpp>
#include <boost/shared_ptr.hpp>
#include "FileNotFoundException.h"
#include "FileReader.h"
#include <sys/stat.h>

using namespace boost;

namespace dma
{

	namespace external
	{

		class CsvFileReader;
		typedef shared_ptr<CsvFileReader> CsvFileReaderPtr;

		/**
		 *	Class representing reader used for accessing CSV ( Comma-Separated-Values ) files
		 */
		class CsvFileReader: public FileReader
		{

		private:

			/**
			 * Possible separators list
			 */
			shared_ptr<std::vector<std::string> > possibleSeparators_;

			/**
			 * Data fields separator used
			 */
			std::string separator_;

			/**
			 * EOL line character
			 */
			char newlineCharacter_;

		public:

			/**
			 * Constructor
			 *
			 * @param filename Name of the field to-be-opened
			 * @param separator Data fields separator. If not given separator will be determined
			 * @return
			 * @exception FileNotFoundException
			 */
			CsvFileReader(std::string filename, std::string separator = "");

			virtual ~CsvFileReader();

			/**
			 * Set fields separator
			 * @param separator
			 */
			void setSeparator(std::string separator);

			/**
			 * Get separator used
			 * @return
			 */
			std::string getSeparator() const;

			/**
			 * Method reads single line from input file and basing on separator splits it into vector of strings.
			 * On EOL empty vector is returned
			 *
			 * @return
			 */
			shared_ptr<std::vector<std::string> > getLineAndExplode();

		protected:

			/**
			 * Method determines which separator is used is CSV file, basing on number of columns for the first line parsed with all possible separators
			 * The separator which returns maximum number of columns is returned
			 *
			 * @return
			 */
			std::string determineSeparator();

		};
	}
}

#endif /*CSVFILEREADER_H_*/
