#include "FileReader.h"
#include <iostream>

namespace dma
{
	namespace external
	{

		FileReader::FileReader(const std::string& filename)
		{

			if (filename.length() > 0 && FileReader::checkFileExists(filename))
			{

				// Setting filename
				filename_ = filename;

				// Opening file for reading
				fin_.open(filename_.c_str());

				// Determinig newline delimiter
				newLineDelimiter_ = determineNewLineDelimiter();
			}
			else
			{

				throw FileNotFoundException(filename, "");
			}
		}

		FileReader::~FileReader()
		{

			// Closing file
			fin_.close();
		}

		bool FileReader::checkFileExists(const std::string& filename)
		{

			struct stat stFileInfo;
			int intStat;

			// Attempt to get the file attributes
			intStat = stat(filename.c_str(), &stFileInfo);
			if (intStat == 0)
			{

				// If file attributes can be fetched file exists
				return true;
			}
			else
			{

				// File does not exists or permission is not granted to access file
				return false;
			}
		}

		char FileReader::determineNewLineDelimiter()
		{

			// Reading whole file
			int length;
			char * buffer;

			std::ifstream is;
			is.open(filename_.c_str(), std::ios::binary);

			// Getting length of file:
			is.seekg(0, std::ios::end);
			length = is.tellg();
			is.seekg(0, std::ios::beg);

			// Allocating memory for file content
			buffer = new char[length];
			is.read(buffer, length);
			is.close();

			// Converting character into string
			std::string strb(buffer);

			// Counting number of occurrences of different new line delimiter
			unsigned int crNumber = 0;
			unsigned int lfNumber = 0;
			unsigned int i = 0;
			unsigned int size = strb.size();
			for (i = 0; i < size; i++)
			{

				if (strb.at(i) == '\r')
				{

					++crNumber;
				}
				else if (strb.at(i) == '\n')
				{
					++lfNumber;
				}
			}

			// Determining appropriate new line delimiter
			char delim;
			if (lfNumber > crNumber)
			{

				delim = '\n';
			}
			else if (lfNumber == crNumber)
			{

				delim = '\n';
			}
			else
			{

				delim = '\r';
			}

			// Realeasing resources
			delete[] buffer;

			return delim;

		}

		std::string FileReader::getFilename() const
		{
			return filename_;
		}

		void FileReader::resetInputStream()
		{

			fin_.close();
			fin_.open(filename_.c_str());
		}

		std::string* FileReader::getLine()
		{

			std::string* output = NULL;
			std::string s;

			if (fin_)
			{

				if (getline(fin_, s, newLineDelimiter_) != NULL)
				{

					int lenght = s.length();

					// Omitting CR
					if (lenght > 0 && (int) (s.at(lenght - 1)) == 13)
					{
						s.resize(lenght - 1);
					}

					output = new std::string(s);
				}
			}

			return output;
		}
	}
}
