#ifndef FILEREADER_H_
#define FILEREADER_H_

#include <string.h>
#include <iostream>
#include <fstream>
#include <vector>
#include "FileNotFoundException.h"
#include <boost/shared_ptr.hpp>

#include <sys/stat.h>

namespace dma
{
	namespace external
	{

		class FileReader;
		typedef boost::shared_ptr<FileReader> FileReaderPtr;

		/**
		 * Class representing file reader
		 */
		class FileReader
		{
		private:

			/**
			 * Name of the file read
			 */
			std::string filename_;

			/**
			 * Input file stream
			 */
			std::ifstream fin_;

			/**
			 * New line character delimiter
			 */
			char newLineDelimiter_;

		public:

			/**
			 * Create file reader. If file is not found @class FileNotFoundException is generated
			 *
			 * @param filename Name of the file to be read
			 * @return
			 * @exception FileNotFoundException
			 */
			FileReader(const std::string& filename);

			virtual ~FileReader();

			/**
			 * Method checks whether file with given name exists prior to opening it
			 */
			static bool checkFileExists(const std::string& filename);

			/**
			 *
			 * @return Method returns name of the file opened for reading
			 */
			std::string getFilename() const;

			/**
			 * Method responsible for finding new line delimiter
			 * @return
			 */
			char determineNewLineDelimiter();

		protected:

			/**
			 * Reads single line from input. If EOF is uncounted NULL is returned
			 *
			 * @return
			 */
			std::string* getLine();

			/**
			 * Method resets input stream so that getline function returns first row in first attempt
			 */
			void resetInputStream();
		};
	}
}

#endif /*FILEREADER_H_*/
