/*
 * DataParser.h
 *
 *  Created on: Nov 30, 2008
 *      Author: mariusz
 */

#ifndef DATAPARSER_H_
#define DATAPARSER_H_

#include "../io/CsvFileReader.h"
#include "boost/shared_ptr.hpp"

using namespace boost;

namespace dma
{

	namespace external
	{

		class DataParser;
		typedef shared_ptr<DataParser> DataParserPtr;

		/**
		 * Class representing all data parsers
		 */
		class DataParser
		{
		protected:

			/**
			 * Constructor
			 *
			 * @param filename Name of the file with data
			 * @param fileType Name of file type being parsed
			 * @param separator Data fields separator. If not given separator will be determined
			 * @return
			 * @exception IOException
			 * @exception FileNotFoundException
			 */
			DataParser(std::string filename, std::string fileType, std::string separator = "");

			/**
			 * Method trims " characters from given string
			 *
			 * @param input
			 * @return
			 */
			std::string trimQuotation(const std::string& input) const;

			/**
			 * Method converts floating point number written with "," into those with dot
			 *
			 * @param input
			 * @return
			 */
			std::string convertFloatingPoints(const std::string& input) const;

			/**
			 * CSV file reader
			 */
			CsvFileReaderPtr reader_;

			/**
			 * Name of file type parsed
			 */
			std::string fileType_;

		public:

			virtual ~DataParser();

			/**
			 * Retrieve type of file parsed
			 * @return
			 */
			std::string getFileType() const;
		};

	}
}

#endif /* DATAPARSER_H_ */
