/*
 * ExternalDataParser.h
 *
 *  Created on: Jul 4, 2009
 *      Author: mariusz
 */

#ifndef EXTERNALDATAPARSER_H_
#define EXTERNALDATAPARSER_H_

#include <string>
#include "DataParser.h"
#include "HbDataParser.h"
#include "OutputDataParser.h"
#include "PopulationDataParser.h"
#include "ProfileDataParser.h"
#include "MixtureDataParserColumnarFormat.h"
#include "MixtureDataParserLinearFormat.h"
#include "ParserException.h"
#include "TypeDefinitions.h"
#include "boost/shared_ptr.hpp"

using namespace dma::core;

namespace dma
{

	namespace external
	{

		class ExternalDataParser;
		typedef shared_ptr<ExternalDataParser> ExternalDataParserPtr;

		/**
		 * Class used for encapsulating parser and providing further validation of data ( i.e. validation for empty files )
		 */
		class ExternalDataParser
		{
		public:

			/**
			 * Set of possible mixture file formats
			 */
			enum MIXTURE_DATA_FILE_FORMAT
			{
				COLUMNAR_FORMAT,//!< COLUMNAR_FORMAT
				LINEAR_FORMAT //!< LINEAR_FORMAT
			};
		public:

			ExternalDataParser();

			virtual ~ExternalDataParser();

			/**
			 * Method responsible for parsing mixture data file. Method may only throw exception of type ParserException in the following situations:
			 *
			 *  - Lexical error has been found during parsing
			 *  - No sample has been read from data file
			 *  - More than one sample has been read from data file
			 *
			 * @param filename Name of the file parsed
			 * @param mixtureDataFileFormat Type of parsed file
			 * @return
			 */
			SamplePtr parseMixtureDataFile(std::string filename, ExternalDataParser::MIXTURE_DATA_FILE_FORMAT mixtureDataFileFormat) throw (ParserException);

			/**
			 * Method responsible for parsing population data file. Method may only throw exception of type ParserException in the following situations:
			 *
			 *  - Lexical error has been found during parsing
			 *  - No population data has been found for any of markers
			 *
			 * @param filename
			 * @return
			 */
			MarkerPtrVectorPtr parsePopulationDataFile(std::string filename) throw (ParserException);

			/**
			 * Method responsible for parsing heterozygotic balance data file. Method may only throw exception of type ParserException in the following situations:
			 *
			 *  - Lexical error has been found during parsing
			 *  - No minimum heterozygotic balance data has been found for any of markers
			 *
			 * @param filename
			 * @return
			 */
			HbMapPtr parseHbDataFile(std::string filename) throw (ParserException);

			/**
			 * Method responsible for parsing deconvolution reportfile. Method may only throw exception of type ParserException in the following situations:
			 *
			 *  - Lexical error has been found during parsing
			 *  - No data has been found in file
			 *
			 * @param filename
			 * @return
			 */
			SampleConfigurationPtr parseOutputDataFile(std::string filename) throw (ParserException);

		};

	}

}

#endif /* EXTERNALDATAPARSER_H_ */
