/*
 * HbDataParser.cpp
 *
 *  Created on: Nov 8, 2008
 *      Author: mariusz
 */

#include "HbDataParser.h"

namespace dma
{

	namespace external
	{

		HbDataParser::HbDataParser(std::string filename, std::string separator) :
			DataParser(filename, HbDataParser::getFileType(), separator), hb_(new HbMap())
		{

		}

		HbDataParser::~HbDataParser()
		{

		}

		std::string HbDataParser::getFileType()
		{

			return "Heterozygotic balance date file";
		}

		HbMapPtr HbDataParser::parse() throw (ParserException)
		{
			int rowCounter = 0;
			shared_ptr<std::vector<std::string> > row;

			while ((row = reader_->getLineAndExplode()) && row->size() > 0)
			{

				// Checking whether empty row has not been encounted
				if (row->size() > 0)
				{
					// Checking whether first row contains valid data - exceptions is not throwned
					if (rowCounter == 0)
					{

						try
						{
							parseDataRow(row, rowCounter);
						} catch (ParserException)
						{
							// Omitting parse error in first row
						}
					}
					else
					{
						parseDataRow(row, rowCounter);
					}
				}

				++rowCounter;
			}

			return this->getHb();
		}

		void HbDataParser::parseDataRow(shared_ptr<std::vector<std::string> > row, unsigned int lineNumber)
				throw (ParserException)
		{

			// Checking whether row is of appropiate size
			if (row->size() == 2)
			{

				try
				{
					std::string markerName = row->at(0);
					std::string hbMinValue = convertFloatingPoints(row->at(1));
					double dHbMinValue = lexical_cast<double> (hbMinValue);

					hb_->insert(make_pair(markerName, dHbMinValue));
				} catch (bad_lexical_cast& e)
				{

					std::string errorMsg = "Unknown HB value - " + row->at(1);
					throw ParserException(errorMsg, fileType_, reader_->getFilename(), lineNumber + 1);
				}
			}
			else
			{

				throw ParserException("Row with HB data has inappropriate length", fileType_, reader_->getFilename(), lineNumber
						+ 1);
			}
		}

		HbMapPtr HbDataParser::getHb() const
		{

			return hb_;
		}

	}
}
