/*
 * HbDataParser.h
 *
 *  Created on: Nov 8, 2008
 *      Author: mariusz
 */

#ifndef HBDATAPARSER_H_
#define HBDATAPARSER_H_

#include <map>
#include <string>
#include "DataParser.h"
#include "ParserException.h"
#include <boost/shared_ptr.hpp>
#include <boost/lexical_cast.hpp>
#include "TypeDefinitions.h"

using namespace dma::core;

namespace dma
{

	namespace external
	{

		class HbDataParser;
		typedef shared_ptr<HbDataParser> HbDataParserPtr;

		/**
		 * Class responsible for parsing heterozygotic balance data.
		 */
		class HbDataParser: public DataParser
		{
		private:

			/**
			 * Heterozygotic value for all markers
			 */
			HbMapPtr hb_;

		public:

			/**
			 * Constructor
			 *
			 * @param filename Name of the file with data
			 * @param separator Data fields separator. If not given separator will be determined
			 * @return
			 * @exception IOException
			 * @exception FileNotFoundException
			 */
			HbDataParser(std::string filename, std::string separator = "");

			virtual ~HbDataParser();

			/**
			 * Get file type name
			 * @return
			 */
			static std::string getFileType();

			/**
			 * Parse datafile
			 *
			 * @exception ParserException
			 * @returns
			 */
			HbMapPtr parse() throw (ParserException);

			/**
			 * Method returns parsed data
			 * @return
			 */
			HbMapPtr getHb() const;

		private:

			/**
			 * Method responsible for parsing data row.
			 * @param row
			 * @param lineNumber
			 */
			void parseDataRow(shared_ptr<std::vector<std::string> > row, unsigned int lineNumber) throw (ParserException);

		};
	}
}
#endif /* HBDATAPARSER_H_ */
