#include "MixtureDataParserColumnarFormat.h"

namespace dma
{

	namespace external
	{

		MixtureDataParserColumnarFormat::MixtureDataParserColumnarFormat(std::string filename,
				std::string separator,
				unsigned int sampleColumnIndex,
				unsigned int panelColumnIndex,
				unsigned int markerColumnIndex,
				unsigned int firstDataColumnIndex) :
			MixtureDataParser(filename, separator), sampleColumnIndex_(sampleColumnIndex),
					panelColumnIndex_(panelColumnIndex), markerColumnIndex_(markerColumnIndex),
					firstDataColumnIndex_(firstDataColumnIndex)
		{

		}

		MixtureDataParserColumnarFormat::~MixtureDataParserColumnarFormat()
		{

		}

		void MixtureDataParserColumnarFormat::parseHeaderRow(shared_ptr<std::vector<std::string> > row)
		{

			// Header parsing omitted. No vital information given in header
		}

		void MixtureDataParserColumnarFormat::parseDataRow(shared_ptr<std::vector<std::string> > row, unsigned int lineNumber)
				throw (ParserException)
		{

			// Loop counter and column indexes
			unsigned int i, c1, c2, c3, c4;
			AlleleBasePtr allele;

			// Calculating number of data columns in row and allele number
			unsigned int dataColumnsNumber = row->size() - firstDataColumnIndex_;
			unsigned int allelesNumber = dataColumnsNumber / 4;

			// Parsing allele data
			if (dataColumnsNumber >= 4)
			{
				for (i = 0; i < allelesNumber; ++i)
				{
					// Calculating column indices
					c1 = firstDataColumnIndex_ + i + allelesNumber * DATA_ORDER_ALLEL_VALUE;
					c2 = firstDataColumnIndex_ + i + allelesNumber * DATA_ORDER_ALLELE_SIZE;
					c3 = firstDataColumnIndex_ + i + allelesNumber * DATA_ORDER_PEAK_HEIGHT;
					c4 = firstDataColumnIndex_ + i + allelesNumber * DATA_ORDER_PEAK_SIZE;

					// Parsing allele data
					allele = parseAllelData(row->at(c1), row->at(c2), row->at(c3), row->at(c4), lineNumber);

					// Checking whether allele data has been fetched
					if (allele->getAlleleValue() > 0)
					{

						addAllele(row->at(sampleColumnIndex_),
								row->at(panelColumnIndex_),
								row->at(markerColumnIndex_),
								allele,
								lineNumber);
					}
				}
			}
			else
			{

				// Number of data columns is to small to fit data regarding single allele - Generating error message and exception
				std::string errorMsg = "Number of column with data regarding alleles is to small ( expected 4 )";
				throw ParserException(errorMsg, fileType_, reader_->getFilename(), lineNumber + 1);
			}
		}

	}
}
