/*
 * MixtureDataParserLinearFormat.cpp
 *
 *  Created on: Dec 12, 2009
 *      Author: mariusz
 */

#include "MixtureDataParserLinearFormat.h"

namespace dma
{

	namespace external
	{

		MixtureDataParserLinearFormat::MixtureDataParserLinearFormat(std::string filename,
				std::string separator,
				unsigned int sampleColumnIndex,
				unsigned int markerColumnIndex,
				unsigned int allelValueColumnIndex,
				unsigned int allelSizeColumnIndex,
				unsigned int allelHeightColumnIndex,
				unsigned int allelAreaColumnIndex) :
			MixtureDataParser(filename, separator), sampleColumnIndex_(sampleColumnIndex), markerColumnIndex_(markerColumnIndex),
					allelValueColumnIndex_(allelValueColumnIndex), allelSizeColumnIndex_(allelSizeColumnIndex),
					allelHeightColumnIndex_(allelHeightColumnIndex), allelAreaColumnIndex_(allelAreaColumnIndex)
		{

		}

		MixtureDataParserLinearFormat::~MixtureDataParserLinearFormat()
		{

		}

		void MixtureDataParserLinearFormat::parseHeaderRow(shared_ptr<std::vector<std::string> > row)
		{

			// Header parsing omitted. No vital information given in header
		}

		void MixtureDataParserLinearFormat::parseDataRow(shared_ptr<std::vector<std::string> > row, unsigned int lineNumber)
				throw (ParserException)
		{

			// Searching for column index, with highest index
			unsigned int minNumberOfColumns = 0;
			if ( sampleColumnIndex_ > minNumberOfColumns ) minNumberOfColumns = sampleColumnIndex_;
			if ( markerColumnIndex_ > minNumberOfColumns ) minNumberOfColumns = markerColumnIndex_;
			if ( allelValueColumnIndex_ > minNumberOfColumns ) minNumberOfColumns = allelValueColumnIndex_;
			if ( allelSizeColumnIndex_ > minNumberOfColumns ) minNumberOfColumns = allelSizeColumnIndex_;
			if ( allelHeightColumnIndex_ > minNumberOfColumns ) minNumberOfColumns = allelHeightColumnIndex_;
			if ( allelAreaColumnIndex_ > minNumberOfColumns ) minNumberOfColumns = allelAreaColumnIndex_;
			// Incrementing as number of columns, as index is is 0-based
			minNumberOfColumns++;

			// Container for parsed allele
			AlleleBasePtr allele;



			// Parsing allele data
			if ( row->size() >= minNumberOfColumns )
			{

				// Fetching name of parsed sample
				std::string sampleName = row->at(sampleColumnIndex_);

				// Fetching name of marker
				std::string markerName = row->at(markerColumnIndex_);

				// Parsiing allele data
				allele = parseAllelData( row->at(allelValueColumnIndex_), row->at(allelSizeColumnIndex_), row->at (allelHeightColumnIndex_), row->at(allelAreaColumnIndex_), lineNumber );

				if (allele->getAlleleValue() > 0)
				{

					addAllele( sampleName, "", markerName, allele, lineNumber);
				}
			}
			else
			{

				// Number of data columns is to small to fit data regarding single allele - Generating error message and exception
				std::string errorMsg = "Number of column with data regarding alleles is to small ( expected " + lexical_cast<std::string>(minNumberOfColumns) + " )";
				throw ParserException(errorMsg, fileType_, reader_->getFilename(), lineNumber + 1);
			}
		}
	}

}
