/*
 * MixtureDataParserLinearFormat.h
 *
 *  Created on: Dec 12, 2009
 *      Author: mariusz
 */

#ifndef MIXTUREDATAPARSERLINEARFORMAT_H_
#define MIXTUREDATAPARSERLINEARFORMAT_H_

#include "MixtureDataParser.h"

namespace dma
{

	namespace external
	{

		class MixtureDataParserLinearFormat;
		typedef shared_ptr<MixtureDataParserLinearFormat> MixtureDataParserLinearFormatPtr;

		/**
		 * Class represents parsed used for processing mixture samples.
		 * Single lines contains information regarding single allele
		 */
		class MixtureDataParserLinearFormat: public MixtureDataParser
		{
		private:

			/**
			 *	Index of column with sample name
			 */
			unsigned int sampleColumnIndex_;

			/**
			 *	Index of column with marker name
			 */
			unsigned int markerColumnIndex_;

			/**
			 *	Index of column at which allele value is given
			 */
			unsigned int allelValueColumnIndex_;

			/**
			 *	Index of column at which allele size is given
			 */
			unsigned int allelSizeColumnIndex_;

			/**
			 *	Index of column at which allele height is given
			 */
			unsigned int allelHeightColumnIndex_;

			/**
			 *	Index of column at which allele area is given
			 */
			unsigned int allelAreaColumnIndex_;

			/**
			 *	Default column number for sample name
			 */
			static const unsigned int SAMPLE_COLUMN_INDEX_D = 0;

			/**
			 *	Default colum number for marker name
			 */
			static const unsigned int MARKER_COLUMN_INDEX_D = 1;

			/**
			 *	Default column number for allele value
			 */
			static const unsigned int ALLELE_VALUE_COLUMN_INDEX_D = 2;

			/**
			 *	Default column number for allele value
			 */
			static const unsigned int ALLELE_SIZE_COLUMN_INDEX_D = 3;

			/**
			 *	Default column number for allele value
			 */
			static const unsigned int ALLELE_HEIGHT_COLUMN_INDEX_D = 4;

			/**
			 *	Default column number for allele value
			 */
			static const unsigned int ALLELE_AREA_COLUMN_INDEX_D = 5;

		public:

			/**
			 *
			 * @param filename Name of the file to-be-parsed
			 * @param separator Data fields separator. If not given separator will be determined
			 * @param sampleColumnIndex Index of column with sample name
			 * @param markerColumnIndex Index of column with marker name
			 * @param allelValueColumnIndex Index of column with allele value
			 * @param allelSizeColumnIndex Index of column with alele size
			 * @param allelHeightColumnIndex Index of column with allele height
			 * @param allelAreaColumnIndex Index of column with allele area
			 * @return
			 */
			MixtureDataParserLinearFormat(std::string filename,
					std::string separator = "",
					unsigned int sampleColumnIndex = SAMPLE_COLUMN_INDEX_D,
					unsigned int markerColumnIndex = MARKER_COLUMN_INDEX_D,
					unsigned int allelValueColumnIndex = ALLELE_VALUE_COLUMN_INDEX_D,
					unsigned int allelSizeColumnIndex = ALLELE_SIZE_COLUMN_INDEX_D,
					unsigned int allelHeightColumnIndex = ALLELE_HEIGHT_COLUMN_INDEX_D,
					unsigned int allelAreaColumnIndex = ALLELE_AREA_COLUMN_INDEX_D);

			/**
			 * Destructor
			 * @return
			 */
			virtual ~MixtureDataParserLinearFormat();

		protected:

			/**
			 * Method parses header row
			 *
			 * @param row
			 * @exception ParserException
			 */
			void parseHeaderRow(shared_ptr<std::vector<std::string> > row);

			/**
			 * Method parses single data row
			 *
			 * @param row
			 * @param lineNumber
			 * @exception ParserException
			 */
			void parseDataRow(shared_ptr<std::vector<std::string> > row, unsigned int lineNumber) throw (ParserException);
		};

	}

}

#endif /* MIXTUREDATAPARSERLINEARFORMAT_H_ */
