#ifndef PARSEREXCEPTION_H_
#define PARSEREXCEPTION_H_

#include <stdexcept>
#include <string>
#include <boost/lexical_cast.hpp>

namespace dma
{

	namespace external
	{

		class ParserException: public std::runtime_error
		{
		private:

			unsigned int lineNumber_;

			std::string fileType_;

			std::string filename_;

			std::string errorMsg;

		public:

			ParserException(const std::string& s, unsigned int lineNumber) :
				std::runtime_error(s), lineNumber_(lineNumber), fileType_(""), filename_(""), errorMsg(s)
			{

			}

			ParserException(const std::string& errorMsg,
					const std::string& fileType,
					const std::string& fileName,
					unsigned int lineNumber) :
				std::runtime_error(errorMsg), lineNumber_(lineNumber), fileType_(fileType), filename_(fileName), errorMsg(errorMsg)
			{

			}

			ParserException(const std::string& s) :
				std::runtime_error(s), lineNumber_(0), fileType_(""), filename_(""), errorMsg(s)
			{

			}

			ParserException(const char* s) :
				std::runtime_error(s), lineNumber_(0), fileType_(""), filename_(""), errorMsg(s)
			{

			}

			~ParserException() throw ()
			{

			}

			const char* what() const throw ()
			{

				//			runtime_error* parent = (runtime_error*)(this);
				//			const char* message = parent->what();

				std::string s = "Parse error reported while parsing " + fileType_ + " ( " + filename_ + " ) at line #"
						+ boost::lexical_cast<std::string>(lineNumber_ + 1) + ": " + errorMsg;

				return s.c_str();
			}

			void setFileType(const std::string& fileType)
			{

				fileType_ = fileType;
			}

			void setFilename(const std::string& filename)
			{

				filename_ = filename;
			}

			std::string getFileType() const
			{

				return fileType_;
			}

			unsigned int getLineNumber() {

				return lineNumber_;
			}

			std::string getFilename() const
			{

				return filename_;
			}
		};
	}
}

#endif /*PARSEREXCEPTION_H_*/
