/*
 * ProfileDataParser.h
 *
 *  Created on: Dec 6, 2008
 *      Author: mariusz
 */

#ifndef PROFILEDATAPARSER_H_
#define PROFILEDATAPARSER_H_

#include <map>
#include <boost/shared_ptr.hpp>

#include "models/AlleleBase.h"
#include "models/CharacterAllele.h"
#include "models/NumericAllele.h"
#include "models/Locus.h"
#include "models/Profile.h"

#include "DataParser.h"
#include "ParserException.h"
#include "../utils/Validator.h"

using namespace boost;
using namespace dma::core;

namespace dma
{
	namespace external
	{

		class ProfileDataParser;
		typedef shared_ptr<ProfileDataParser> ProfileDataParserPtr;

		class ProfileDataParser: public DataParser
		{
		private:

			/**
			 * Profiles
			 */
			ProfilePtrMapPtr profiles_;

		public:

			/**
			 * Constructor
			 *
			 * @param filename
			 * @param separator Data fields separator. If empty std::string given data fields separator will be deduced from datafile
			 * @return
			 */
			ProfileDataParser(std::string filename, std::string separator = "");

			virtual ~ProfileDataParser();

			/**
			 * Get file type name
			 * @return
			 */
			static std::string getFileType();

			/**
			 * Parse datafile
			 *
			 * @exception ParserException
			 * @returns
			 */
			ProfilePtrMapPtr parse() throw (ParserException);

			/**
			 * Method returns parsed data
			 * @return
			 */
			ProfilePtrMapPtr getProfiles() const;

		private:

			/**
			 * Method responsible for parsing data row.
			 *
			 * @param row
			 * @param lineNumber
			 */
			void parseDataRow(shared_ptr<std::vector<std::string> > row, unsigned int lineNumber) throw (ParserException);

			/**
			 * Adds locus to profile with given name in collection of profiles
			 * @param profileName
			 * @param
			 */
			void addMarker(std::string profileName, std::string markerName, LocusPtr marker);
		};

	}
}

#endif /* PROFILEDATAPARSER_H_ */
