import glob, platform, subprocess

### ----------------------------------------------------- #
### ------------------    INCLUDES     ------------------ #
### ----------------------------------------------------- #

# Declaring common include directories
INC_DMA_CORE = '../../DmaCore/src/';
INC_DMA_EXTERNAL = '../../DmaExternal/src/';
INC_COMMON = [ INC_DMA_CORE, INC_DMA_EXTERNAL]

# Declaring system dependent include directories 
INC_DARWIN_BOOST_DIR = '/Users/mariusz/Software/libs/boost';
INC_DARWIN_PYTHON = '/Library/Frameworks/Python.framework/Headers';
INC_DARWIN_DIRS = [ INC_DARWIN_BOOST_DIR, INC_DARWIN_PYTHON ]

INC_LINUX_BOOST_DIR = '/usr/local/include/boost';
INC_LINUX_PYTHON = '/usr/include/python2.7';
INC_LINUX_DIRS = [ INC_LINUX_BOOST_DIR, INC_LINUX_PYTHON ]

# Adding system dependent include directories to include directory array 
INC_DIRS = INC_COMMON;
if ( platform.system() == 'Darwin' ):
    
    for INC_DIR in INC_DARWIN_DIRS: INC_DIRS.append( INC_DIR );
elif ( platform.system() == 'Linux' ):
    
    for INC_DIR in INC_LINUX_DIRS: INC_DIRS.append( INC_DIR );
else:
    print 'Platform not supported, quiting...';
    exit(0);







### ----------------------------------------------------- #
### ------------------    LIBRARIES    ------------------ #
### ----------------------------------------------------- #

# Declaring common libraries and directories
LIBS_COMMON= [ 'DmaExternal', 'DmaCore' ];
LIBPATH_DMA_CORE = '../../DmaCore/build';
LIBPATH_DMA_EXTERNAL = '../../DmaExternal/build';
LIBPATH_COMMON = [ LIBPATH_DMA_CORE, LIBPATH_DMA_EXTERNAL ]

# Declaring system dependent libraries and directories
LIBS_DARWIN = [ 'boost_python' ]
LIBPATH_DARWIN_BOOST = '/Users/mariusz/Software/libs/boost/bin.v2/libs/';
LIBPATH_DARWIN_PYTHON = '';
LIBPATH_DARWIN = [ LIBPATH_DARWIN_BOOST, LIBPATH_DARWIN_PYTHON ]

LIBS_LINUX = [ 'python2.5', 'boost_python-mt'  ]
LIBPATH_LINUX_BOOST = '/usr/lib';
LIBPATH_LINUX_PYTHON = '/usr/lib/python2.5'
LIBPATH_LINUX = [ LIBPATH_LINUX_BOOST, LIBPATH_LINUX_PYTHON ]

# Adding system dependent include directories to include directory array
LIBS_ARR = LIBS_COMMON;
LIBPATH_ARR = LIBPATH_COMMON;
if ( platform.system() == 'Darwin' ):
    
    for tLIB in LIBS_DARWIN: LIBS_ARR.append( tLIB );
    for tLIBPATH in LIBPATH_DARWIN: LIBPATH_ARR.append( tLIBPATH )
elif ( platform.system() == 'Linux' ):
    
    for LIB in LIBS_LINUX: LIBS_ARR.append( LIB );
    for tLIBPATH in LIBPATH_LINUX: LIBPATH_ARR.append( tLIBPATH )
else:
    print 'Platform not supported, quiting...';
    exit(0);








### ----------------------------------------------------- #
### ------------------    ENVIRONMENT    ---------------- #
### ----------------------------------------------------- #

# Environment 1 - Shared library for use with web front-end
env1 = Environment( CPPPATH = INC_DIRS, LIBS = LIBS_ARR, LIBPATH = LIBPATH_ARR, LIBPREFIX = '', LIBSUFFIX = 'so' );

# Additional system dependent flags
if(platform.system() == 'Darwin'):
    
    env1['SHLINKFLAGS'] = '$LINKFLAGS -bundle'
    env1['SHLIBSUFFIX'] = '.so'
    env1.AppendUnique(FRAMEWORKS=Split('Python'))
elif(platform.system() == 'Linux'):
    pass;
else:
    print 'Platform not supported, quiting...';
    exit(0);

Export('env1');

# Environment 2 - Shared library for use with test set
env2 = Environment( CPPPATH = INC_DIRS, LIBS = LIBS_ARR, LIBPATH = LIBPATH_ARR );

# Additional system dependent flags
if(platform.system() == 'Darwin'):
    
    env2.AppendUnique(FRAMEWORKS=Split('Python'))
elif(platform.system() == 'Linux'):
    pass;
else:
    print 'Platform not supported, quiting...';
    exit(0);

Export('env2');

# Performing build
VariantDir('build', 'src', duplicate=0)
SConscript('build/SConscript')


