/*
 * PythonDeconvolutionController.h
 *
 *  Created on: Jul 30, 2009
 *      Author: mariusz
 */

#ifndef PYTHONDECONVOLUTIONCONTROLLER_H_
#define PYTHONDECONVOLUTIONCONTROLLER_H_

#include <string>
#include <utility>
#include "models/MixtureProportion.h"
#include "controllers/DeconvolutionController.h"
#include "generators/DeconvolutionReportGenerator.h"
#include "../view/Table.h"

using namespace dma::core;
using namespace dma::external;

namespace dma
{

	namespace python
	{

		class PythonDeconvolutionController
		{
		private:

			/**
			 * Mixture data filename
			 */
			std::string mdFilename_;

			/**
			 * Mixture file format
			 */
			ExternalDataParser::MIXTURE_DATA_FILE_FORMAT mixtureDataFileFormat_;

			/**
			 * Heterozygotic balance data filename
			 */
			std::string hbdFilename_;

			/**
			 * Report filename
			 */
			std::string reportFilename_;

			/**
			 * Mixture proportions
			 */
			MixProportionPtrMapPtr mixtureProportions_;

			/**
			 * Calculation-module controller
			 */
			DeconvolutionControllerPtr cmController;

		public:

			/**
			 * Constructor :)
			 *
			 * @param mdFilename Mixture data filename
			 * @param hbdFilename Heterozygotic balance data filename
			 * @param oFilename Report filename
			 * @return
			 */
			PythonDeconvolutionController(std::string mdFilename, ExternalDataParser::MIXTURE_DATA_FILE_FORMAT mixtureDataFileFormat, std::string hbdFilename, std::string reportFilename);

			virtual ~PythonDeconvolutionController();

			/**
			 * Add mixture proportion described by string in the form ( A:B:C ), where A,B,C are relative mixture proportions.
			 * Method may throw @class std::runtime_error if parsing fails
			 *
			 * @param mpString
			 * @exception std::runtime_error
			 */
			void addMixtureProportion(std::string mpString) throw (std::runtime_error);

			/**
			 * Method performs mixture deconvolution resulting in:
			 *
			 * - Generation of report in selected location
			 * - Generation of table with deconvolution result ( valid configurations only )
			 * @return
			 */
			Table deconvulate();

		private:

			/**
			 * Method generates results table
			 * @param configurations
			 * @return
			 */
			Table generateResultTable(SampleConfigurationPtr configurations);

			/**
			 * Method used for generating header row
			 *
			 * @param maximumLociNumber
			 * @param prefixedColumns
			 * @return
			 */
			TableRow generateHeaderRow(unsigned int maximumLociNumber, shared_ptr<std::vector<std::string> > prefixedColumns);

			/**
			 * Method used for generating data row.
			 *
			 * @param markerConfiguration
			 * @param configurationNumber
			 * @param contributorsNumber
			 * @param prefixedColumns
			 * @return
			 */
			TableRow generateDataRow(MarkerConfigurationPtr markerConfiguration, unsigned int configurationNumber, shared_ptr<
					std::vector<std::string> > prefixedColumns);

		};
	}

}

#endif /* PYTHONDECONVOLUTIONCONTROLLER_H_ */
