/*
 * HypothesisPython.cpp
 *
 *  Created on: Jan 24, 2009
 *      Author: mariusz
 */

#include "HypothesisPython.h"

namespace dma
{

	namespace python
	{

		HypothesisPython::HypothesisPython(std::string name, std::string type) :
			name_(name), type_(type), contributors_(new std::vector<PersonPython>())
		{

			// Checking whether hypothesis names has been given
			if (name.length() == 0)
			{
				throw std::runtime_error("Hypothesis name has not been given");
			}

			if (type.length() == 1)
			{

				if (type.at(0) != 'P' && type.at(0) != 'D')
				{

					throw std::runtime_error("Hypothesis type is invalid. Valid hypothesis types are 'P' and 'D'");
				}
			}
			else
			{

				throw std::runtime_error("Hypothesis type has not been given");
			}
		}

		HypothesisPython::~HypothesisPython()
		{

		}

		void HypothesisPython::addContributor(PersonPython contributor)
		{

			contributors_->push_back(contributor);
		}

		HypothesisPtr HypothesisPython::getHypothesis() const
		{

			// Creating hypothesis
			Hypothesis::TYPE hType;
			if (type_.at(0) == 'P')
			{

				hType = Hypothesis::PROSECUTOR;
			}
			else
			{

				hType = Hypothesis::DEFENCE;
			}
			HypothesisPtr h(new Hypothesis(name_, hType));

			// Adding contributors
			std::vector<PersonPython>::const_iterator it1;
			for (it1 = contributors_->begin(); it1 != contributors_->end(); ++it1)
			{

				PersonPython pp = (*it1);
				h->addContributor(pp.getPerson());
			}

			return h;
		}

	}
}
