/*
 * PersonPython.cpp
 *
 *  Created on: Jan 24, 2009
 *      Author: mariusz
 */

#include "PersonPython.h"

namespace dma
{

	namespace python
	{

		PersonPython::PersonPython(std::string personId, std::string markersInJson) :
			personId_(personId), profile_(new Profile())
		{

			// Replacing ' to "
			markersInJson = replaceQuotes(markersInJson);

			// If profile has been given parsing it
			if (markersInJson.length() > 0)
			{
				parseJson(markersInJson);
			}
		}

		PersonPython::~PersonPython()
		{

		}

		PersonPtr PersonPython::getPerson() const
		{

			if (profile_->getMarkerNames()->size() > 0)
			{

				PersonPtr person(new Person(personId_, Person::UNKNOWN, profile_));
				return person;
			}
			else
			{

				PersonPtr person(new Person(personId_, Person::UNKNOWN));
				return person;
			}
		}

		std::string PersonPython::replaceQuotes(std::string original) const
		{

			std::string output = "";
			char c;

			unsigned int i;
			unsigned int length = original.length();
			for (i = 0; i < length; i++)
			{
				c = original.at(i);
				if (c == '\'')
				{
					c = '"';
				}
				output.append(1, c);
			}

			return output;
		}

		void PersonPython::parseJson(std::string jsonString) const
		{

			unsigned int counter = 1;

			// Parsing JSON
			json_spirit::Value level1;
			json_spirit::read(jsonString, level1);

			if (level1.type() == json_spirit::array_type)
			{

				json_spirit::Array level1Array = level1.get_array();
				json_spirit::Array::iterator level1It;
				for (level1It = level1Array.begin(); level1It != level1Array.end(); ++level1It)
				{

					json_spirit::Value level2 = *level1It;
					std::pair<std::string, LocusPtr> markerData = parseJsonMarkerData(level2, counter);
					profile_->addMarker(markerData.first, markerData.second);

					// Increasing "line counter" for
					++counter;
				}
			}
			else
			{
				// LEVEL 1 OBJECT IS NOT AN ARRAY

				std::string errorMsg = "JSON parse error: Expected array in level 1 of DNA data for profile '" + personId_ + "'";
				throw std::runtime_error(errorMsg);
			}
		}

		std::pair<std::string, LocusPtr> PersonPython::parseJsonMarkerData(json_spirit::Value level2,
				unsigned int counter) const
		{

			if (level2.type() == json_spirit::array_type)
			{

				json_spirit::Array level2Array = level2.get_array();
				if (level2Array.size() == 3)
				{

					// Checking whether data is of appropriate types
					if (level2Array.at(0).type() != json_spirit::str_type)
					{
						std::string errorMsg =
								"JSON parse error: Expected marker name to be std::string in level 3 of DNA data for profile '"
										+ personId_ + "' at element with index '" + lexical_cast<std::string> (counter) + "'";
						throw std::runtime_error(errorMsg);
					}
					if (level2Array.at(1).type() != json_spirit::str_type)
					{
						std::string errorMsg =
								"JSON parse error: Expected allele 1 to be std::string in level 3 of DNA data for profile '"
										+ personId_ + "' at element with index '" + lexical_cast<std::string> (counter) + "'";
						throw std::runtime_error(errorMsg);
					}
					if (level2Array.at(2).type() != json_spirit::str_type)
					{
						std::string errorMsg =
								"JSON parse error: Expected allele 2 to be std::string in level 3 of DNA data for profile '"
										+ personId_ + "' at element with index '" + lexical_cast<std::string> (counter) + "'";
						throw std::runtime_error(errorMsg);
					}

					// Fetching data
					std::string markerName = level2Array.at(0).get_str();
					std::string sAllele1 = level2Array.at(1).get_str();
					std::string sAllele2 = level2Array.at(2).get_str();

					// Creating allele representations
					AlleleBasePtr allel1;
					AlleleBasePtr allel2;

					// Determining allele types
					AlleleBase::ALLELE_TYPE alleleType;
					bool isDouble1 = Validator::isDouble(sAllele1);
					bool isDouble2 = Validator::isDouble(sAllele2);
					if (isDouble1 == true && isDouble2 == true)
					{
						alleleType = AlleleBase::NUMERIC;
					}
					else
					{
						alleleType = AlleleBase::CHARACTER;
					}

					if (alleleType == AlleleBase::NUMERIC)
					{

						// Parsing floats
						double dAllele1 = lexical_cast<double> (sAllele1);
						double dAllele2 = lexical_cast<double> (sAllele2);

						// Setting appropriate allele representations
						allel1.reset(new NumericAllele(dAllele1));
						allel2.reset(new NumericAllele(dAllele2));
					}
					else
					{

						// Setting appropriate allele representations
						allel1.reset(new CharacterAllele(sAllele1.at(0)));
						allel2.reset(new CharacterAllele(sAllele2.at(0)));
					}

					// Creating locus with given alleles
					try
					{
						LocusPtr locus(new Locus(allel1, allel2));

						// Creating output container
						return std::make_pair(markerName, locus);
					} catch (AlleleTypeConflictException e)
					{

						throw AlleleTypeConflictException(e.getOldAlleleType(),
								e.getNewAlleleType(),
								e.getMarkerName(),
								"PersonPython::parseJsonMarkerData");
					}
				}
				else
				{

					// LEVEL 2 OBJECT IS NOT AN ARRAY
					std::string errorMsg = "JSON parse error: Expected array of size 3 in level 2 of DNA data for profile '"
							+ personId_ + "' at element with index '" + lexical_cast<std::string> (counter) + "'";
					throw std::runtime_error(errorMsg);
				}
			}
			else
			{
				// LEVEL 2 OBJECT IS NOT AN ARRAY
				std::string errorMsg = "JSON parse error: Expected array in level 2 of DNA data for profile '" + personId_
						+ "' at element with index '" + lexical_cast<std::string> (counter) + "'";
				throw std::runtime_error(errorMsg);
			}
		}

	}
}
