/*
 * PersonPython.h
 *
 *  Created on: Jan 24, 2009
 *      Author: mariusz
 */

#ifndef PERSONPYTHON_H_
#define PERSONPYTHON_H_

#include <string>
#include "models/Person.h"
#include "models/Profile.h"
#include "models/CharacterAllele.h"
#include "models/NumericAllele.h"
#include "models/Person.h"
#include "json_spirit/json_spirit.h"
#include "utils/Validator.h"

using namespace dma::core;
using namespace dma::external;

namespace dma
{

	namespace python
	{

		/**
		 * Class used for wrapping class Person with build-in profile JSON parser
		 */
		class PersonPython
		{
		private:

			/**
			 * Person identifier
			 */
			std::string personId_;

			/**
			 * Parsed person object
			 */
			ProfilePtr profile_;

		public:

			/**
			 * Simple constructor
			 *
			 * @param personId Person identifier
			 * @param markersInJson DNA profile given in JSON format
			 * @return
			 */
			PersonPython(std::string personId, std::string markersInJson);

			virtual ~PersonPython();

			/**
			 * Get parsed person profile
			 * @return
			 */
			PersonPtr getPerson() const;

		private:

			/**
			 * Method replaces character '"' with ''' character in given string
			 *
			 * @param original
			 * @return
			 */
			std::string replaceQuotes(std::string original) const;

			/**
			 * Method parses profile given in JSON format, i.e. [["AMEL", "X", "Y"], ["vWA", "10.1", "10.3"]]
			 * Found marker data is added to profile of person
			 *
			 * @param jsonString Data given in JSON format
			 */

			void parseJson(std::string jsonString) const;

			/**
			 * Method parses marker data given in JSON format, i.e. ["vWA", "10.1", "10.3"]
			 *
			 * @param level2 Element containing 3-element array with data
			 * @param counter
			 * @return
			 */
			std::pair<std::string, LocusPtr>
					parseJsonMarkerData(json_spirit::Value level2, unsigned int counter) const;

		};
	}
}

#endif /* PERSONPYTHON_H_ */
