import unittest
import dmapython
import simplejson as json

class LikelihoodRatioPythonTestCase(unittest.TestCase):

    def setUp(self):
    
        # Setting filenames for input data files
        self.reportFilename = '../../../data/report/deconvolution_report.csv';
        self.populationDataFilename = '../../../data/population_data/allele_freq_pl.csv';
        
        # Setting up contributors
        profile_Hp1 = [
            ['AMEL','X','Y'],
            ['D16S539','12','13'],
            ['D18S51','12','14'],
            ['D19S433','12','14.2'],
            ['D21S11','28','33.2'],
            ['D2S1338','17','22'],
            ['D3S1358','14','14'],
            ['D8S1179','11','12'],
            ['FGA','20','22'],
            ['TH01','6','8'],
            ['vWA','14','18']
        ];
        contributor_Hp1 = dmapython.PersonPython('Hp_1', json.dumps(profile_Hp1) );
        contributor_Hp2 = dmapython.PersonPython('Hp_2', '' );
        contributor_Hd1 = dmapython.PersonPython('Hd_1', '' );
        contributor_Hd2 = dmapython.PersonPython('Hd_2', '' );
        
        # Forming hypothesis
        self.hp = dmapython.HypothesisPython('Hypothesis - prosecutor', 'P');
        self.hd = dmapython.HypothesisPython('Hypothesis - defense', 'D');
        self.hp.addContributor( contributor_Hp1 );
        self.hp.addContributor( contributor_Hp2 );
        self.hd.addContributor( contributor_Hd1 );
        self.hd.addContributor( contributor_Hd2 );
        
    def testLikelihoodRatioPython(self):
        
        # Creating controller
      controller = dmapython.PythonLrCalculationController( self.reportFilename, self.populationDataFilename, self.hp, self.hd);

      # Performing calculation
      lr = controller.calculateLr();
      
      # Checking whether marker names are present
      markerNames = lr.getMarkerNames();
      self.assertEqual( 1, markerNames.count('D16S539') );
      self.assertEqual( 1, markerNames.count('D18S51') );
      self.assertEqual( 1, markerNames.count('D19S433') );
      self.assertEqual( 1, markerNames.count('D21S11') );
      self.assertEqual( 1, markerNames.count('D2S1338') );
      self.assertEqual( 1, markerNames.count('D3S1358') );
      self.assertEqual( 1, markerNames.count('D8S1179') );
      self.assertEqual( 1, markerNames.count('FGA') );
      self.assertEqual( 1, markerNames.count('TH01') );
      self.assertEqual( 1, markerNames.count('vWA') );
      
      # Checking accessor for LR formula - existing marker
      self.assert_( len(lr.getLrFormula('vWA')) > 0 );
      
      # Checking accessor for LR formula - non-existing marker
      self.assertRaises( RuntimeError, lr.getLrFormula, 'NonExistingMarker' );
      
      # Checking accessor for LR value - existing marker
      self.assertEqual( 'float', type( lr.getLrValue('vWA') ).__name__);
      
      # Checking accessor for LR formula - non-existing marker
      self.assertRaises( RuntimeError, lr.getLrValue, 'NonExistingMarker' );
      
      # Checking accessor for total LR value
      self.assertEqual( 'float', type( lr.getTotalLrValue() ).__name__);