import unittest
import dmapython
import os;
import os.path

class PythonDeconvolutionControllerTestCase(unittest.TestCase):

    def setUp(self):
        self.mixtureDataFilename = '../../../data/mixture_data/mix_sample_118_353_1a_mac.csv';
        self.hdDataFilename = '../../../data/hb_data/hbmin.csv';
        self.reportFilename = '../../../data/generated_reports/report.csv';
        
    def testPythonDeconvolutionController_Valid(self):
        
        # Creating controller
        controller1 = dmapython.PythonDeconvolutionController(self.mixtureDataFilename, dmapython.MixtureDataFileFormat.ColumnarFormat, self.hdDataFilename, self.reportFilename);
        controller1.addMixtureProportion('0.24:0.76');
        
        # Performing deconvolution
        table = controller1.deconvulate();
        
        # Checking whether deconvolution result contains data in valid form ( header + N data rows )
        self.assertEqual( 'Table', type(table).__name__ );
        self.assert_( table.getRowsNumber() >= 2 );
        self.assert_( table.getRow(0).getCellsNumber() > 0 );
        self.assert_( table.getRow(1).getCellsNumber() > 0 );
        self.assertEqual( 'H', table.getRow(0).getCell(0).getCellType() );
        self.assertEqual( 'D', table.getRow(1).getCell(0).getCellType() );
        self.assert_( len( table.getRow(0).getCell(0).getContents() ) > 0 );
        self.assert_( len( table.getRow(1).getCell(0).getContents() ) > 0 );
        
        # Checking whether report file has been generated
        self.assertEqual ( True, os.path.isfile( self.reportFilename ) );
        
        # Deleting generated report file
        os.remove( self.reportFilename );
     
        
    def testPythonDeconvolutionController_Invalid(self):
        
        # Creating controller
        controller1 = dmapython.PythonDeconvolutionController(self.mixtureDataFilename, dmapython.MixtureDataFileFormat.ColumnarFormat, self.hdDataFilename, self.reportFilename);
        
        # Testing incorrect mixture proportions - empty
        self.assertRaises( RuntimeError, controller1.addMixtureProportion, '');
        
        # Testing incorrect mixture proportions - incorrect separator
        self.assertRaises( RuntimeError, controller1.addMixtureProportion, '0.24|0.76');
        
        # Testing incorrect mixture proportions - incorrect proportion value
        self.assertRaises( RuntimeError, controller1.addMixtureProportion, 'qwe:0.76');
        