/*
 * TestConfigurationGenerator.h
 *
 *  Created on: Oct 5, 2008
 *      Author: mariusz
 */

#ifndef TESTGENOTYPEGENERATOR_H_
#define TESTGENOTYPEGENERATOR_H_

#include <map>
#include <string>
#include <boost/shared_ptr.hpp>
#include <utility>

#include "cute.h"

#include "models/Marker.h"
#include "models/NumericAllele.h"
#include "models/CharacterAllele.h"
#include "models/MixtureProportion.h"
#include "models/AlleleBase.h"
#include "analysis/ConfigurationGenerator.h"
#include "analysis/PersonNumberAnalyzer.h"
#include "utils/MixtureDataPrinters.h"
#include "analysis/Strategies.h"
#include "models/statuses/MCStatusValid.h"
#include "models/statuses/MCStatusExcEquivalence.h"

using namespace dma::core;

namespace dma
{

	namespace test
	{

		/**
		 * Unit test class for @class ConfigurationGenerator
		 *
		 * Tested class methods coverage:
		 *	- generate()
		 *	- generateConfigurations()
		 *		- testGenerateConfigurations_2A_1P()
		 *		- testGenerateConfigurations_3A_2P()
		 *	- filterEquivalentConfigurations()
		 *		- testFilterEquivalentConfigurations()
		 *	- filterConfigurationsByAllelePresence()
		 *		- testFilterConfigurationsByAllelePresence()
		 *	- generatePairwiseCombinations()
		 *		- testGeneratePairwiseCombinations()
		 *	- checkForConfigurationDuplicates()
		 *		- testCheckForConfigurationDuplicates()
		 *	- checkForConfigurationEquivalents()
		 *		- testCheckForConfigurationEquivalents()
		 */
		class TestConfigurationGenerator: public ConfigurationGenerator
		{
		private:
			ConfigurationGenerator generator;
			NumericAllelePtr na10;
			NumericAllelePtr na11;
			NumericAllelePtr na12;
			NumericAllelePtr na13;
			NumericAllelePtr na14;
			NumericAllelePtr na15;
		public:
			TestConfigurationGenerator();
			virtual ~TestConfigurationGenerator();

			void testGeneratePairwiseCombinations();
			void testCheckForConfigurationDuplicates();
			void testCheckForConfigurationEquivalents();
			void testFilterEquivalentConfigurations();
			void testFilterConfigurationsByAllelePresence();
			void testGenerateConfigurations_2A_1P();
			void testGenerateConfigurations_3A_2P();

			void testConfigurationNumber();


		private:

			/**
			 * Method checks whether given allele pair exists in collection
			 *
			 * @param pair
			 * @param collection
			 * @return
			 */
			bool testGeneratePairwiseCombinations_ContainsPair(AlleleBasePtrPair pair, shared_ptr<std::vector<AlleleBasePtrPair> > collection);

			/**
			 * Method checks whether given configuration exists in collection
			 *
			 * @param configuration
			 * @param collection
			 * @return
			 */
			bool testGenerateConfigurations_ConfigurationExists( MarkerConfigurationPtr configuration, MarkerConfigurationPtrVectorPtr collection);
		};
	}
}

#endif /* TESTGENOTYPEGENERATOR_H_ */
