/*
 * TestLrCalculator.cpp
 *
 *  Created on: Dec 16, 2008
 *      Author: mariusz
 */

#include "TestLrCalculator.h"

namespace dma
{

	namespace test
	{

		TestLrCalculator::TestLrCalculator()
		{

		}

		TestLrCalculator::~TestLrCalculator()
		{

		}

		void TestLrCalculator::testFindAlleleInPopulationData()
		{

			/* Generating population data and creating tested object instance. Alleles in population data:
			 *
			 * Marker name	Allele value	Allele frequency
			 * MarkerA		10				0.2
			 * MarkerA		11				0.3
			 * MarkerA		12				0.4
			 * MarkerA		13				0.1
			 * MarkerB		20				0.1
			 * MarkerB		21				0.1
			 * MarkerB		22				0.6
			 * MarkerB		23				0.2
			 */
			MarkerPtrVectorPtr pd = generatePopulationData();
			LrCalculatorPtr lrCalc(new LrCalculator(pd));

			// Generating searched alleles
			NumericAllelePtr allele_A_10(new NumericAllele(10.0));
			NumericAllelePtr allele_A_13(new NumericAllele(13.0));
			NumericAllelePtr allele_A_14(new NumericAllele(14.0));
			NumericAllelePtr allele_B_20(new NumericAllele(20.0));
			NumericAllelePtr allele_B_23(new NumericAllele(23.0));
			NumericAllelePtr allele_B_24(new NumericAllele(24.0));
			allele_A_10->setMarkerName("MarkerA");
			allele_A_13->setMarkerName("MarkerA");
			allele_A_14->setMarkerName("MarkerA");
			allele_B_20->setMarkerName("MarkerB");
			allele_B_23->setMarkerName("MarkerB");
			allele_B_24->setMarkerName("MarkerB");

			// Performing tests - marker A
			ASSERT_EQUAL( 10, lrCalc->findAlleleInPopulationData(allele_A_10)->getAlleleValue() );
			ASSERT_EQUAL( 13, lrCalc->findAlleleInPopulationData(allele_A_13)->getAlleleValue() );
			ASSERT_EQUAL( 0.2, lrCalc->findAlleleInPopulationData(allele_A_10)->getFrequency() );
			ASSERT_EQUAL( 0.1, lrCalc->findAlleleInPopulationData(allele_A_13)->getFrequency() );
			ASSERT_THROWS( lrCalc->findAlleleInPopulationData(allele_A_14), AlleleNotFoundException );

			// Performing tests - marker B
			ASSERT_EQUAL( 20, lrCalc->findAlleleInPopulationData(allele_B_20)->getAlleleValue() );
			ASSERT_EQUAL( 23, lrCalc->findAlleleInPopulationData(allele_B_23)->getAlleleValue() );
			ASSERT_EQUAL( 0.1, lrCalc->findAlleleInPopulationData(allele_B_20)->getFrequency() );
			ASSERT_EQUAL( 0.2, lrCalc->findAlleleInPopulationData(allele_B_23)->getFrequency() );
			ASSERT_THROWS( lrCalc->findAlleleInPopulationData(allele_B_24), AlleleNotFoundException );
		}

		void TestLrCalculator::testBuildAEForLocus()
		{

			/* Generating population data and creating tested object instance. Alleles in population data:
			 *
			 * Marker name	Allele value	Allele frequency
			 * MarkerA		10				0.2
			 * MarkerA		11				0.3
			 * MarkerA		12				0.4
			 * MarkerA		13				0.1
			 * MarkerB		20				0.1
			 * MarkerB		21				0.1
			 * MarkerB		22				0.6
			 * MarkerB		23				0.2
			 */
			MarkerPtrVectorPtr pd = generatePopulationData();
			LrCalculatorPtr lrCalc(new LrCalculator(pd));

			NumericAllelePtr nap10(new NumericAllele(10.0));
			NumericAllelePtr nap11(new NumericAllele(11.0));

			LocusPtr lcp_10_10(new Locus(nap10->cloneToPtr(), nap10->cloneToPtr()));
			LocusPtr lcp_10_11(new Locus(nap10->cloneToPtr(), nap11->cloneToPtr()));
			lcp_10_10->setMarkerName("MarkerA");
			lcp_10_11->setMarkerName("MarkerA");

			AlgValuePtr expected1(new AlgValue("( p_10 * p_10 )", 0.04));
			AlgValuePtr expected2(new AlgValue("( 2 * p_10 * p_11 )", 0.12));

			AlgExpPtr calculated1 = lrCalc->buildAEForLocus(lcp_10_10);
			AlgExpPtr calculated2 = lrCalc->buildAEForLocus(lcp_10_11);

			ASSERT_EQUAL( expected1->getSymbol(), calculated1->getSymbol());
			ASSERT_EQUAL( expected1->getValue(), calculated1->getValue());
			ASSERT_EQUAL( expected2->getSymbol(), calculated2->getSymbol());
			ASSERT_EQUAL( expected2->getValue(), calculated2->getValue());
		}

		void TestLrCalculator::testBuildAEForMarkerConfiguration()
		{

			/* Generating population data and creating tested object instance. Alleles in population data:
			 *
			 * Marker name	Allele value	Allele frequency
			 * MarkerA		10				0.2
			 * MarkerA		11				0.3
			 * MarkerA		12				0.4
			 * MarkerA		13				0.1
			 * MarkerB		20				0.1
			 * MarkerB		21				0.1
			 * MarkerB		22				0.6
			 * MarkerB		23				0.2
			 */
			MarkerPtrVectorPtr pd = generatePopulationData();
			LrCalculatorPtr lrCalc(new LrCalculator(pd));

			NumericAllelePtr nap10(new NumericAllele(10.0));
			NumericAllelePtr nap11(new NumericAllele(11.0));
			NumericAllelePtr nap12(new NumericAllele(12.0));
			NumericAllelePtr nap13(new NumericAllele(13.0));

			LocusPtr lcp_10_11(new Locus(nap10->cloneToPtr(), nap11->cloneToPtr()));
			LocusPtr lcp_12_13(new Locus(nap12->cloneToPtr(), nap13->cloneToPtr()));
			MarkerConfigurationPtr mcp_10_11__12_13(new MarkerConfiguration("MarkerA"));
			mcp_10_11__12_13->addLocus(lcp_10_11);
			mcp_10_11__12_13->addLocus(lcp_12_13);

			// Building algebraic expression
			AlgExpPtr lr = lrCalc->buildAEForMarkerConfiguration(mcp_10_11__12_13);

			// Testing
			std::string expected1 = "( 2 * ( 2 * p_10 * p_11 ) * ( 2 * p_12 * p_13 ) )";
			ASSERT_EQUAL( expected1, lr->getSymbol() );
		}

		void TestLrCalculator::testBuildAEForMarkerConfigurations()
		{

			/* Generating population data and creating tested object instance. Alleles in population data:
			 *
			 * Marker name	Allele value	Allele frequency
			 * MarkerA		10				0.2
			 * MarkerA		11				0.3
			 * MarkerA		12				0.4
			 * MarkerA		13				0.1
			 * MarkerB		20				0.1
			 * MarkerB		21				0.1
			 * MarkerB		22				0.6
			 * MarkerB		23				0.2
			 */
			MarkerPtrVectorPtr pd = generatePopulationData();
			LrCalculatorPtr lrCalc(new LrCalculator(pd));

			NumericAllelePtr nap10(new NumericAllele(10.0));
			NumericAllelePtr nap11(new NumericAllele(11.0));
			NumericAllelePtr nap12(new NumericAllele(12.0));
			NumericAllelePtr nap13(new NumericAllele(13.0));

			// Generating configuration 1
			LocusPtr lcp_10_11(new Locus(nap10->cloneToPtr(), nap11->cloneToPtr()));
			LocusPtr lcp_12_13(new Locus(nap12->cloneToPtr(), nap13->cloneToPtr()));
			MarkerConfigurationPtr mcp_10_11__12_13(new MarkerConfiguration("MarkerA"));
			mcp_10_11__12_13->addLocus(lcp_10_11);
			mcp_10_11__12_13->addLocus(lcp_12_13);

			// Generating configuration 2
			LocusPtr lcp_10_12(new Locus(nap10->cloneToPtr(), nap12->cloneToPtr()));
			LocusPtr lcp_11_13(new Locus(nap11->cloneToPtr(), nap13->cloneToPtr()));
			MarkerConfigurationPtr mcp_10_12__11_13(new MarkerConfiguration("MarkerA"));
			mcp_10_12__11_13->addLocus(lcp_10_12);
			mcp_10_12__11_13->addLocus(lcp_11_13);

			// Creating configuration set
			MarkerConfigurationPtrVectorPtr configurations(new MarkerConfigurationPtrVector());
			configurations->push_back(mcp_10_11__12_13);
			configurations->push_back(mcp_10_12__11_13);

			// Building algebraic expression
			AlgExpPtr lr = lrCalc->buildAEForMarkerConfigurations(configurations);

			// Testing
			std::string expected1 =
					"( ( 2 * ( 2 * p_10 * p_11 ) * ( 2 * p_12 * p_13 ) ) + ( 2 * ( 2 * p_10 * p_12 ) * ( 2 * p_11 * p_13 ) ) )";
			ASSERT_EQUAL( expected1, lr->getSymbol() );
		}

		void TestLrCalculator::testCalculateLr()
		{

			/* Generating population data
			 *
			 * Marker name	Allele value	Allele frequency
			 * MarkerA		10				0.1
			 * MarkerA		11				0.1
			 * MarkerA		12				0.1
			 * MarkerA		13				0.1
			 * MarkerA		14				0.1
			 * MarkerA		15				0.1
			 */
			MarkerPtrVectorPtr pd(new MarkerPtrVector());
			MarkerPtr pdMarkerA = generatePopulationData("MarkerA", 10, 6, 0.1);
			pd->push_back(pdMarkerA);

			// Creating tested class instance
			LrCalculatorPtr lrCalculator(new LrCalculator(pd));

			// Generating genetic data
			NumericAllelePtr na10(new NumericAllele(10.0));
			NumericAllelePtr na11(new NumericAllele(11.0));
			NumericAllelePtr na12(new NumericAllele(12.0));
			NumericAllelePtr na13(new NumericAllele(13.0));
			NumericAllelePtr na14(new NumericAllele(14.0));
			NumericAllelePtr na15(new NumericAllele(15.0));
			CharacterAllelePtr caX(new CharacterAllele('X'));
			CharacterAllelePtr caY(new CharacterAllele('Y'));

			LocusPtr locus_10_11(new Locus(na10->cloneToPtr(), na11->cloneToPtr()));
			LocusPtr locus_12_13(new Locus(na12->cloneToPtr(), na13->cloneToPtr()));
			LocusPtr locus_14_15(new Locus(na14->cloneToPtr(), na15->cloneToPtr()));
			LocusPtr locus_X_X(new Locus(caX->cloneToPtr(), caX->cloneToPtr()));
			LocusPtr locus_X_Y(new Locus(caX->cloneToPtr(), caY->cloneToPtr()));

			// Creating configurations and sample configurations
			MarkerConfigurationPtr mc_10_11_12_13(new MarkerConfiguration("MarkerA"));
			mc_10_11_12_13->addLocus(locus_10_11->cloneToSmartPtr());
			mc_10_11_12_13->addLocus(locus_12_13->cloneToSmartPtr());
			MarkerConfigurationPtr mc_12_13_14_15(new MarkerConfiguration("MarkerA"));
			mc_12_13_14_15->addLocus(locus_12_13->cloneToSmartPtr());
			mc_12_13_14_15->addLocus(locus_14_15->cloneToSmartPtr());
			MarkerConfigurationPtr mc_X_X__X_Y(new MarkerConfiguration("MarkerB"));
			mc_X_X__X_Y->addLocus(locus_X_X->cloneToSmartPtr());
			mc_X_X__X_Y->addLocus(locus_X_Y->cloneToSmartPtr());
			MarkerConfigurationPtr mc_X_Y__X_Y(new MarkerConfiguration("MarkerB"));
			mc_X_Y__X_Y->addLocus(locus_X_Y->cloneToSmartPtr());
			mc_X_Y__X_Y->addLocus(locus_X_Y->cloneToSmartPtr());

			MarkerConfigurationPtrVectorPtr configurationsMarkerA(new MarkerConfigurationPtrVector());
			configurationsMarkerA->push_back(mc_10_11_12_13);
			configurationsMarkerA->push_back(mc_12_13_14_15);
			MarkerConfigurationPtrVectorPtr configurationsMarkerB(new MarkerConfigurationPtrVector());
			configurationsMarkerB->push_back(mc_X_X__X_Y);
			configurationsMarkerB->push_back(mc_X_Y__X_Y);

			SampleConfigurationPtr sample1( new SampleConfiguration("Sample1") );
			sample1->addMarkerConfigurations( "MarkerA", configurationsMarkerA );
			sample1->addMarkerConfigurations( "MarkerB", configurationsMarkerB );

			// Creating known suspect profile
			ProfilePtr profile(new Profile());
			profile->addMarker("MarkerA", locus_10_11->cloneToSmartPtr());

			// Creating hypothesis
			PersonPtr hpSuspect(new Person("SUSPECT_1", Person::SUSPECT, profile));
			PersonPtr hpUnknown(new Person("UNKOWN_HP_1", Person::UNKNOWN));
			HypothesisPtr hp(new Hypothesis("Prosecutor's hypothesis", Hypothesis::PROSECUTOR));
			hp->addContributor(hpSuspect);
			hp->addContributor(hpUnknown);

			PersonPtr hdUnknown1(new Person("UNKOWN_HD_1", Person::UNKNOWN));
			PersonPtr hdUnknown2(new Person("UNKOWN_HD_2", Person::UNKNOWN));
			HypothesisPtr hd(new Hypothesis("Defence's hypothesis", Hypothesis::DEFENCE));
			hd->addContributor(hdUnknown1);
			hd->addContributor(hdUnknown2);

			// Building up expected LR for Marker A
			std::string expectedMarkerA_Hp = "( 2 * p_12 * p_13 )";
			std::string expectedMarkerA_Hd =
					"( ( 2 * ( 2 * p_10 * p_11 ) * ( 2 * p_12 * p_13 ) ) + ( 2 * ( 2 * p_12 * p_13 ) * ( 2 * p_14 * p_15 ) ) )";
			std::string expectedMarkerA = "( " + expectedMarkerA_Hp + " / " + expectedMarkerA_Hd + " )";

			// Performing likelihood ratio calculation
			LikelihoodRatioPtr lr = lrCalculator->calculateLr(sample1, hp, hd);

			// Testing
			ASSERT_EQUAL( 1, lr->getMarkerNames()->size() );
			ASSERT_EQUAL( "MarkerA", lr->getMarkerNames()->at(0) );
			ASSERT_EQUAL( expectedMarkerA, lr->getLikelihoodRatio("MarkerA")->getSymbol() );
		}

		MarkerPtr TestLrCalculator::generatePopulationData(std::string markerName,
				unsigned int firstAlleleValue,
				unsigned int alleleNumber,
				double frequency)
		{

			// Creating marker
			MarkerPtr marker(new Marker(markerName));

			// Adding alleles
			unsigned int i;
			unsigned int alleleValue = firstAlleleValue;
			for (i = 0; i < alleleNumber; i++)
			{

				// Creating allele
				double d = (double) alleleValue;
				NumericAllelePtr nap(new NumericAllele(d));
				nap->setFrequency(frequency);
				marker->addAllele(nap);

				// Incrementing allele values
				alleleValue++;
			}

			return marker;
		}

		MarkerPtrVectorPtr TestLrCalculator::generatePopulationData()
		{

			NumericAllelePtr nap10(new NumericAllele(10.0));
			NumericAllelePtr nap11(new NumericAllele(11.0));
			NumericAllelePtr nap12(new NumericAllele(12.0));
			NumericAllelePtr nap13(new NumericAllele(13.0));
			nap10->setFrequency(0.2);
			nap11->setFrequency(0.3);
			nap12->setFrequency(0.4);
			nap13->setFrequency(0.1);

			MarkerPtr markerA(new Marker("MarkerA"));
			markerA->addAllele(nap10);
			markerA->addAllele(nap11);
			markerA->addAllele(nap12);
			markerA->addAllele(nap13);

			NumericAllelePtr nap20(new NumericAllele(20.0));
			NumericAllelePtr nap21(new NumericAllele(21.0));
			NumericAllelePtr nap22(new NumericAllele(22.0));
			NumericAllelePtr nap23(new NumericAllele(23.0));
			nap20->setFrequency(0.1);
			nap21->setFrequency(0.1);
			nap22->setFrequency(0.6);
			nap23->setFrequency(0.2);

			MarkerPtr markerB(new Marker("MarkerB"));
			markerB->addAllele(nap20);
			markerB->addAllele(nap21);
			markerB->addAllele(nap22);
			markerB->addAllele(nap23);

			MarkerPtrVectorPtr pd(new MarkerPtrVector());
			pd->push_back(markerA);
			pd->push_back(markerB);

			return pd;
		}
	}
}

/*
 void TestLrCalculator::testBaldingP89()
 {

 try
 {

 std::string CONFIGURATIONS_FILENAME = "../data/tests/lr_calculator/balding_p_89/sample_balding_p_89.csv";
 std::string POPULATION_FILENAME = "../data/tests/lr_calculator/balding_p_89/allele_freq_balding_p_89.csv";
 std::string PROFILE_FILENAME = "../data/tests/lr_calculator/balding_p_89/profile_balding_p_89.csv";

 // Creating parser and parsing data
 OutputDataParserPtr outputDataParserPtr(new OutputDataParser(CONFIGURATIONS_FILENAME));
 PopulationDataParserPtr populationDataParser(new PopulationDataParser(POPULATION_FILENAME));
 ProfileDataParserPtr profileDataParser(new ProfileDataParser(PROFILE_FILENAME));
 ProfilePtrMapPtr profiles = profileDataParser->parse();
 SampleConfigurationPtr configurations = outputDataParserPtr->parse();
 MarkerPtrVectorPtr populationData = populationDataParser->parse();

 ProfilePtr profile = profiles->find("sample_balding_p_89")->second;

 // Creating hypothesis
 PersonPtr hpSuspect(new Person("SUSPECT_1", Person::SUSPECT, profile));
 HypothesisPtr hp(new Hypothesis("Prosecutor's hypothesis", Hypothesis::PROSECUTOR));
 hp->addContributor(hpSuspect);

 PersonPtr hdUnknown(new Person("UNKNOWN_1", Person::UNKNOWN));
 HypothesisPtr hd(new Hypothesis("Defence's hypothesis", Hypothesis::DEFENCE));
 hd->addContributor(hdUnknown);

 // Calculating LR
 LrCalculatorPtr lrCalculator(new LrCalculator(populationData));
 LikelihoodRatioPtr lr = lrCalculator->calculateLr(configurations, hp, hd);

 ASSERT_EQUAL( 45000.0, lr->getLikelihoodRatio("D18")->getValue() );
 ASSERT_EQUAL( 32000.0, lr->getLikelihoodRatio("D21")->getValue() );
 ASSERT_EQUAL( 90000.0, lr->getLikelihoodRatio("TH01")->getValue() );
 ASSERT_EQUAL( 59000.0, lr->getLikelihoodRatio("D8")->getValue() );

 } catch (std::exception& e)
 {

 std::cout << e.what() << std::endl;
 }
 }
 */
