/*
 * TestLrCalculator.h
 *
 *  Created on: Dec 16, 2008
 *      Author: mariusz
 */

#ifndef TESTLRCALCULATOR_H_
#define TESTLRCALCULATOR_H_

#include "cute.h"
#include <string>
#include "analysis/LrCalculator.h"
#include "models/Hypothesis.h"
#include "models/Person.h"
#include "models/NumericAllele.h"
#include "models/CharacterAllele.h"

using namespace dma::core;

namespace dma
{

	namespace test
	{

		/**
		 * Unit test class for @class Marker
		 *
		 * Public method of tested class the test methods associated:
		 *
		 * 	- setPopulationData()
		 * 		- testFindAlleleInPopulationData()
		 * 	- calculateLr()
		 * 		- testCalculateLr()
		 * 	- calculateLrForMarker()
		 * 		- testCalculateLr()
		 * 	- findAlleleInPopulationData()
		 * 		- testFindAlleleInPopulationData()
		 * 	- buildAEForMarkerConfigurations()
		 * 		- testbuildAEForMarkerConfigurations()
		 * 	- buildAEForMarkerConfiguration()
		 * 		- testBuildAEForMarkerConfiguration()
		 * 	- buildAEForLocus()
		 * 		- testBuildAEForLocus()
		 */
		class TestLrCalculator
		{
		private:

			/**
			 * Generated population data
			 */
			MarkerPtrVectorPtr populationData_;

		public:
			TestLrCalculator();
			virtual ~TestLrCalculator();

			// OLD
			// void testBaldingP89(); // Test do weryfikacji
			// OLD

			void testFindAlleleInPopulationData();
			void testBuildAEForLocus();
			void testBuildAEForMarkerConfiguration();
			void testBuildAEForMarkerConfigurations();
			void testCalculateLr();
		private:

			/**
			 * Generate population data for given marker. Allele values differ by 1.0
			 *
			 * @param markerName Name of the marker
			 * @param firstAlleleValue Value of first allele, i.e. 20. Next allele in the set will have value 21
			 * @param alleleNumber Number of alleles generated.
			 * @param frequency Allele frequency
			 * @return
			 */
			MarkerPtr generatePopulationData(std::string markerName,
					unsigned int firstAlleleValue,
					unsigned int alleleNumber,
					double frequency);

			/**
			 * Method generates following population data:
			 *
			 * Marker name	Allele value	Allele frequency
			 * MarkerA		10				0.2
			 * MarkerA		11				0.3
			 * MarkerA		12				0.4
			 * MarkerA		13				0.1
			 * MarkerB		20				0.1
			 * MarkerB		21				0.1
			 * MarkerB		22				0.6
			 * MarkerB		23				0.2
			 *
			 * @return
			 */
			MarkerPtrVectorPtr generatePopulationData();
		};
	}
}

#endif /* TESTLRCALCULATOR_H_ */
