/*
 * TestPeakAllotter.cpp
 *
 *  Created on: Nov 13, 2008
 *      Author: mariusz
 */

#include "TestPeakAllotter.h"

namespace dma
{

	namespace test
	{

		TestPeakAllotter::TestPeakAllotter() :
			na_c_1(new NumericAllele(1.0)), na_c_2(new NumericAllele(2.0)), na_c_3(new NumericAllele(3.0)),
					na_c_4(new NumericAllele(4.0)), na_c_5(new NumericAllele(5.0)), na_c_6(new NumericAllele(6.0))
		{

		}

		TestPeakAllotter::~TestPeakAllotter()
		{

		}

		void TestPeakAllotter::test_2P_3A_HE_1_1()
		{

			// Generating mixture data
			NumericAllelePtr na1(new NumericAllele(1.0, 0.0, 100.0, 1000.0));
			NumericAllelePtr na2(new NumericAllele(2.0, 0.0, 200.0, 2000.0));
			NumericAllelePtr na3(new NumericAllele(3.0, 0.0, 100.0, 1000.0));
			MarkerPtr mixMarker1(new Marker("MarkerMixture"));
			mixMarker1->addAllele(na1);
			mixMarker1->addAllele(na2);
			mixMarker1->addAllele(na3);

			// Generating contributor genotypes and configurations
			LocusPtr locus_1_2(new Locus(na_c_1->cloneToPtr(), na_c_2->cloneToPtr()));
			LocusPtr locus_2_3(new Locus(na_c_2->cloneToPtr(), na_c_3->cloneToPtr()));

			MarkerConfigurationPtr mc_12_23(new MarkerConfiguration("MARKER_A"));
			mc_12_23->addLocus(locus_1_2);
			mc_12_23->addLocus(locus_2_3);

			// Generating mixture proportion
			MixtureProportionPtr mixProp1(new MixtureProportion());
			mixProp1->addContributor(1);
			mixProp1->addContributor(1);

			// Performing deconvolution
			performDeconvolution(mc_12_23, mixMarker1, mixProp1);

			// Validating results - area
			ASSERT_EQUAL(1000, mc_12_23->getLocus(0)->getAllele(0)->getPeakArea());
			ASSERT_EQUAL(1000, mc_12_23->getLocus(0)->getAllele(1)->getPeakArea());
			ASSERT_EQUAL(1000, mc_12_23->getLocus(1)->getAllele(0)->getPeakArea());
			ASSERT_EQUAL(1000, mc_12_23->getLocus(1)->getAllele(1)->getPeakArea());

			// Validating results - height
			ASSERT_EQUAL(100, mc_12_23->getLocus(0)->getAllele(0)->getPeakHeight());
			ASSERT_EQUAL(100, mc_12_23->getLocus(0)->getAllele(1)->getPeakHeight());
			ASSERT_EQUAL(100, mc_12_23->getLocus(1)->getAllele(0)->getPeakHeight());
			ASSERT_EQUAL(100, mc_12_23->getLocus(1)->getAllele(1)->getPeakHeight());
		}

		void TestPeakAllotter::test_2P_3A_HO_1_1()
		{

			// Generating mixture data
			NumericAllelePtr na1(new NumericAllele(1.0, 0.0, 200.0, 2000.0));
			NumericAllelePtr na2(new NumericAllele(2.0, 0.0, 100.0, 1000.0));
			NumericAllelePtr na3(new NumericAllele(3.0, 0.0, 100.0, 1000.0));
			MarkerPtr mixMarker1(new Marker("MarkerMixture"));
			mixMarker1->addAllele(na1);
			mixMarker1->addAllele(na2);
			mixMarker1->addAllele(na3);

			// Generating contributor genotypes and configurations
			LocusPtr locus_1_1(new Locus(na_c_1->cloneToPtr(), na_c_1->cloneToPtr()));
			LocusPtr locus_2_3(new Locus(na_c_2->cloneToPtr(), na_c_3->cloneToPtr()));

			MarkerConfigurationPtr mc_11_23(new MarkerConfiguration("MARKER_A"));
			mc_11_23->addLocus(locus_1_1);
			mc_11_23->addLocus(locus_2_3);

			// Generating mixture proportion
			MixtureProportionPtr mixProp1(new MixtureProportion());
			mixProp1->addContributor(1);
			mixProp1->addContributor(1);

			// Performing deconvolution
			performDeconvolution(mc_11_23, mixMarker1, mixProp1);

			// Validating results - area
			ASSERT_EQUAL(1000, mc_11_23->getLocus(0)->getAllele(0)->getPeakArea());
			ASSERT_EQUAL(1000, mc_11_23->getLocus(0)->getAllele(1)->getPeakArea());
			ASSERT_EQUAL(1000, mc_11_23->getLocus(1)->getAllele(0)->getPeakArea());
			ASSERT_EQUAL(1000, mc_11_23->getLocus(1)->getAllele(1)->getPeakArea());

			// Validating results - height
			ASSERT_EQUAL(100, mc_11_23->getLocus(0)->getAllele(0)->getPeakHeight());
			ASSERT_EQUAL(100, mc_11_23->getLocus(0)->getAllele(1)->getPeakHeight());
			ASSERT_EQUAL(100, mc_11_23->getLocus(1)->getAllele(0)->getPeakHeight());
			ASSERT_EQUAL(100, mc_11_23->getLocus(1)->getAllele(1)->getPeakHeight());
		}

		void TestPeakAllotter::test_2P_4A_HE_1_1()
		{

			// Generating mixture data
			NumericAllelePtr na1(new NumericAllele(1.0, 0.0, 150.0, 1500.0));
			NumericAllelePtr na2(new NumericAllele(2.0, 0.0, 150.0, 1500.0));
			NumericAllelePtr na3(new NumericAllele(3.0, 0.0, 100.0, 1000.0));
			NumericAllelePtr na4(new NumericAllele(4.0, 0.0, 100.0, 1000.0));
			MarkerPtr mixMarker1(new Marker("MarkerMixture"));
			mixMarker1->addAllele(na1);
			mixMarker1->addAllele(na2);
			mixMarker1->addAllele(na3);
			mixMarker1->addAllele(na4);

			// Generating contributor genotypes and configurations
			LocusPtr locus_1_2(new Locus(na_c_1->cloneToPtr(), na_c_2->cloneToPtr()));
			LocusPtr locus_3_4(new Locus(na_c_3->cloneToPtr(), na_c_4->cloneToPtr()));

			MarkerConfigurationPtr mc_12_34(new MarkerConfiguration("MARKER_A"));
			mc_12_34->addLocus(locus_1_2);
			mc_12_34->addLocus(locus_3_4);

			// Generating mixture proportion
			MixtureProportionPtr mixProp1(new MixtureProportion());
			mixProp1->addContributor(1);
			mixProp1->addContributor(1);

			// Performing deconvolution
			performDeconvolution(mc_12_34, mixMarker1, mixProp1);

			// Validating results - area
			ASSERT_EQUAL(1500, mc_12_34->getLocus(0)->getAllele(0)->getPeakArea());
			ASSERT_EQUAL(1500, mc_12_34->getLocus(0)->getAllele(1)->getPeakArea());
			ASSERT_EQUAL(1000, mc_12_34->getLocus(1)->getAllele(0)->getPeakArea());
			ASSERT_EQUAL(1000, mc_12_34->getLocus(1)->getAllele(1)->getPeakArea());

			// Validating results - height
			ASSERT_EQUAL(150, mc_12_34->getLocus(0)->getAllele(0)->getPeakHeight());
			ASSERT_EQUAL(150, mc_12_34->getLocus(0)->getAllele(1)->getPeakHeight());
			ASSERT_EQUAL(100, mc_12_34->getLocus(1)->getAllele(0)->getPeakHeight());
			ASSERT_EQUAL(100, mc_12_34->getLocus(1)->getAllele(1)->getPeakHeight());
		}

		void TestPeakAllotter::test_3P_3A_HE_1_1_1()
		{

			// Generating mixture data
			NumericAllelePtr na1(new NumericAllele(1.0, 0.0, 100.0, 1000.0));
			NumericAllelePtr na2(new NumericAllele(2.0, 0.0, 300.0, 3000.0));
			NumericAllelePtr na3(new NumericAllele(3.0, 0.0, 200.0, 2000.0));
			MarkerPtr mixMarker1(new Marker("MarkerMixture"));
			mixMarker1->addAllele(na1);
			mixMarker1->addAllele(na2);
			mixMarker1->addAllele(na3);

			// Generating contributor genotypes and configurations
			LocusPtr locus_1_2(new Locus(na_c_1->cloneToPtr(), na_c_2->cloneToPtr()));
			LocusPtr locus_2_3(new Locus(na_c_2->cloneToPtr(), na_c_3->cloneToPtr()));

			MarkerConfigurationPtr mc_12_23_23(new MarkerConfiguration("MARKER_A"));
			mc_12_23_23->addLocus(locus_1_2);
			mc_12_23_23->addLocus(locus_2_3->cloneToSmartPtr());
			mc_12_23_23->addLocus(locus_2_3->cloneToSmartPtr());

			// Generating mixture proportion
			MixtureProportionPtr mixProp1(new MixtureProportion());
			mixProp1->addContributor(1);
			mixProp1->addContributor(1);
			mixProp1->addContributor(1);

			// Performing deconvolution
			performDeconvolution(mc_12_23_23, mixMarker1, mixProp1);

			// Validating results - area
			ASSERT_EQUAL(1000, mc_12_23_23->getLocus(0)->getAllele(0)->getPeakArea());
			ASSERT_EQUAL(1000, mc_12_23_23->getLocus(0)->getAllele(1)->getPeakArea());
			ASSERT_EQUAL(1000, mc_12_23_23->getLocus(1)->getAllele(0)->getPeakArea());
			ASSERT_EQUAL(1000, mc_12_23_23->getLocus(1)->getAllele(1)->getPeakArea());
			ASSERT_EQUAL(1000, mc_12_23_23->getLocus(2)->getAllele(0)->getPeakArea());
			ASSERT_EQUAL(1000, mc_12_23_23->getLocus(2)->getAllele(1)->getPeakArea());

			// Validating results - height
			ASSERT_EQUAL(100, mc_12_23_23->getLocus(0)->getAllele(0)->getPeakHeight());
			ASSERT_EQUAL(100, mc_12_23_23->getLocus(0)->getAllele(1)->getPeakHeight());
			ASSERT_EQUAL(100, mc_12_23_23->getLocus(1)->getAllele(0)->getPeakHeight());
			ASSERT_EQUAL(100, mc_12_23_23->getLocus(1)->getAllele(1)->getPeakHeight());
			ASSERT_EQUAL(100, mc_12_23_23->getLocus(2)->getAllele(0)->getPeakHeight());
			ASSERT_EQUAL(100, mc_12_23_23->getLocus(2)->getAllele(1)->getPeakHeight());
		}

		void TestPeakAllotter::test_3P_3A_HO_1_1_1()
		{

			// Generating mixture data
			NumericAllelePtr na1(new NumericAllele(1.0, 0.0, 200.0, 2000.0));
			NumericAllelePtr na2(new NumericAllele(2.0, 0.0, 300.0, 3000.0));
			NumericAllelePtr na3(new NumericAllele(3.0, 0.0, 100.0, 1000.0));
			MarkerPtr mixMarker1(new Marker("MarkerMixture"));
			mixMarker1->addAllele(na1);
			mixMarker1->addAllele(na2);
			mixMarker1->addAllele(na3);

			// Generating contributor genotypes and configurations
			LocusPtr locus_1_2(new Locus(na_c_1->cloneToPtr(), na_c_2->cloneToPtr()));
			LocusPtr locus_2_2(new Locus(na_c_2->cloneToPtr(), na_c_2->cloneToPtr()));
			LocusPtr locus_1_3(new Locus(na_c_1->cloneToPtr(), na_c_3->cloneToPtr()));

			MarkerConfigurationPtr mc_12_22_13(new MarkerConfiguration("MARKER_A"));
			mc_12_22_13->addLocus(locus_1_2);
			mc_12_22_13->addLocus(locus_2_2);
			mc_12_22_13->addLocus(locus_1_3);

			// Generating mixture proportion
			MixtureProportionPtr mixProp1(new MixtureProportion());
			mixProp1->addContributor(1);
			mixProp1->addContributor(1);
			mixProp1->addContributor(1);

			// Performing deconvolution
			performDeconvolution(mc_12_22_13, mixMarker1, mixProp1);

			// Validating results - area
			ASSERT_EQUAL(1000, mc_12_22_13->getLocus(0)->getAllele(0)->getPeakArea());
			ASSERT_EQUAL(1000, mc_12_22_13->getLocus(0)->getAllele(1)->getPeakArea());
			ASSERT_EQUAL(1000, mc_12_22_13->getLocus(1)->getAllele(0)->getPeakArea());
			ASSERT_EQUAL(1000, mc_12_22_13->getLocus(1)->getAllele(1)->getPeakArea());
			ASSERT_EQUAL(1000, mc_12_22_13->getLocus(2)->getAllele(0)->getPeakArea());
			ASSERT_EQUAL(1000, mc_12_22_13->getLocus(2)->getAllele(1)->getPeakArea());

			// Validating results - height
			ASSERT_EQUAL(100, mc_12_22_13->getLocus(0)->getAllele(0)->getPeakHeight());
			ASSERT_EQUAL(100, mc_12_22_13->getLocus(0)->getAllele(1)->getPeakHeight());
			ASSERT_EQUAL(100, mc_12_22_13->getLocus(1)->getAllele(0)->getPeakHeight());
			ASSERT_EQUAL(100, mc_12_22_13->getLocus(1)->getAllele(1)->getPeakHeight());
			ASSERT_EQUAL(100, mc_12_22_13->getLocus(2)->getAllele(0)->getPeakHeight());
			ASSERT_EQUAL(100, mc_12_22_13->getLocus(2)->getAllele(1)->getPeakHeight());
		}

		void TestPeakAllotter::test_2P_3A_HE_2_1()
		{

			// Generating mixture data
			NumericAllelePtr na1(new NumericAllele(1.0, 0.0, 200.0, 2000.0));
			NumericAllelePtr na2(new NumericAllele(2.0, 0.0, 300.0, 3000.0));
			NumericAllelePtr na3(new NumericAllele(3.0, 0.0, 100.0, 1000.0));
			MarkerPtr mixMarker1(new Marker("MarkerMixture"));
			mixMarker1->addAllele(na1);
			mixMarker1->addAllele(na2);
			mixMarker1->addAllele(na3);

			// Generating contributor genotypes and configurations
			LocusPtr locus_1_2(new Locus(na_c_1->cloneToPtr(), na_c_2->cloneToPtr()));
			LocusPtr locus_2_3(new Locus(na_c_2->cloneToPtr(), na_c_3->cloneToPtr()));

			MarkerConfigurationPtr mc_12_23(new MarkerConfiguration("MARKER_A"));
			mc_12_23->addLocus(locus_1_2);
			mc_12_23->addLocus(locus_2_3);

			// Generating mixture proportion
			MixtureProportionPtr mixProp1(new MixtureProportion());
			mixProp1->addContributor(2);
			mixProp1->addContributor(1);

			// Performing deconvolution
			performDeconvolution(mc_12_23, mixMarker1, mixProp1);

			// Validating results - area
			ASSERT_EQUAL(2000, mc_12_23->getLocus(0)->getAllele(0)->getPeakArea());
			ASSERT_EQUAL(2000, mc_12_23->getLocus(0)->getAllele(1)->getPeakArea());
			ASSERT_EQUAL(1000, mc_12_23->getLocus(1)->getAllele(0)->getPeakArea());
			ASSERT_EQUAL(1000, mc_12_23->getLocus(1)->getAllele(1)->getPeakArea());

			// Validating results - height
			ASSERT_EQUAL(200, mc_12_23->getLocus(0)->getAllele(0)->getPeakHeight());
			ASSERT_EQUAL(200, mc_12_23->getLocus(0)->getAllele(1)->getPeakHeight());
			ASSERT_EQUAL(100, mc_12_23->getLocus(1)->getAllele(0)->getPeakHeight());
			ASSERT_EQUAL(100, mc_12_23->getLocus(1)->getAllele(1)->getPeakHeight());
		}

		void TestPeakAllotter::test_2P_3A_HO_2_1()
		{

			// Generating mixture data
			NumericAllelePtr na1(new NumericAllele(1.0, 0.0, 400.0, 4000.0));
			NumericAllelePtr na2(new NumericAllele(2.0, 0.0, 100.0, 1000.0));
			NumericAllelePtr na3(new NumericAllele(3.0, 0.0, 100.0, 1000.0));
			MarkerPtr mixMarker1(new Marker("MarkerMixture"));
			mixMarker1->addAllele(na1);
			mixMarker1->addAllele(na2);
			mixMarker1->addAllele(na3);

			// Generating contributor genotypes and configurations
			LocusPtr locus_1_1(new Locus(na_c_1->cloneToPtr(), na_c_1->cloneToPtr()));
			LocusPtr locus_2_3(new Locus(na_c_2->cloneToPtr(), na_c_3->cloneToPtr()));

			MarkerConfigurationPtr mc_11_23(new MarkerConfiguration("MARKER_A"));
			mc_11_23->addLocus(locus_1_1);
			mc_11_23->addLocus(locus_2_3);

			// Generating mixture proportion
			MixtureProportionPtr mixProp1(new MixtureProportion());
			mixProp1->addContributor(2);
			mixProp1->addContributor(1);

			// Performing deconvolution
			performDeconvolution(mc_11_23, mixMarker1, mixProp1);

			// Validating results - area
			ASSERT_EQUAL(2000, mc_11_23->getLocus(0)->getAllele(0)->getPeakArea());
			ASSERT_EQUAL(2000, mc_11_23->getLocus(0)->getAllele(1)->getPeakArea());
			ASSERT_EQUAL(1000, mc_11_23->getLocus(1)->getAllele(0)->getPeakArea());
			ASSERT_EQUAL(1000, mc_11_23->getLocus(1)->getAllele(1)->getPeakArea());

			// Validating results - height
			ASSERT_EQUAL(200, mc_11_23->getLocus(0)->getAllele(0)->getPeakHeight());
			ASSERT_EQUAL(200, mc_11_23->getLocus(0)->getAllele(1)->getPeakHeight());
			ASSERT_EQUAL(100, mc_11_23->getLocus(1)->getAllele(0)->getPeakHeight());
			ASSERT_EQUAL(100, mc_11_23->getLocus(1)->getAllele(1)->getPeakHeight());
		}

		void TestPeakAllotter::test_2P_4A_HE_2_1()
		{

			// Generating mixture data
			NumericAllelePtr na1(new NumericAllele(1.0, 0.0, 200.0, 2000.0));
			NumericAllelePtr na2(new NumericAllele(2.0, 0.0, 200.0, 2000.0));
			NumericAllelePtr na3(new NumericAllele(3.0, 0.0, 100.0, 1000.0));
			NumericAllelePtr na4(new NumericAllele(4.0, 0.0, 100.0, 1000.0));
			MarkerPtr mixMarker1(new Marker("MarkerMixture"));
			mixMarker1->addAllele(na1);
			mixMarker1->addAllele(na2);
			mixMarker1->addAllele(na3);
			mixMarker1->addAllele(na4);

			// Generating contributor genotypes and configurations
			LocusPtr locus_1_2(new Locus(na_c_1->cloneToPtr(), na_c_2->cloneToPtr()));
			LocusPtr locus_3_4(new Locus(na_c_3->cloneToPtr(), na_c_4->cloneToPtr()));

			MarkerConfigurationPtr mc_12_34(new MarkerConfiguration("MARKER_A"));
			mc_12_34->addLocus(locus_1_2);
			mc_12_34->addLocus(locus_3_4);

			// Generating mixture proportion
			MixtureProportionPtr mixProp1(new MixtureProportion());
			mixProp1->addContributor(2);
			mixProp1->addContributor(1);

			// Performing deconvolution
			performDeconvolution(mc_12_34, mixMarker1, mixProp1);

			// Validating results - area
			ASSERT_EQUAL(2000, mc_12_34->getLocus(0)->getAllele(0)->getPeakArea());
			ASSERT_EQUAL(2000, mc_12_34->getLocus(0)->getAllele(1)->getPeakArea());
			ASSERT_EQUAL(1000, mc_12_34->getLocus(1)->getAllele(0)->getPeakArea());
			ASSERT_EQUAL(1000, mc_12_34->getLocus(1)->getAllele(1)->getPeakArea());

			// Validating results - height
			ASSERT_EQUAL(200, mc_12_34->getLocus(0)->getAllele(0)->getPeakHeight());
			ASSERT_EQUAL(200, mc_12_34->getLocus(0)->getAllele(1)->getPeakHeight());
			ASSERT_EQUAL(100, mc_12_34->getLocus(1)->getAllele(0)->getPeakHeight());
			ASSERT_EQUAL(100, mc_12_34->getLocus(1)->getAllele(1)->getPeakHeight());
		}

		void TestPeakAllotter::test_3P_3A_HE_1_3_2()
		{

			// Generating mixture data
			NumericAllelePtr na1(new NumericAllele(1.0, 0.0, 100.0, 1000.0));
			NumericAllelePtr na2(new NumericAllele(2.0, 0.0, 600.0, 6000.0));
			NumericAllelePtr na3(new NumericAllele(3.0, 0.0, 500.0, 5000.0));
			MarkerPtr mixMarker1(new Marker("MarkerMixture"));
			mixMarker1->addAllele(na1);
			mixMarker1->addAllele(na2);
			mixMarker1->addAllele(na3);

			// Generating contributor genotypes and configurations
			LocusPtr locus_1_2(new Locus(na_c_1->cloneToPtr(), na_c_2->cloneToPtr()));
			LocusPtr locus_2_3(new Locus(na_c_2->cloneToPtr(), na_c_3->cloneToPtr()));

			MarkerConfigurationPtr mc_12_23_23(new MarkerConfiguration("MARKER_A"));
			mc_12_23_23->addLocus(locus_1_2);
			mc_12_23_23->addLocus(locus_2_3->cloneToSmartPtr());
			mc_12_23_23->addLocus(locus_2_3->cloneToSmartPtr());

			// Generating mixture proportion
			MixtureProportionPtr mixProp1(new MixtureProportion());
			mixProp1->addContributor(1);
			mixProp1->addContributor(3);
			mixProp1->addContributor(2);

			// Performing deconvolution
			performDeconvolution(mc_12_23_23, mixMarker1, mixProp1);

			// Validating results - area
			ASSERT_EQUAL(1000, mc_12_23_23->getLocus(0)->getAllele(0)->getPeakArea());
			ASSERT_EQUAL(1000, mc_12_23_23->getLocus(0)->getAllele(1)->getPeakArea());
			ASSERT_EQUAL(3000, mc_12_23_23->getLocus(1)->getAllele(0)->getPeakArea());
			ASSERT_EQUAL(3000, mc_12_23_23->getLocus(1)->getAllele(1)->getPeakArea());
			ASSERT_EQUAL(2000, mc_12_23_23->getLocus(2)->getAllele(0)->getPeakArea());
			ASSERT_EQUAL(2000, mc_12_23_23->getLocus(2)->getAllele(1)->getPeakArea());

			// Validating results - height
			ASSERT_EQUAL(100, mc_12_23_23->getLocus(0)->getAllele(0)->getPeakHeight());
			ASSERT_EQUAL(100, mc_12_23_23->getLocus(0)->getAllele(1)->getPeakHeight());
			ASSERT_EQUAL(300, mc_12_23_23->getLocus(1)->getAllele(0)->getPeakHeight());
			ASSERT_EQUAL(300, mc_12_23_23->getLocus(1)->getAllele(1)->getPeakHeight());
			ASSERT_EQUAL(200, mc_12_23_23->getLocus(2)->getAllele(0)->getPeakHeight());
			ASSERT_EQUAL(200, mc_12_23_23->getLocus(2)->getAllele(1)->getPeakHeight());

		}
		void TestPeakAllotter::test_3P_3A_HO_1_3_2()
		{

			// Generating mixture data
			NumericAllelePtr na1(new NumericAllele(1.0, 0.0, 300.0, 3000.0));
			NumericAllelePtr na2(new NumericAllele(2.0, 0.0, 700.0, 7000.0));
			NumericAllelePtr na3(new NumericAllele(3.0, 0.0, 200.0, 2000.0));
			MarkerPtr mixMarker1(new Marker("MarkerMixture"));
			mixMarker1->addAllele(na1);
			mixMarker1->addAllele(na2);
			mixMarker1->addAllele(na3);

			// Generating contributor genotypes and configurations
			LocusPtr locus_1_2(new Locus(na_c_1->cloneToPtr(), na_c_2->cloneToPtr()));
			LocusPtr locus_2_2(new Locus(na_c_2->cloneToPtr(), na_c_2->cloneToPtr()));
			LocusPtr locus_1_3(new Locus(na_c_1->cloneToPtr(), na_c_3->cloneToPtr()));

			MarkerConfigurationPtr mc_12_22_13(new MarkerConfiguration("MARKER_A"));
			mc_12_22_13->addLocus(locus_1_2);
			mc_12_22_13->addLocus(locus_2_2);
			mc_12_22_13->addLocus(locus_1_3);

			// Generating mixture proportion
			MixtureProportionPtr mixProp1(new MixtureProportion());
			mixProp1->addContributor(1);
			mixProp1->addContributor(3);
			mixProp1->addContributor(2);

			// Performing deconvolution
			performDeconvolution(mc_12_22_13, mixMarker1, mixProp1);

			// Validating results - area
			ASSERT_EQUAL(1000, mc_12_22_13->getLocus(0)->getAllele(0)->getPeakArea());
			ASSERT_EQUAL(1000, mc_12_22_13->getLocus(0)->getAllele(1)->getPeakArea());
			ASSERT_EQUAL(3000, mc_12_22_13->getLocus(1)->getAllele(0)->getPeakArea());
			ASSERT_EQUAL(3000, mc_12_22_13->getLocus(1)->getAllele(1)->getPeakArea());
			ASSERT_EQUAL(2000, mc_12_22_13->getLocus(2)->getAllele(0)->getPeakArea());
			ASSERT_EQUAL(2000, mc_12_22_13->getLocus(2)->getAllele(1)->getPeakArea());

			// Validating results - height
			ASSERT_EQUAL(100, mc_12_22_13->getLocus(0)->getAllele(0)->getPeakHeight());
			ASSERT_EQUAL(100, mc_12_22_13->getLocus(0)->getAllele(1)->getPeakHeight());
			ASSERT_EQUAL(300, mc_12_22_13->getLocus(1)->getAllele(0)->getPeakHeight());
			ASSERT_EQUAL(300, mc_12_22_13->getLocus(1)->getAllele(1)->getPeakHeight());
			ASSERT_EQUAL(200, mc_12_22_13->getLocus(2)->getAllele(0)->getPeakHeight());
			ASSERT_EQUAL(200, mc_12_22_13->getLocus(2)->getAllele(1)->getPeakHeight());
		}

	/*
	 void TestPeakAllotter::test_MixSample_118_353_1a_vWA_14_18_15_17()
	 {

	 // Sample Name	Panel	Marker	Allele 1	Allele 2	Allele 3	Allele 4	Size 1	Size 2	Size 3	Size 4	Height 1	Height 2	Height 3	Height 4	Peak Area 1	Peak Area 2	Peak Area 3	Peak Area 4
	 // 118_353_1a	SGM_Plus_v2	vWA	14	15	17	18	166.94	170.95	178.99	182.94	539	1273	1013	342	6506	15605	12607	4236

	 // Mixture marker
	 NumericAllelePtr ma1(new NumericAllele(14.0, 0.0, 539, 6506.0));
	 NumericAllelePtr ma2(new NumericAllele(15.0, 0.0, 1273, 15605.0));
	 NumericAllelePtr ma3(new NumericAllele(17.0, 0.0, 1013, 12607.0));
	 NumericAllelePtr ma4(new NumericAllele(18.0, 0.0, 342, 4236.0));
	 MarkerPtr mixMarker1(new Marker("MixtureMarker"));
	 mixMarker1->addAllele(ma1);
	 mixMarker1->addAllele(ma2);
	 mixMarker1->addAllele(ma3);
	 mixMarker1->addAllele(ma4);

	 // Creating contributors genotype configuration
	 LocusPtr lcp_1_14_18(new Locus());
	 LocusPtr lcp_2_15_16(new Locus());
	 NumericAllelePtr cgca1_14(new NumericAllele(14.0));
	 NumericAllelePtr cgca1_18(new NumericAllele(18.0));
	 NumericAllelePtr cgca2_15(new NumericAllele(15.0));
	 NumericAllelePtr cgca2_16(new NumericAllele(17.0));
	 lcp_1_14_18->addAllele(cgca1_14);
	 lcp_1_14_18->addAllele(cgca1_18);
	 lcp_2_15_16->addAllele(cgca2_15);
	 lcp_2_15_16->addAllele(cgca2_16);

	 MarkerConfigurationPtr mc14_18__15_16(new MarkerConfiguration("MARKER_A"));
	 mc14_18__15_16->addLocus(lcp_1_14_18);
	 mc14_18__15_16->addLocus(lcp_2_15_16);

	 // Creating mixture proportion
	 MixtureProportionPtr mixProp1(new MixtureProportion());
	 mixProp1->addContributor(1);
	 mixProp1->addContributor(1);

	 // Performing deconvolution
	 performDeconvolution(mc14_18__15_16, mixMarker1, mixProp1);

	 // Testing
	 // Sample Name	Panel	Marker	Allele 1	Allele 2	Allele 3	Allele 4	Size 1	Size 2	Size 3	Size 4	Height 1	Height 2	Height 3	Height 4	Peak Area 1	Peak Area 2	Peak Area 3	Peak Area 4
	 // 118_353_1a	SGM_Plus_v2	vWA	14	15	17	18	166.94	170.95	178.99	182.94	539	1273	1013	342	6506	15605	12607	4236
	 /*
	 * Expected results are
	 *
	 * Contributor 1:
	 * Allele 1: 1500 RFU
	 * Allele 2: 1500 RFU
	 *
	 * Contributor 2:
	 * Allele 1: 1000 RFU
	 * Allele 2: 1000 RFU
	 */

	/*

	 ASSERT_EQUAL(6506, mc14_18__15_16->getLocus(0)->getAllele(0)->getPeakArea());
	 ASSERT_EQUAL(4236, mc14_18__15_16->getLocus(0)->getAllele(1)->getPeakArea());
	 ASSERT_EQUAL(15605, mc14_18__15_16->getLocus(1)->getAllele(0)->getPeakArea());
	 ASSERT_EQUAL(12607, mc14_18__15_16->getLocus(1)->getAllele(1)->getPeakArea());
	 }
	 */
	}
}
