/*
 * TestPeakRestrictionFilter.cpp
 *
 *  Created on: Jul 19, 2009
 *      Author: mariusz
 */

#include "TestPeakRestrictionFilter.h"

namespace dma
{

	namespace test
	{

		TestPeakRestrictionFilter::TestPeakRestrictionFilter()
		{

		}

		TestPeakRestrictionFilter::~TestPeakRestrictionFilter()
		{

		}

		void TestPeakRestrictionFilter::testFilterByHb()
		{

			// Generating HB minimas
			HbMapPtr hbMin(new HbMap());
			hbMin->insert(std::make_pair("MarkerA", 0.7));

			// Generating genotypes and configurations
			NumericAllelePtr na10(new NumericAllele(10, 0, 80, 800));
			NumericAllelePtr na11(new NumericAllele(11, 0, 100, 1000));
			NumericAllelePtr na12(new NumericAllele(12, 0, 75, 750));
			NumericAllelePtr na13(new NumericAllele(13, 0, 100, 1000));
			NumericAllelePtr na14(new NumericAllele(14, 0, 60, 600));
			NumericAllelePtr na15(new NumericAllele(15, 0, 100, 1000));
			LocusPtr locus_10_11(new Locus(na10->cloneToPtr(), na11->cloneToPtr()));
			LocusPtr locus_12_13(new Locus(na12->cloneToPtr(), na13->cloneToPtr()));
			LocusPtr locus_14_15(new Locus(na14->cloneToPtr(), na15->cloneToPtr()));

			MarkerConfigurationPtr mc_10_11__12_13(new MarkerConfiguration("MarkerA"));
			mc_10_11__12_13->addLocus(locus_10_11);
			mc_10_11__12_13->addLocus(locus_12_13);
			MarkerConfigurationPtr mc_10_11__14_15(new MarkerConfiguration("MarkerA"));
			mc_10_11__14_15->addLocus(locus_10_11);
			mc_10_11__14_15->addLocus(locus_14_15);
			MarkerConfigurationPtr mc_12_13__10_11(new MarkerConfiguration("MarkerA"));
			mc_12_13__10_11->addLocus(locus_12_13);
			mc_12_13__10_11->addLocus(locus_10_11);
			MarkerConfigurationPtrVectorPtr configurations(new MarkerConfigurationPtrVector());
			configurations->push_back(mc_10_11__12_13);
			configurations->push_back(mc_10_11__14_15);
			configurations->push_back(mc_12_13__10_11);

			ASSERT_EQUAL( 3, configurations->size() );
			ASSERT_EQUAL( MCStatusValid::getStatusName() , configurations->at(0)->getStatus()->getName() );
			ASSERT_EQUAL( MCStatusValid::getStatusName() , configurations->at(1)->getStatus()->getName() );
			ASSERT_EQUAL( MCStatusValid::getStatusName() , configurations->at(2)->getStatus()->getName() );

			// Cloning configurations
			MarkerConfigurationPtrVectorPtr configurations1 = MarkerConfiguration::cloneMarkerConfigurations(configurations);
			MarkerConfigurationPtrVectorPtr configurations2 = MarkerConfiguration::cloneMarkerConfigurations(configurations);

			// Creating tested class instance
			StrategiesPtr strategies(new Strategies());
			PeakRestrictionFilterPtr filter(new PeakRestrictionFilter(strategies, hbMin));

			// Testing filtering - assign status
			filter->getStrategies()->setExcludedConfigurationsStrategy(Strategies::ECS_ASSIGN_STATUS);
			configurations1 = filter->filterByHb("MarkerA", configurations1);
			ASSERT_EQUAL( 3, configurations1->size() );
			ASSERT_EQUAL( MCStatusValid::getStatusName() , configurations1->at(0)->getStatus()->getName() );
			ASSERT_EQUAL( MCStatusExcHeterozygoticImbalance::getStatusName() , configurations1->at(1)->getStatus()->getName() );
			ASSERT_EQUAL( MCStatusValid::getStatusName() , configurations1->at(2)->getStatus()->getName() );

			// Testing filtering - exclude
			filter->getStrategies()->setExcludedConfigurationsStrategy(Strategies::ECS_EXCLUDE_FROM_COMBINATIONS);
			configurations2 = filter->filterByHb("MarkerA", configurations2);
			ASSERT_EQUAL( 2, configurations2->size() );
			ASSERT_EQUAL( MCStatusValid::getStatusName() , configurations2->at(0)->getStatus()->getName() );
			ASSERT_EQUAL( MCStatusValid::getStatusName() , configurations2->at(1)->getStatus()->getName() );
		}

		void TestPeakRestrictionFilter::testFilterByMp()
		{
			/*
			 * 	Allele 1 value	Allele 1 area	Allele 2 value	Allele 2 area	Allele 1 value	Allele 1 area	Allele 2 value	Allele 2 area
			 10	1000	11	1000	12	2000	13	2000
			 10	1000	11	1000	14	3000	15	3000
			 10	1000	11	1000	16	300	17	300
			 10	1000	11	1000	18	1000	19	1000
			 */

			// Generating mixture proportion container
			MixtureProportionPtr mp2(new MixtureProportion());
			mp2->addContributor(0.5);
			mp2->addContributor(0.5);
			MixProportionPtrMapPtr mpMap(new MixProportionPtrMap());
			mpMap->insert(std::make_pair(2, mp2));

			// Setting maximum deviation for mixture proportion
			double maxMpDev = 0.2;

			// Generating genotypes and configurations
			NumericAllelePtr na10(new NumericAllele(10, 0, 100, 1000));
			NumericAllelePtr na11(new NumericAllele(11, 0, 100, 1000));
			NumericAllelePtr na12(new NumericAllele(12, 0, 200, 2000));
			NumericAllelePtr na13(new NumericAllele(13, 0, 200, 1000));
			NumericAllelePtr na14(new NumericAllele(14, 0, 300, 3000));
			NumericAllelePtr na15(new NumericAllele(15, 0, 300, 3000));
			NumericAllelePtr na16(new NumericAllele(14, 0, 30, 300));
			NumericAllelePtr na17(new NumericAllele(15, 0, 30, 300));
			NumericAllelePtr na18(new NumericAllele(18, 0, 100, 1000));
			NumericAllelePtr na19(new NumericAllele(19, 0, 100, 1000));
			LocusPtr locus_10_11(new Locus(na10->cloneToPtr(), na11->cloneToPtr()));
			LocusPtr locus_12_13(new Locus(na12->cloneToPtr(), na13->cloneToPtr()));
			LocusPtr locus_14_15(new Locus(na14->cloneToPtr(), na15->cloneToPtr()));
			LocusPtr locus_16_17(new Locus(na16->cloneToPtr(), na17->cloneToPtr()));
			LocusPtr locus_18_19(new Locus(na18->cloneToPtr(), na19->cloneToPtr()));

			MarkerConfigurationPtr mc_10_11__12_13(new MarkerConfiguration("MarkerA"));
			MarkerConfigurationPtr mc_10_11__14_15(new MarkerConfiguration("MarkerA"));
			MarkerConfigurationPtr mc_10_11__16_17(new MarkerConfiguration("MarkerA"));
			MarkerConfigurationPtr mc_10_11__18_19(new MarkerConfiguration("MarkerA"));
			mc_10_11__12_13->addLocus(locus_10_11);
			mc_10_11__12_13->addLocus(locus_12_13);
			mc_10_11__14_15->addLocus(locus_10_11);
			mc_10_11__14_15->addLocus(locus_14_15);
			mc_10_11__16_17->addLocus(locus_10_11);
			mc_10_11__16_17->addLocus(locus_16_17);
			mc_10_11__18_19->addLocus(locus_10_11);
			mc_10_11__18_19->addLocus(locus_18_19);

			MarkerConfigurationPtrVectorPtr configurations(new MarkerConfigurationPtrVector());
			configurations->push_back(mc_10_11__12_13);
			configurations->push_back(mc_10_11__14_15);
			configurations->push_back(mc_10_11__16_17);
			configurations->push_back(mc_10_11__18_19);

			// Performing check on configuration prior to filtering
			ASSERT_EQUAL( 4, configurations->size() );
			ASSERT_EQUAL( MCStatusValid::getStatusName() , configurations->at(0)->getStatus()->getName() );
			ASSERT_EQUAL( MCStatusValid::getStatusName() , configurations->at(1)->getStatus()->getName() );
			ASSERT_EQUAL( MCStatusValid::getStatusName() , configurations->at(2)->getStatus()->getName() );
			ASSERT_EQUAL( MCStatusValid::getStatusName() , configurations->at(3)->getStatus()->getName() );

			// Cloning configurations
			MarkerConfigurationPtrVectorPtr configurations1 = MarkerConfiguration::cloneMarkerConfigurations(configurations);
			MarkerConfigurationPtrVectorPtr configurations2 = MarkerConfiguration::cloneMarkerConfigurations(configurations);

			// Creating tested class instance
			StrategiesPtr strategies(new Strategies());
			HbMapPtr hbMin(new HbMap());
			PeakRestrictionFilterPtr filter(new PeakRestrictionFilter(strategies, hbMin));

			// Testing filtering - assign status
			filter->getStrategies()->setExcludedConfigurationsStrategy(Strategies::ECS_ASSIGN_STATUS);
			configurations1 = filter->filterByMp(configurations1, mpMap, maxMpDev);
			ASSERT_EQUAL( 4, configurations1->size() );
			ASSERT_EQUAL( MCStatusValid::getStatusName() , configurations1->at(0)->getStatus()->getName() );
			ASSERT_EQUAL( MCStatusExcMixtureProportion::getStatusName() , configurations1->at(1)->getStatus()->getName() );
			ASSERT_EQUAL( MCStatusExcMixtureProportion::getStatusName() , configurations1->at(2)->getStatus()->getName() );
			ASSERT_EQUAL( MCStatusValid::getStatusName() , configurations1->at(3)->getStatus()->getName() );

			// Testing filtering - exclude
			filter->getStrategies()->setExcludedConfigurationsStrategy(Strategies::ECS_EXCLUDE_FROM_COMBINATIONS);
			configurations2 = filter->filterByMp(configurations2, mpMap, maxMpDev);
			ASSERT_EQUAL( 2, configurations2->size() );
			ASSERT_EQUAL( MCStatusValid::getStatusName() , configurations2->at(0)->getStatus()->getName() );
			ASSERT_EQUAL( MCStatusValid::getStatusName() , configurations2->at(1)->getStatus()->getName() );
		}

	}

}
