/*
 * TestPersonNumberAnalyzer.cpp
 *
 *  Created on: Jul 19, 2009
 *      Author: mariusz
 */

#include "TestPersonNumberAnalyzer.h"

namespace dma
{

	namespace test
	{

		TestPersonNumberAnalyzer::TestPersonNumberAnalyzer() :
			PersonNumberAnalyzer()
		{

			// Generating "population" data - heterozygotic balance minimas
			hbMin_->insert(std::make_pair("MarkerA", 0.9));
			hbMin_->insert(std::make_pair("MarkerB", 0.5));
			hbMin_->insert(std::make_pair("MarkerC", 0.5));
			hbMin_->insert(std::make_pair("MarkerD", 0.5));
		}

		TestPersonNumberAnalyzer::~TestPersonNumberAnalyzer()
		{

		}

		void TestPersonNumberAnalyzer::testGetMinimumForSample()
		{

			// Generating alleles
			NumericAllelePtr na10(new NumericAllele(10.0));
			NumericAllelePtr na11(new NumericAllele(11.0));
			NumericAllelePtr na12(new NumericAllele(12.0));
			NumericAllelePtr na13(new NumericAllele(13.0));
			NumericAllelePtr na14(new NumericAllele(14.0));

			// Generating markers
			MarkerPtr markerA(new Marker("MarkerA"));
			markerA->addAllele(na10->cloneToPtr());
			markerA->addAllele(na11->cloneToPtr());
			markerA->addAllele(na12->cloneToPtr());
			MarkerPtr markerB(new Marker("MarkerB"));
			markerB->addAllele(na10->cloneToPtr());
			markerB->addAllele(na11->cloneToPtr());
			markerB->addAllele(na12->cloneToPtr());
			markerB->addAllele(na13->cloneToPtr());
			markerB->addAllele(na14->cloneToPtr());

			// Generating samples
			SamplePtr sampleAB(new Sample("SampleAB", "Panel1"));
			sampleAB->addMarker(markerA);
			sampleAB->addMarker(markerB);
			SamplePtr sampleBA(new Sample("SampleBA", "Panel1"));
			sampleBA->addMarker(markerB);
			sampleBA->addMarker(markerA);

			// Testing
			ASSERT_EQUAL( 3, this->getMinimumForSample( sampleAB) );
			ASSERT_EQUAL( 3, this->getMinimumForSample( sampleBA) );
		}

		void TestPersonNumberAnalyzer::testGetMinimumForMarker()
		{

			// Generating data for case 1 - marker with 1 allele
			MarkerPtr markerA(new Marker("MarkerA"));
			NumericAllelePtr na_A_10(new NumericAllele(10.0, 0, 100, 1000));
			markerA->addAllele(na_A_10);

			// Generating data for case 2 - marker with 2 alleles ( meeting HB requirement )
			MarkerPtr markerB(new Marker("MarkerB"));
			NumericAllelePtr na_B_10(new NumericAllele(10.0, 0, 80, 800));
			NumericAllelePtr na_B_11(new NumericAllele(11.0, 0, 100, 1000));
			markerB->addAllele(na_B_10);
			markerB->addAllele(na_B_11);

			// Generating data for case 3 - marker with 2 alleles ( NOT meeting HB requirement )
			MarkerPtr markerC(new Marker("MarkerC"));
			NumericAllelePtr na_C_10(new NumericAllele(10.0, 0, 40, 400));
			NumericAllelePtr na_C_11(new NumericAllele(11.0, 0, 100, 1000));
			markerC->addAllele(na_C_10);
			markerC->addAllele(na_C_11);

			// Generating data for case 4 - marker with 3 alleles
			MarkerPtr markerD(new Marker("MarkerD"));
			NumericAllelePtr na_D_10(new NumericAllele(10.0, 0, 100, 1000));
			NumericAllelePtr na_D_11(new NumericAllele(11.0, 0, 100, 1000));
			NumericAllelePtr na_D_12(new NumericAllele(12.0, 0, 100, 1000));
			markerD->addAllele(na_D_10);
			markerD->addAllele(na_D_11);
			markerD->addAllele(na_D_12);

			// Testing
			ASSERT_EQUAL( 1, this->getMinimumForMarker( markerA ) );
			ASSERT_EQUAL( 1, this->getMinimumForMarker( markerB ) );
			ASSERT_EQUAL( 2, this->getMinimumForMarker( markerC ) );
			ASSERT_EQUAL( 2, this->getMinimumForMarker( markerD ) );
		}

		void TestPersonNumberAnalyzer::testGetHBmin()
		{

			ASSERT_EQUAL( 0.9, this->getHBMin("MarkerA") );
			ASSERT_EQUAL( 0.5, this->getHBMin("MarkerB") );
			ASSERT_THROWS( this->getHBMin("MarkerE"), MarkerNotFoundException );
		}

	}

}
