/*
 * TestPopulationDataAnalyzer.cpp
 *
 *  Created on: Feb 1, 2009
 *      Author: mariusz
 */

#include "TestPopulationDataAnalyzer.h"

namespace dma
{
	namespace test
	{

		TestPopulationDataAnalyzer::TestPopulationDataAnalyzer()
		{

		}

		TestPopulationDataAnalyzer::~TestPopulationDataAnalyzer()
		{

		}

		void TestPopulationDataAnalyzer::testVerifyPopulationData()
		{

			// Creating population data analyzer with generated data
			MarkerPtrVectorPtr populationData = generatePopulationData();
			PopulationDataAnalyzer analyzer(populationData);

			// Generating sample data and performing verification
			SampleConfigurationPtr sample = generateSampleConfiguration();
			MarkerPtrVectorPtr missingAlleles = analyzer.verifyPopulationData(sample);

			ASSERT_EQUAL( 3, missingAlleles->size() );
			ASSERT_EQUAL( "MarkerA", missingAlleles->at(0)->getName() );
			ASSERT_EQUAL( "MarkerB", missingAlleles->at(1)->getName() );
			ASSERT_EQUAL( "MarkerC", missingAlleles->at(2)->getName() );

			ASSERT_EQUAL( 2, missingAlleles->at(0)->getAllelesNumber() );
			ASSERT_EQUAL( 1, missingAlleles->at(1)->getAllelesNumber() );
			ASSERT_EQUAL( 2, missingAlleles->at(2)->getAllelesNumber() );

			ASSERT_EQUAL( 14.0, missingAlleles->at(0)->getAllele(0)->getAlleleValue() );
			ASSERT_EQUAL( 15.0, missingAlleles->at(0)->getAllele(1)->getAlleleValue() );
			ASSERT_EQUAL( 23.0, missingAlleles->at(1)->getAllele(0)->getAlleleValue() );
			ASSERT_EQUAL( 40.0, missingAlleles->at(2)->getAllele(0)->getAlleleValue() );
			ASSERT_EQUAL( 41.0, missingAlleles->at(2)->getAllele(1)->getAlleleValue() );

		}

		MarkerPtrVectorPtr TestPopulationDataAnalyzer::generatePopulationData()
		{

			// Generating alleles
			NumericAllelePtr na10(new NumericAllele(10.0));
			NumericAllelePtr na11(new NumericAllele(11.0));
			NumericAllelePtr na12(new NumericAllele(12.0));
			NumericAllelePtr na13(new NumericAllele(13.0));
			NumericAllelePtr na20(new NumericAllele(20.0));
			NumericAllelePtr na21(new NumericAllele(21.0));
			NumericAllelePtr na22(new NumericAllele(22.0));

			// Generating markers
			MarkerPtr markerA(new Marker("MarkerA"));
			MarkerPtr markerB(new Marker("MarkerB"));

			// Adding alleles to markers
			markerA->addAllele(na10);
			markerA->addAllele(na11);
			markerA->addAllele(na12);
			markerA->addAllele(na13);
			markerB->addAllele(na20);
			markerB->addAllele(na21);
			markerB->addAllele(na22);

			// Creating container
			MarkerPtrVectorPtr output(new MarkerPtrVector());
			output->push_back(markerA);
			output->push_back(markerB);

			return output;
		}

		SampleConfigurationPtr TestPopulationDataAnalyzer::generateSampleConfiguration()
		{

			// Generating alleles and loci
			NumericAllelePtr na10(new NumericAllele(10.0));
			NumericAllelePtr na11(new NumericAllele(11.0));
			NumericAllelePtr na12(new NumericAllele(12.0));
			NumericAllelePtr na13(new NumericAllele(13.0));
			NumericAllelePtr na14(new NumericAllele(14.0));
			NumericAllelePtr na15(new NumericAllele(15.0));

			NumericAllelePtr na20(new NumericAllele(20.0));
			NumericAllelePtr na21(new NumericAllele(21.0));
			NumericAllelePtr na22(new NumericAllele(22.0));
			NumericAllelePtr na23(new NumericAllele(23.0));

			NumericAllelePtr na30(new NumericAllele(40.0));
			NumericAllelePtr na31(new NumericAllele(41.0));

			// Generating loci
			LocusPtr locus_10_11(new Locus( na10->cloneToPtr(), na11->cloneToPtr() ));
			LocusPtr locus_12_13(new Locus( na12->cloneToPtr(), na13->cloneToPtr() ));
			LocusPtr locus_14_15(new Locus( na14->cloneToPtr(), na15->cloneToPtr() ));

			LocusPtr locus_20_23(new Locus( na20->cloneToPtr(), na23->cloneToPtr() ));
			LocusPtr locus_21_22(new Locus( na21->cloneToPtr(), na22->cloneToPtr() ));

			LocusPtr locus_40_41(new Locus( na30->cloneToPtr(), na31->cloneToPtr() ));

			// Generating marker configurations
			MarkerConfigurationPtr mc_A_10_11(new MarkerConfiguration("MarkerA"));
			mc_A_10_11->addLocus(locus_10_11);
			MarkerConfigurationPtr mc_A_12_13(new MarkerConfiguration("MarkerA"));
			mc_A_12_13->addLocus(locus_12_13);
			MarkerConfigurationPtr mc_A_14_15(new MarkerConfiguration("MarkerA"));
			mc_A_14_15->addLocus(locus_14_15);

			MarkerConfigurationPtr mc_B_20_23__21_22(new MarkerConfiguration("MarkerB"));
			mc_B_20_23__21_22->addLocus(locus_20_23);
			mc_B_20_23__21_22->addLocus(locus_21_22);

			MarkerConfigurationPtr mc_C_40_41(new MarkerConfiguration("MarkerC"));
			mc_C_40_41->addLocus(locus_40_41);

			// Creating sample
			SampleConfigurationPtr sample(new SampleConfiguration("SAMPLE_1"));
			sample->addMarkerConfiguration("MarkerA", mc_A_10_11);
			sample->addMarkerConfiguration("MarkerA", mc_A_12_13);
			sample->addMarkerConfiguration("MarkerA", mc_A_14_15);
			sample->addMarkerConfiguration("MarkerB", mc_B_20_23__21_22);
			sample->addMarkerConfiguration("MarkerC", mc_C_40_41);

			return sample;
		}
	}
}
