/*
 * TestProfilesFilter.cpp
 *
 *  Created on: Jul 21, 2009
 *      Author: mariusz
 */

#include "TestProfilesFilter.h"

namespace dma
{

	namespace test
	{

		TestProfilesFilter::TestProfilesFilter() :
			na10(new NumericAllele(10.0)), na11(new NumericAllele(11.0)), na12(new NumericAllele(12.0)),
					na13(new NumericAllele(13.0)), na14(new NumericAllele(14.0)), na15(new NumericAllele(15.0))
		{

		}

		TestProfilesFilter::~TestProfilesFilter()
		{

		}

		void TestProfilesFilter::testFilterSampleByKnownProfile()
		{

			// Generating loci
			LocusPtr locus_10_11(new Locus(na10->cloneToPtr(), na11->cloneToPtr()));
			LocusPtr locus_12_13(new Locus(na12->cloneToPtr(), na13->cloneToPtr()));
			LocusPtr locus_14_15(new Locus(na14->cloneToPtr(), na15->cloneToPtr()));

			// Generating known profile
			ProfilePtr profile1(new Profile());
			profile1->addMarker("MarkerA", locus_10_11->cloneToSmartPtr());
			profile1->addMarker("MarkerB", locus_14_15->cloneToSmartPtr());

			// Generating configurations - marker A
			MarkerConfigurationPtr mc_A_10_11__12_13(new MarkerConfiguration("MarkerA"));
			mc_A_10_11__12_13->addLocus(locus_10_11->cloneToSmartPtr());
			mc_A_10_11__12_13->addLocus(locus_12_13->cloneToSmartPtr());
			MarkerConfigurationPtr mc_A_10_11__14_15(new MarkerConfiguration("MarkerA"));
			mc_A_10_11__14_15->addLocus(locus_10_11->cloneToSmartPtr());
			mc_A_10_11__14_15->addLocus(locus_14_15->cloneToSmartPtr());
			MarkerConfigurationPtr mc_A_12_13__14_15(new MarkerConfiguration("MarkerA"));
			mc_A_12_13__14_15->addLocus(locus_12_13->cloneToSmartPtr());
			mc_A_12_13__14_15->addLocus(locus_14_15->cloneToSmartPtr());

			// Generating configurations - marker B
			MarkerConfigurationPtr mc_B_10_11__12_13(new MarkerConfiguration("MarkerA"));
			mc_B_10_11__12_13->addLocus(locus_10_11->cloneToSmartPtr());
			mc_B_10_11__12_13->addLocus(locus_12_13->cloneToSmartPtr());
			MarkerConfigurationPtr mc_B_10_11__14_15(new MarkerConfiguration("MarkerA"));
			mc_B_10_11__14_15->addLocus(locus_10_11->cloneToSmartPtr());
			mc_B_10_11__14_15->addLocus(locus_14_15->cloneToSmartPtr());
			MarkerConfigurationPtr mc_B_12_13__14_15(new MarkerConfiguration("MarkerA"));
			mc_B_12_13__14_15->addLocus(locus_12_13->cloneToSmartPtr());
			mc_B_12_13__14_15->addLocus(locus_14_15->cloneToSmartPtr());

			MarkerConfigurationPtrVectorPtr configurationsA(new MarkerConfigurationPtrVector());
			configurationsA->push_back(mc_A_10_11__12_13->cloneToSmartPtr());
			configurationsA->push_back(mc_A_10_11__14_15->cloneToSmartPtr());
			configurationsA->push_back(mc_A_12_13__14_15->cloneToSmartPtr());
			configurationsA->push_back(mc_A_10_11__12_13->cloneToSmartPtr());
			MarkerConfigurationPtrVectorPtr configurationsB(new MarkerConfigurationPtrVector());
			configurationsB->push_back(mc_B_10_11__12_13->cloneToSmartPtr());
			configurationsB->push_back(mc_B_10_11__14_15->cloneToSmartPtr());
			configurationsB->push_back(mc_B_12_13__14_15->cloneToSmartPtr());
			configurationsB->push_back(mc_B_10_11__12_13->cloneToSmartPtr());

			// Testing configurations before filtering
			ASSERT_EQUAL( 4, configurationsA->size() );
			ASSERT_EQUAL( 4, configurationsB->size() );
			ASSERT_EQUAL( MCStatusValid::getStatusName() , configurationsA->at(0)->getStatus()->getName() );
			ASSERT_EQUAL( MCStatusValid::getStatusName() , configurationsA->at(1)->getStatus()->getName() );
			ASSERT_EQUAL( MCStatusValid::getStatusName() , configurationsA->at(2)->getStatus()->getName() );
			ASSERT_EQUAL( MCStatusValid::getStatusName() , configurationsA->at(3)->getStatus()->getName() );
			ASSERT_EQUAL( MCStatusValid::getStatusName() , configurationsB->at(0)->getStatus()->getName() );
			ASSERT_EQUAL( MCStatusValid::getStatusName() , configurationsB->at(1)->getStatus()->getName() );
			ASSERT_EQUAL( MCStatusValid::getStatusName() , configurationsB->at(2)->getStatus()->getName() );
			ASSERT_EQUAL( MCStatusValid::getStatusName() , configurationsB->at(3)->getStatus()->getName() );

			// Cloning configurations
			MarkerConfigurationPtrVectorPtr configurationsA1 = MarkerConfiguration::cloneMarkerConfigurations(configurationsA);
			MarkerConfigurationPtrVectorPtr configurationsA2 = MarkerConfiguration::cloneMarkerConfigurations(configurationsA);
			MarkerConfigurationPtrVectorPtr configurationsB1 = MarkerConfiguration::cloneMarkerConfigurations(configurationsB);
			MarkerConfigurationPtrVectorPtr configurationsB2 = MarkerConfiguration::cloneMarkerConfigurations(configurationsB);

			// Creating samples
			SampleConfigurationPtr sc1(new SampleConfiguration("Sample1"));
			sc1->addMarkerConfigurations("MarkerA", configurationsA1);
			sc1->addMarkerConfigurations("MarkerB", configurationsB1);
			SampleConfigurationPtr sc2(new SampleConfiguration("Sample1"));
			sc2->addMarkerConfigurations("MarkerA", configurationsA2);
			sc2->addMarkerConfigurations("MarkerB", configurationsB2);

			// Creating tested class instance
			ProfilesFilterPtr pf(new ProfilesFilter());

			// Performing filtering - assigning status
			pf->getStrategies()->setExcludedConfigurationsStrategy(Strategies::ECS_ASSIGN_STATUS);
			pf->filterSampleByKnownProfile(sc1, profile1);
			ASSERT_EQUAL( 4, sc1->getMarkerConfigurations("MarkerA")->size() );
			ASSERT_EQUAL( 4, sc1->getMarkerConfigurations("MarkerB")->size() );
			ASSERT_EQUAL( MCStatusValid::getStatusName() , sc1->getMarkerConfigurations("MarkerA")->at(0)->getStatus()->getName() );
			ASSERT_EQUAL( MCStatusValid::getStatusName() , sc1->getMarkerConfigurations("MarkerA")->at(1)->getStatus()->getName() );
			ASSERT_EQUAL( MCStatusExcKnownProfile::getStatusName() , sc1->getMarkerConfigurations("MarkerA")->at(2)->getStatus()->getName() );
			ASSERT_EQUAL( MCStatusValid::getStatusName() , sc1->getMarkerConfigurations("MarkerA")->at(3)->getStatus()->getName() );
			ASSERT_EQUAL( MCStatusExcKnownProfile::getStatusName() , sc1->getMarkerConfigurations("MarkerB")->at(0)->getStatus()->getName() );
			ASSERT_EQUAL( MCStatusValid::getStatusName() , sc1->getMarkerConfigurations("MarkerB")->at(1)->getStatus()->getName() );
			ASSERT_EQUAL( MCStatusValid::getStatusName() , sc1->getMarkerConfigurations("MarkerB")->at(2)->getStatus()->getName() );
			ASSERT_EQUAL( MCStatusExcKnownProfile::getStatusName() , sc1->getMarkerConfigurations("MarkerB")->at(3)->getStatus()->getName() );

			// Performing filtering - excluding configurations

			pf->getStrategies()->setExcludedConfigurationsStrategy(Strategies::ECS_EXCLUDE_FROM_COMBINATIONS);
			pf->filterSampleByKnownProfile(sc2, profile1);
			ASSERT_EQUAL( 3, sc2->getMarkerConfigurations("MarkerA")->size() );
			ASSERT_EQUAL( 2, sc2->getMarkerConfigurations("MarkerB")->size() );
			ASSERT_EQUAL( MCStatusValid::getStatusName() , sc2->getMarkerConfigurations("MarkerA")->at(0)->getStatus()->getName() );
			ASSERT_EQUAL( MCStatusValid::getStatusName() , sc2->getMarkerConfigurations("MarkerA")->at(1)->getStatus()->getName() );
			ASSERT_EQUAL( MCStatusValid::getStatusName() , sc2->getMarkerConfigurations("MarkerA")->at(2)->getStatus()->getName() );
			ASSERT_EQUAL( MCStatusValid::getStatusName() , sc2->getMarkerConfigurations("MarkerB")->at(0)->getStatus()->getName() );
			ASSERT_EQUAL( MCStatusValid::getStatusName() , sc2->getMarkerConfigurations("MarkerB")->at(1)->getStatus()->getName() );
		}
	}
}
