/*
 * TestCsvFileReader.cpp
 *
 *  Created on: Nov 30, 2008
 *      Author: mariusz
 */

#include "TestCsvFileReader.h"

namespace dma
{

	namespace test
	{

		TestCsvFileReader::TestCsvFileReader()
		{

			csvFileCommaSeparated = "data/io/test_csv_file_reader/test_file_reader_csv_comma_separated.txt";
			csvFileSemicolonSeparated = "data/io/test_csv_file_reader/test_file_reader_csv_semicolon_separated.txt";
			csvFileTabSeparated = "data/io/test_csv_file_reader/test_file_reader_csv_tab_separated.txt";
		}

		TestCsvFileReader::~TestCsvFileReader()
		{

		}

		void TestCsvFileReader::testSeparatorDetermination()
		{

			// Creating file readers
			CsvFileReaderPtr csvFR1(new CsvFileReader(csvFileCommaSeparated, ""));
			CsvFileReaderPtr csvFR2(new CsvFileReader(csvFileSemicolonSeparated, ""));
			CsvFileReaderPtr csvFR3(new CsvFileReader(csvFileTabSeparated, ""));

			// Testing
			ASSERT_EQUAL( ",", csvFR1->getSeparator() );
			ASSERT_EQUAL( ";", csvFR2->getSeparator() );
			ASSERT_EQUAL( "\t", csvFR3->getSeparator() );
		}

		void TestCsvFileReader::testGetLineAndExplode()
		{

			// Creating file reader
			CsvFileReaderPtr csvFR1(new CsvFileReader(csvFileCommaSeparated, ""));

			// Container for read data
			boost::shared_ptr<std::vector<shared_ptr<std::vector<std::string> > > > lines(new std::vector<shared_ptr<std::vector<
					std::string> > >());

			// Performing reading
			shared_ptr<std::vector<std::string> > extractedLine(new std::vector<std::string>());
			do
			{

				// Reading single line
				extractedLine = csvFR1->getLineAndExplode();
				if (extractedLine->size() > 0)
				{

					lines->push_back(extractedLine);
				}
				else
				{

					break;
				}
			} while (1 == 1);

			// Testing result
			ASSERT_EQUAL( 12, lines->size() );
			ASSERT_EQUAL( 6, lines->at(0)->size() );
			ASSERT_EQUAL( 6, lines->at(11)->size() );
			ASSERT_EQUAL( "Column 1 text value 1", lines->at(1)->at(0) );
			ASSERT_EQUAL( "1060", lines->at(1)->at(5) );
			ASSERT_EQUAL( "Column 1 text value 11", lines->at(11)->at(0) );
			ASSERT_EQUAL( "1360", lines->at(11)->at(5) );
		}
	}
}
