/*
 * TestFileReader.cpp
 *
 *  Created on: Nov 30, 2008
 *      Author: mariusz
 */

#include "TestFileReader.h"

namespace dma
{
	namespace test
	{

		TestFileReader::TestFileReader()
		{

			emptyFile_ = "data/io/test_file_reader/test_file_reader_empty_file.txt";
			nonExistingFile_ = "some-no-existing-file";
			nonEmptyFileWin = "data/io/test_file_reader/test_file_reader_non_empty_win.txt";
			nonEmptyFileMac = "data/io/test_file_reader/test_file_reader_non_empty_mac.txt";
			nonEmptyFileUnix = "data/io/test_file_reader/test_file_reader_non_empty_unix.txt";
		}

		TestFileReader::~TestFileReader()
		{

		}

		void TestFileReader::testCheckFileExists() {

			// Performing test on non-existing file
			bool resultExistingFile = FileReader::checkFileExists(emptyFile_);

			// Performing test on existing file
			bool resultNonExistingFile = FileReader::checkFileExists(nonExistingFile_);

			// Testing
			ASSERT_EQUAL( true, resultExistingFile );
			ASSERT_EQUAL( false, resultNonExistingFile );
		}

		void TestFileReader::testGetFilename() {

			// Getting filename of existing opened file
			FileReaderPtr fr1(new FileReader(emptyFile_));
			ASSERT_EQUAL( emptyFile_, fr1->getFilename());

			// Checking whether exception is generated when attempt to open non-existing file
			ASSERT_THROWS( new FileReader(nonExistingFile_), FileNotFoundException );
		}

		void TestFileReader::testDetermineNewLineDelimiter() {


			// Creating filer reader
			FileReaderPtr frWin(new FileReader(nonEmptyFileWin));
			FileReaderPtr frMac(new FileReader(nonEmptyFileMac));
			FileReaderPtr frUnix(new FileReader(nonEmptyFileUnix));

			// Determining EOL
			ASSERT_EQUAL( '\n', frWin->determineNewLineDelimiter() );
			ASSERT_EQUAL( '\r', frMac->determineNewLineDelimiter() );
			ASSERT_EQUAL( '\n', frUnix->determineNewLineDelimiter() );

			CUTE_VERSION_H_

		}

	}
}
