/*
 * TestLikelihoodRatio.cpp
 *
 *  Created on: Dec 22, 2008
 *      Author: mariusz
 */

#include "TestLikelihoodRatio.h"

namespace dma
{

	namespace test
	{

		TestLikelihoodRatio::TestLikelihoodRatio()
		{

		}

		TestLikelihoodRatio::~TestLikelihoodRatio()
		{

		}

		void TestLikelihoodRatio::testMarkerLrAddition() {


			// Generating likelihood ratios for markers
			AlgExpPtr marker1Lr( new AlgValue("100", 100) );
			AlgExpPtr marker2Lr( new AlgValue("200", 200) );
			AlgExpPtr marker3Lr( new AlgValue("300", 300) );
			AlgExpPtr marker3Lr_2( new AlgValue("500", 500) );

			// Adding likelihood ratios
			LikelihoodRatioPtr lr(new LikelihoodRatio());
			lr->addLRForMarker( "Marker1", marker1Lr );
			lr->addLRForMarker( "Marker2", marker2Lr );
			lr->addLRForMarker( "Marker3", marker3Lr );

			// Testing
			ASSERT_EQUAL( 3, lr->getMarkerNames()->size() );
			ASSERT_EQUAL( "Marker1", lr->getMarkerNames()->at(0) );
			ASSERT_EQUAL( "Marker2", lr->getMarkerNames()->at(1) );
			ASSERT_EQUAL( "Marker3", lr->getMarkerNames()->at(2) );
			ASSERT_EQUAL( 100, lr->getLikelihoodRatio("Marker1")->getValue() );
			ASSERT_EQUAL( 200, lr->getLikelihoodRatio("Marker2")->getValue() );
			ASSERT_EQUAL( 300, lr->getLikelihoodRatio("Marker3")->getValue() );

			// Testing marker re-addition
			lr->addLRForMarker( "Marker3", marker3Lr_2 );
			ASSERT_EQUAL( 500, lr->getLikelihoodRatio("Marker3")->getValue() );
		}


		void TestLikelihoodRatio::testTotalLrCalculation()
		{

			LikelihoodRatioPtr lr(new LikelihoodRatio());

			AlgExpPtr av_A(new AlgValue("A", 1.0));
			AlgExpPtr av_B(new AlgValue("B", 2.0));
			AlgExpPtr av_C(new AlgValue("C", 3.0));
			AlgExpPtr av_D(new AlgValue("D", 4.0));
			AlgExpPtr av_E(new AlgValue("E", 5.0));
			AlgExpPtr av_F(new AlgValue("F", 6.0));

			// No markers present in likelihood ratio
			AlgExpPtr lr0 = lr->calculateTotalLr();

			// Addition of marker A
			lr->addLRForMarker("A", av_A);
			AlgExpPtr lrA = lr->calculateTotalLr();

			// Addition of marker B
			lr->addLRForMarker("B", av_B);
			AlgExpPtr lrB = lr->calculateTotalLr();

			// Addition of marker C
			lr->addLRForMarker("C", av_C);
			AlgExpPtr lrC = lr->calculateTotalLr();

			// Addition of marker D
			lr->addLRForMarker("D", av_D);
			AlgExpPtr lrD = lr->calculateTotalLr();

			// Addition of marker E
			lr->addLRForMarker("E", av_E);
			AlgExpPtr lrE = lr->calculateTotalLr();

			// Addition of marker F
			lr->addLRForMarker("F", av_F);
			AlgExpPtr lrF = lr->calculateTotalLr();

			ASSERT_EQUAL( 0, lr0->getValue() );
			ASSERT_EQUAL( 1, lrA->getValue() );
			ASSERT_EQUAL( 2, lrB->getValue() );
			ASSERT_EQUAL( 6, lrC->getValue() );
			ASSERT_EQUAL( 24, lrD->getValue() );
			ASSERT_EQUAL( 120, lrE->getValue() );
			ASSERT_EQUAL( 720, lrF->getValue() );

			// Product is 720
		}
	}
}
