#include "TestLocus.h"

namespace dma
{

	namespace test
	{

		TestLocus::TestLocus()
		{
		}

		TestLocus::~TestLocus()
		{
		}

		void TestLocus::testAlleleAddition()
		{

			// Creating alleles
			NumericAllelePtr na30(new NumericAllele(30.0));
			NumericAllelePtr na40(new NumericAllele(40.0));
			CharacterAllelePtr caX(new CharacterAllele('X'));
			CharacterAllelePtr caY(new CharacterAllele('Y'));

			// Creating numerical homozygotic locus
			LocusPtr locus1(new Locus(na30, na30));
			ASSERT_EQUALM("Numerical Homozygotic locus ( 30.0, 30.0 ) - test type failed", AlleleBase::NUMERIC, locus1->getType());
			ASSERT_EQUALM("Numerical Homozygotic locus ( 30.0, 30.0 ) - test zygosity failed", Locus::HOMOZYGOTE, locus1->getZygosity());
			ASSERT_EQUALM("Numerical Homozygotic locus ( 30.0, 30.0 ) - test alleles number failed", 2, locus1->getAlleleNumber());
			ASSERT_EQUALM("Numerical Homozygotic locus ( 30.0, 30.0 ) - test allele value ( expected 30.0 ) failed", 30.0 ,locus1->getAllele(0)->getAlleleValue() );
			ASSERT_EQUALM("Numerical Homozygotic locus ( 30.0, 30.0 ) - test allele value ( expected 30.0 ) failed", 30.0 ,locus1->getAllele(1)->getAlleleValue() );
			ASSERT_EQUALM("Numerical Homozygotic locus ( 30.0, 30.0 ) - search for existing ( 30.0 ) allele failed", true ,locus1->isAllelePresent(30.0) );
			ASSERT_EQUALM("Numerical Homozygotic locus ( 30.0, 30.0 ) - search for non-existing ( 40.0 ) allele failed", false ,locus1->isAllelePresent(40.0) );

			// Creating numerical heterozygotic locus - second allele greater
			LocusPtr locus2(new Locus(na30, na40));
			ASSERT_EQUALM("Numerical Heterozygotic locus ( 30.0, 40.0 ) - test type failed", AlleleBase::NUMERIC, locus2->getType());
			ASSERT_EQUALM("Numerical Heterozygotic locus ( 30.0, 40.0 ) - test zygosity failed", Locus::HETEROZYGOTE, locus2->getZygosity());
			ASSERT_EQUALM("Numerical Heterozygotic locus ( 30.0, 40.0 ) - test alleles number failed", 2, locus2->getAlleleNumber());
			ASSERT_EQUALM("Numerical Heterozygotic locus ( 30.0, 40.0 ) - test allele value ( expected 30.0 ) failed", 30.0 ,locus2->getAllele(0)->getAlleleValue() );
			ASSERT_EQUALM("Numerical Heterozygotic locus ( 30.0, 40.0 ) - test allele value ( expected 40.0 ) failed", 40.0 ,locus2->getAllele(1)->getAlleleValue() );

			// Creating numerical heterozygotic locus - first allele greater
			LocusPtr locus3(new Locus(na40, na30));
			ASSERT_EQUALM("Numerical Heterozygotic locus ( 40.0, 30.0 ) - test type failed", AlleleBase::NUMERIC, locus3->getType());
			ASSERT_EQUALM("Numerical Heterozygotic locus ( 40.0, 30.0 ) - test zygosity failed", Locus::HETEROZYGOTE, locus3->getZygosity());
			ASSERT_EQUALM("Numerical Heterozygotic locus ( 40.0, 30.0 ) - test alleles number failed", 2, locus3->getAlleleNumber());
			ASSERT_EQUALM("Numerical Heterozygotic locus ( 40.0, 30.0 ) - test allele value ( expected 30.0 ) failed", 30.0 ,locus3->getAllele(0)->getAlleleValue() );
			ASSERT_EQUALM("Numerical Heterozygotic locus ( 40.0, 30.0 ) - test allele value ( expected 40.0 ) failed", 40.0 ,locus3->getAllele(1)->getAlleleValue() );

			// Creating character homozygotic locus - first allele greater
			LocusPtr locus4(new Locus(caX, caY));
			ASSERT_EQUALM("Character Heterozygotic locus ( X, Y ) - test type failed", AlleleBase::CHARACTER, locus4->getType());
			ASSERT_EQUALM("Character Heterozygotic locus ( X, Y ) - test zygosity failed", Locus::HETEROZYGOTE, locus4->getZygosity());
			ASSERT_EQUALM("Character Heterozygotic locus ( X, Y ) - test alleles number failed", 2, locus4->getAlleleNumber());
			ASSERT_EQUALM("Character Heterozygotic locus ( X, Y ) - test allele value ( expected X ) failed", (int)'X' ,locus4->getAllele(0)->getAlleleValue() );
			ASSERT_EQUALM("Character Heterozygotic locus ( X, Y ) - test allele value ( expected Y ) failed", (int)'Y' ,locus4->getAllele(1)->getAlleleValue() );
			ASSERT_EQUALM("Character Heterozygotic locus ( X, Y ) - search for existing ( X ) allele failed", true ,locus4->isAllelePresent( (double)'X' ) );
			ASSERT_EQUALM("Character Heterozygotic locus ( X, Y ) - search for non-existing ( Z ) allele failed", false ,locus4->isAllelePresent( (double)'Z') );

			// Adding alleles of different types - exception should be throwned
			ASSERT_THROWSM("Locus genotype with one allele, second of different type - exception has not ben throwned", new Locus(na30, caX), AlleleTypeConflictException);

		}

		void TestLocus::testHashGeneration()
		{

			// Prime 1 - 47
			// Prime 2 - 67

			// Creating alleles
			NumericAllelePtr na1(new NumericAllele(1.0));
			NumericAllelePtr na2(new NumericAllele(2.0));

			// Creating loci
			LocusPtr locus11(new Locus(na1, na1));
			LocusPtr locus12(new Locus(na1, na2));
			LocusPtr locus21(new Locus(na2, na1));
			LocusPtr locus22(new Locus(na2, na2));

			// Calculating hash
			ASSERT_EQUAL(114, locus11->getHash());
			ASSERT_EQUAL(181, locus12->getHash());
			ASSERT_EQUAL(181, locus21->getHash());
			ASSERT_EQUAL(228, locus22->getHash());

		}

		void TestLocus::testHbCalculation()
		{

			// Creating alleles
			NumericAllelePtr na10(new NumericAllele(10.0, 0, 100, 1000));
			NumericAllelePtr na20(new NumericAllele(20.0, 0, 200, 4000));
			NumericAllelePtr na30(new NumericAllele(30.0, 0, 500, 25000));

			// Creating loci
			LocusPtr locus_10_10(new Locus(na10->cloneToPtr(), na10->cloneToPtr()));
			LocusPtr locus_10_20(new Locus(na10->cloneToPtr(), na20->cloneToPtr()));
			LocusPtr locus_30_10(new Locus(na30->cloneToPtr(), na10->cloneToPtr()));

			// Calculating HB - by peak height
			ASSERT_EQUAL( 1.0, locus_10_10->calculateHb( AlleleBase::PEAK_PROPERTY_HEIGHT) );
			ASSERT_EQUAL( 0.5, locus_10_20->calculateHb( AlleleBase::PEAK_PROPERTY_HEIGHT) );
			ASSERT_EQUAL( 0.2, locus_30_10->calculateHb( AlleleBase::PEAK_PROPERTY_HEIGHT) );

			// Calculating HB - by peak area
			ASSERT_EQUAL( 1.0, locus_10_10->calculateHb( AlleleBase::PEAK_PROPERTY_AREA) );
			ASSERT_EQUAL( 0.25, locus_10_20->calculateHb( AlleleBase::PEAK_PROPERTY_AREA) );
			ASSERT_EQUAL( 0.04, locus_30_10->calculateHb( AlleleBase::PEAK_PROPERTY_AREA) );
		}

		void TestLocus::testPeakAreaHeightCalculation()
		{

			// Creating alleles
			NumericAllelePtr na10(new NumericAllele(10.0, 0, 100, 1000));
			NumericAllelePtr na20(new NumericAllele(20.0, 0, 200, 4000));
			NumericAllelePtr na30(new NumericAllele(30.0, 0, 500, 25000));

			// Creating loci
			LocusPtr locus_10_10(new Locus(na10->cloneToPtr(), na10->cloneToPtr()));
			LocusPtr locus_10_20(new Locus(na10->cloneToPtr(), na20->cloneToPtr()));
			LocusPtr locus_30_10(new Locus(na30->cloneToPtr(), na10->cloneToPtr()));

			// Calculating total area
			ASSERT_EQUAL( 2000, locus_10_10->getTotalAlleleArea() );
			ASSERT_EQUAL( 5000, locus_10_20->getTotalAlleleArea() );
			ASSERT_EQUAL( 26000, locus_30_10->getTotalAlleleArea() );

			// Calculating total height
			ASSERT_EQUAL( 200, locus_10_10->getTotalAlleleHeight() );
			ASSERT_EQUAL( 300, locus_10_20->getTotalAlleleHeight() );
			ASSERT_EQUAL( 600, locus_30_10->getTotalAlleleHeight() );
		}

		void TestLocus::testCloning()
		{

			// Creating alleles
			NumericAllelePtr na30(new NumericAllele(30.0));
			NumericAllelePtr na40(new NumericAllele(40.0));
			CharacterAllelePtr caX(new CharacterAllele('X'));
			CharacterAllelePtr caY(new CharacterAllele('Y'));

			// Creating numerical homozygotic locus
			LocusPtr locus1(new Locus(na30, na30));
			locus1->setMarkerName("TEST1");

			// Checking locus before cloning
			ASSERT_EQUAL( AlleleBase::NUMERIC, locus1->getType());
			ASSERT_EQUAL( Locus::HOMOZYGOTE, locus1->getZygosity());
			ASSERT_EQUAL( 2, locus1->getAlleleNumber());
			ASSERT_EQUAL( 30.0 ,locus1->getAllele(0)->getAlleleValue() );
			ASSERT_EQUAL( 30.0 ,locus1->getAllele(1)->getAlleleValue() );
			ASSERT_EQUAL( true ,locus1->isAllelePresent(30.0) );
			ASSERT_EQUAL( false ,locus1->isAllelePresent(40.0) );
			ASSERT_EQUAL( "TEST1" ,locus1->getMarkerName() );

			// Cloning
			LocusPtr locus1C = locus1->cloneToSmartPtr();

			// Checking cloned locus
			ASSERT_EQUAL( AlleleBase::NUMERIC, locus1C->getType());
			ASSERT_EQUAL( Locus::HOMOZYGOTE, locus1C->getZygosity());
			ASSERT_EQUAL( 2, locus1C->getAlleleNumber());
			ASSERT_EQUAL( 30.0 ,locus1C->getAllele(0)->getAlleleValue() );
			ASSERT_EQUAL( 30.0 ,locus1C->getAllele(1)->getAlleleValue() );
			ASSERT_EQUAL( true ,locus1C->isAllelePresent(30.0) );
			ASSERT_EQUAL( false ,locus1C->isAllelePresent(40.0) );
			ASSERT_EQUAL( "TEST1" ,locus1C->getMarkerName() );

		}
	}
}
