 /*
 * TestMarker.cpp
 *
 *  Created on: Jul 9, 2009
 *      Author: mariusz
 */

#include "TestMarker.h"

namespace dma
{

	namespace test
	{

		TestMarker::TestMarker() :
			na10(new NumericAllele(10.0, 40, 100, 1000)),
			na11(new NumericAllele(11.0, 44, 200, 4000)),
			na20(new NumericAllele(20.0, 80, 400, 5000)),
			na21(new NumericAllele(21.0, 88, 500, 25000))
		{

		}

		TestMarker::~TestMarker()
		{

		}

		void TestMarker::testCreationAndAddition()
		{

			// Marker 1 - EMPTY
			MarkerPtr marker1(new Marker("Marker1"));

			// Testing properties of empty marker
			ASSERT_EQUAL( 0, marker1->getAllelesNumber() );
			ASSERT_EQUAL( "Marker1", marker1->getName() );
			ASSERT_EQUAL( AlleleBase::UNDEFINED, marker1->getType() );

			// Marker 2 - Alleles present
			MarkerPtr marker2(new Marker("Marker2"));
			marker2->addAllele(na10->cloneToPtr());
			marker2->addAllele(na11->cloneToPtr());

			// Testing valid operations
			ASSERT_EQUAL( 2, marker2->getAllelesNumber() );
			ASSERT_EQUAL( AlleleBase::NUMERIC, marker2->getType() );
			ASSERT_EQUAL( na10->getAlleleValue(), marker2->getAllele(0)->getAlleleValue() );
			ASSERT_EQUAL( na11->getAlleleValue(), marker2->getAllele(1)->getAlleleValue() );
			ASSERT_EQUAL( na10->getAlleleValue(), marker2->getAlleleByValue( na10->getAlleleValue())->getAlleleValue() );
			ASSERT_EQUAL( true, marker2->isAllelePresent( na10->getAlleleValue() ));

			// Testing error handling
			ASSERT_THROWS( marker2->getAllele(2), OutOfBoundsException );
			ASSERT_THROWS( marker2->getAlleleByValue(12.0), AlleleNotFoundException );
			ASSERT_EQUAL( false, marker2->isAllelePresent(12.0));
			CharacterAllelePtr caX(new CharacterAllele('X'));
			ASSERT_THROWS( marker2->addAllele(caX), AlleleTypeConflictException );

		}

		void TestMarker::testHbCalculation()
		{

			// Marker 1 - EMPTY
			MarkerPtr marker1(new Marker("Marker1"));

			// Testing error handling
			ASSERT_THROWS( marker1->calculateHb( 0, 1, AlleleBase::PEAK_PROPERTY_AREA ), OutOfBoundsException );

			// Marker 2 - Alleles present
			MarkerPtr marker2(new Marker("Marker2"));
			marker2->addAllele(na10->cloneToPtr());
			marker2->addAllele(na11->cloneToPtr());
			marker2->addAllele(na20->cloneToPtr());
			marker2->addAllele(na21->cloneToPtr());

			// Testing HB calculation ( based on area )
			ASSERT_EQUAL( 0.25, marker2->calculateHb(0, 1, AlleleBase::PEAK_PROPERTY_AREA ) );
			ASSERT_EQUAL( 0.25, marker2->calculateHb(1, 0, AlleleBase::PEAK_PROPERTY_AREA ) );
			ASSERT_EQUAL( 0.20, marker2->calculateHb(0, 2, AlleleBase::PEAK_PROPERTY_AREA ) );
			ASSERT_EQUAL( 0.04, marker2->calculateHb(3, 0, AlleleBase::PEAK_PROPERTY_AREA ) );

			// Testing HB calculation ( based on height )
			ASSERT_EQUAL( 0.50, marker2->calculateHb(0, 1, AlleleBase::PEAK_PROPERTY_HEIGHT ) );
			ASSERT_EQUAL( 0.50, marker2->calculateHb(1, 0, AlleleBase::PEAK_PROPERTY_HEIGHT ) );
			ASSERT_EQUAL( 0.25, marker2->calculateHb(0, 2, AlleleBase::PEAK_PROPERTY_HEIGHT ) );
			ASSERT_EQUAL( 0.20, marker2->calculateHb(3, 0, AlleleBase::PEAK_PROPERTY_HEIGHT ) );
		}

		void TestMarker::testCopyingAndAssignment() {

			// Marker 1 - EMPTY
			Marker marker1("Marker1");
			marker1.addAllele(na10->cloneToPtr());
			marker1.addAllele(na11->cloneToPtr());

			// Performing assigment
			Marker marker2 = marker1;

			// Testing
			ASSERT_EQUAL( "Marker1", marker2.getName() );
			ASSERT_EQUAL( 2, marker2.getAllelesNumber() );
			ASSERT_EQUAL( 10, marker2.getAllele(0)->getAlleleValue() );
			ASSERT_EQUAL( 11, marker2.getAllele(1)->getAlleleValue() );


		}
	}

}
