/*
 * TestMarkerConfiguration.cpp
 *
 *  Created on: Oct 16, 2008
 *      Author: mariusz
 */

#include "TestMarkerConfiguration.h"

namespace dma
{

	namespace test
	{

		TestMarkerConfiguration::TestMarkerConfiguration() :
			na1(new NumericAllele(1)), na2(new NumericAllele(2)), na3(new NumericAllele(3)), na4(new NumericAllele(4)),
					na5(new NumericAllele(5)), na6(new NumericAllele(6)), na7(new NumericAllele(7)), na8(new NumericAllele(8)),
					na9(new NumericAllele(9))
		{

		}

		TestMarkerConfiguration::~TestMarkerConfiguration()
		{

		}

		void TestMarkerConfiguration::testCopyAndAssigment()
		{

			// Generating loci
			LocusPtr locus_11(new Locus(na1->cloneToPtr(), na1->cloneToPtr()));
			LocusPtr locus_12(new Locus(na1->cloneToPtr(), na2->cloneToPtr()));
			LocusPtr locus_23(new Locus(na2->cloneToPtr(), na3->cloneToPtr()));

			// Generating marker configuration
			MarkerConfiguration mc1("MARKER_A");
			mc1.addLocus(locus_11);
			mc1.addLocus(locus_12);
			mc1.addLocus(locus_23);

			// Performing assignment
			MarkerConfiguration mc2 = mc1;
			ASSERT_EQUAL( mc1.getMarkerName(), mc2.getMarkerName() );
			ASSERT_EQUAL( mc1.getStatus()->getName(), mc2.getStatus()->getName() );
			ASSERT_EQUAL( mc1.getLociNumber(), mc2.getLociNumber() );
			ASSERT_EQUAL( mc1.getLocus(0), mc2.getLocus(0) );
		}

		void TestMarkerConfiguration::testCloneMarkerConfigurations()
		{

			// Input data generation
			MarkerConfigurationPtrVectorPtr mcInput(new MarkerConfigurationPtrVector());

			NumericAllelePtr nap10(new NumericAllele(10.0));
			NumericAllelePtr nap11(new NumericAllele(11.0));
			NumericAllelePtr nap12(new NumericAllele(12.0));
			NumericAllelePtr nap13(new NumericAllele(13.0));

			NumericAllelePtr nap20(new NumericAllele(20.0));
			NumericAllelePtr nap21(new NumericAllele(21.0));
			NumericAllelePtr nap22(new NumericAllele(22.0));
			NumericAllelePtr nap23(new NumericAllele(23.0));

			LocusPtr lgp_10_11(new Locus(nap10->cloneToPtr(), nap11->cloneToPtr()));
			LocusPtr lgp_12_13(new Locus(nap12->cloneToPtr(), nap13->cloneToPtr()));

			MarkerConfigurationPtr mcp_10_11_12_13(new MarkerConfiguration("MARKER_A"));
			mcp_10_11_12_13->addLocus(lgp_10_11);
			mcp_10_11_12_13->addLocus(lgp_12_13);

			LocusPtr lgp_20_21(new Locus(nap20->cloneToPtr(), nap21->cloneToPtr()));
			LocusPtr lgp_22_23(new Locus(nap22->cloneToPtr(), nap23->cloneToPtr()));
			MarkerConfigurationPtr mcp_20_21_22_23(new MarkerConfiguration("MARKER_A"));
			mcp_20_21_22_23->addLocus(lgp_20_21);
			mcp_20_21_22_23->addLocus(lgp_22_23);

			mcInput->push_back(mcp_10_11_12_13);
			mcInput->push_back(mcp_20_21_22_23);

			ASSERT_EQUAL( 2, mcInput->size());

			// Cloning
			MarkerConfigurationPtrVectorPtr mcCloned = MarkerConfiguration::cloneMarkerConfigurations(mcInput);

			// Checking cloning result
			ASSERT_EQUAL( 2, mcCloned->size() );

			mcp_10_11_12_13 = mcCloned->at(0);
			ASSERT_EQUAL( 2, mcp_10_11_12_13->getLociNumber() );
			ASSERT_EQUAL( true, mcp_10_11_12_13->isValid() );
			ASSERT_EQUAL( 2, mcp_10_11_12_13->getLocus(0)->getAlleleNumber() );
			ASSERT_EQUAL( 2, mcp_10_11_12_13->getLocus(1)->getAlleleNumber() );
			ASSERT_EQUAL( 10.0, mcp_10_11_12_13->getLocus(0)->getAllele(0)->getAlleleValue() );
			ASSERT_EQUAL( 11.0, mcp_10_11_12_13->getLocus(0)->getAllele(1)->getAlleleValue() );
			ASSERT_EQUAL( 12.0, mcp_10_11_12_13->getLocus(1)->getAllele(0)->getAlleleValue() );
			ASSERT_EQUAL( 13.0, mcp_10_11_12_13->getLocus(1)->getAllele(1)->getAlleleValue() );

			mcp_20_21_22_23 = mcCloned->at(1);
			ASSERT_EQUAL( 2, mcp_20_21_22_23->getLociNumber() );
			ASSERT_EQUAL( true, mcp_20_21_22_23->isValid() );
			ASSERT_EQUAL( 2, mcp_20_21_22_23->getLocus(0)->getAlleleNumber() );
			ASSERT_EQUAL( 2, mcp_20_21_22_23->getLocus(1)->getAlleleNumber() );
			ASSERT_EQUAL( 20.0, mcp_20_21_22_23->getLocus(0)->getAllele(0)->getAlleleValue() );
			ASSERT_EQUAL( 21.0, mcp_20_21_22_23->getLocus(0)->getAllele(1)->getAlleleValue() );
			ASSERT_EQUAL( 22.0, mcp_20_21_22_23->getLocus(1)->getAllele(0)->getAlleleValue() );
			ASSERT_EQUAL( 23.0, mcp_20_21_22_23->getLocus(1)->getAllele(1)->getAlleleValue() );
		}

		void TestMarkerConfiguration::testEquivalence()
		{

			// Generating loci
			LocusPtr locus_11(new Locus(na1->cloneToPtr(), na1->cloneToPtr()));
			LocusPtr locus_12(new Locus(na1->cloneToPtr(), na2->cloneToPtr()));
			LocusPtr locus_22(new Locus(na2->cloneToPtr(), na2->cloneToPtr()));
			LocusPtr locus_34(new Locus(na3->cloneToPtr(), na4->cloneToPtr()));
			LocusPtr locus_56(new Locus(na5->cloneToPtr(), na6->cloneToPtr()));
			LocusPtr locus_78(new Locus(na7->cloneToPtr(), na8->cloneToPtr()));

			// Creating configurations
			MarkerConfigurationPtr mc_12_34_56(new MarkerConfiguration("MARKER_A"));
			mc_12_34_56->addLocus(locus_12->cloneToSmartPtr());
			mc_12_34_56->addLocus(locus_34->cloneToSmartPtr());
			mc_12_34_56->addLocus(locus_56->cloneToSmartPtr());

			MarkerConfigurationPtr mc_56_34_12(new MarkerConfiguration("MARKER_A"));
			mc_56_34_12->addLocus(locus_56->cloneToSmartPtr());
			mc_56_34_12->addLocus(locus_34->cloneToSmartPtr());
			mc_56_34_12->addLocus(locus_12->cloneToSmartPtr());

			MarkerConfigurationPtr mc_12_34_78(new MarkerConfiguration("MARKER_A"));
			mc_12_34_78->addLocus(locus_12->cloneToSmartPtr());
			mc_12_34_78->addLocus(locus_34->cloneToSmartPtr());
			mc_12_34_78->addLocus(locus_78->cloneToSmartPtr());

			MarkerConfigurationPtr mc_11_11(new MarkerConfiguration("MARKER_A"));
			mc_11_11->addLocus(locus_11->cloneToSmartPtr());
			mc_11_11->addLocus(locus_11->cloneToSmartPtr());

			MarkerConfigurationPtr mc_11_12(new MarkerConfiguration("MARKER_A"));
			mc_11_12->addLocus(locus_11->cloneToSmartPtr());
			mc_11_12->addLocus(locus_12->cloneToSmartPtr());

			MarkerConfigurationPtr mc_11_22(new MarkerConfiguration("MARKER_A"));
			mc_11_22->addLocus(locus_11->cloneToSmartPtr());
			mc_11_22->addLocus(locus_12->cloneToSmartPtr());

			MarkerConfigurationPtr mc_12_34(new MarkerConfiguration("MARKER_A"));
			mc_12_34->addLocus(locus_12->cloneToSmartPtr());
			mc_12_34->addLocus(locus_34->cloneToSmartPtr());

			// Performing equivalence tests
			bool flag1 = mc_12_34_56->isEquivalent(mc_56_34_12);
			bool flag2 = mc_12_34_56->isEquivalent(mc_12_34_78);
			bool flag3 = mc_11_11->isEquivalent(mc_11_12);
			bool flag4 = mc_11_11->isEquivalent(mc_11_22);
			bool flag5 = mc_12_34_56->isEquivalent(mc_12_34);
			bool flag6 = mc_12_34_56->isEqual(mc_12_34_56);
			bool flag7 = mc_12_34_56->isEqual(mc_56_34_12);

			ASSERT_EQUALM("Equivalence test failed. Compared configurations - 12 34 56 	| 56 34 12 	- equivalent ", true, flag1 );
			ASSERT_EQUALM("Equivalence test failed. Compared configurations - 12 34 56 	| 12 34 78 	- not equivalent", false, flag2 );
			ASSERT_EQUALM("Equivalence test failed. Compared configurations - 11 11 	| 11 12 	- not equivalent", false, flag3 );
			ASSERT_EQUALM("Equivalence test failed. Compared configurations - 11 11    	| 11 22 	- not equivalent", false, flag4 );
			ASSERT_EQUALM("Equivalence test failed. Compared configurations - 12 34 56 	| 12 34 	- not equivalent", false, flag5 );
			ASSERT_EQUALM("Equality test failed. 	Compared configurations - 12 34 56 	| 12 34 56 	- not equal ", true, flag6 );
			ASSERT_EQUALM("Equality test failed. 	Compared configurations - 12 34 56 	| 56 34 12 	- not equal ", false, flag7 );
		}

		void TestMarkerConfiguration::testUniqueAlleles()
		{

			// Generating loci
			LocusPtr locus_11(new Locus(na1->cloneToPtr(), na1->cloneToPtr()));
			LocusPtr locus_12(new Locus(na1->cloneToPtr(), na2->cloneToPtr()));
			LocusPtr locus_23(new Locus(na2->cloneToPtr(), na3->cloneToPtr()));

			// Creating marker configuration and performing test
			MarkerConfigurationPtr mc1(new MarkerConfiguration("MARKER_A"));
			mc1->addLocus(locus_11);
			mc1->addLocus(locus_12);
			mc1->addLocus(locus_23);

			shared_ptr<std::vector<AlleleBasePtr> > result = mc1->getUniqueAlleles();

			ASSERT_EQUAL(3, result->size());
			ASSERT_EQUAL(na1->getAlleleValue(), result->at(0)->getAlleleValue());
			ASSERT_EQUAL(na2->getAlleleValue(), result->at(1)->getAlleleValue());
			ASSERT_EQUAL(na3->getAlleleValue(), result->at(2)->getAlleleValue());
		}

		void TestMarkerConfiguration::testLocusSearching()
		{

			// Generating loci
			LocusPtr locus_11(new Locus(na1->cloneToPtr(), na1->cloneToPtr()));
			LocusPtr locus_12(new Locus(na1->cloneToPtr(), na2->cloneToPtr()));
			LocusPtr locus_23(new Locus(na2->cloneToPtr(), na3->cloneToPtr()));

			// Generating marker configuration
			MarkerConfigurationPtr mc1(new MarkerConfiguration("MARKER_A"));
			mc1->addLocus(locus_11);
			mc1->addLocus(locus_12);
			mc1->addLocus(locus_23);

			// Generating searched loci - for valid case alleles given revere order
			LocusPtr locus_21(new Locus(na2->cloneToPtr(), na1->cloneToPtr()));
			LocusPtr locus_13(new Locus(na1->cloneToPtr(), na3->cloneToPtr()));

			bool testlocus_21 = mc1->isLocusPresent(locus_12);
			bool testlocus_13 = mc1->isLocusPresent(locus_13);

			ASSERT_EQUAL( true, testlocus_21);
			ASSERT_EQUAL( false, testlocus_13);
		}

		void TestMarkerConfiguration::testLocusSubtraction()
		{

			// Generating loci
			LocusPtr locus_11(new Locus(na1->cloneToPtr(), na1->cloneToPtr()));
			LocusPtr locus_12(new Locus(na1->cloneToPtr(), na2->cloneToPtr()));
			LocusPtr locus_23(new Locus(na2->cloneToPtr(), na3->cloneToPtr()));

			// Generating marker configuration
			MarkerConfigurationPtr mc(new MarkerConfiguration("MARKER_A"));
			mc->addLocus(locus_11);
			mc->addLocus(locus_12);
			mc->addLocus(locus_23);

			// Generating tested loci
			LocusPtr locus_21(new Locus(na2->cloneToPtr(), na1->cloneToPtr()));

			// Checking whether loci are present
			bool testlocus_21_before = mc->isLocusPresent(locus_21);
			mc->subtractLocus(locus_21);
			bool testlocus_21_after = mc->isLocusPresent(locus_21);

			ASSERT_EQUAL( true, testlocus_21_before);
			ASSERT_EQUAL( false, testlocus_21_after);

		}

		void TestMarkerConfiguration::testStatusAssignment()
		{

			// Creating empty configuration - inital status should be valid
			MarkerConfigurationPtr mc1(new MarkerConfiguration("MARKER_A"));

			// Checking whether configuraiton is in valid status
			MCStatusValidPtr statusValid(new MCStatusValid());
			ASSERT_EQUAL( statusValid->getName(), mc1->getStatus()->getName() );
			ASSERT_EQUAL( true, mc1->isValid() );

			// Setting status resulting in exclusion ( invalid status )
			MCStatusExcDuplicationPtr newStatus( new MCStatusExcDuplication() );
			mc1->setStatus( newStatus );

			// Checkign whether status has been changed properly
			ASSERT_EQUAL( newStatus->getName(), mc1->getStatus()->getName() );
			ASSERT_EQUAL( false, mc1->isValid() );
		}
	}
}
