/*
 * TestMixtureProportion.cpp
 *
 *  Created on: Nov 13, 2008
 *      Author: mariusz
 */

#include "TestMixtureProportion.h"

namespace dma
{

	namespace test
	{

		TestMixtureProportion::TestMixtureProportion()
		{


		}

		TestMixtureProportion::~TestMixtureProportion()
		{
			
		}

		void TestMixtureProportion::testSingleContributorShareCalculation()
		{

			MixtureProportionPtr mp1(new MixtureProportion());
			mp1->addContributor(1.0);
			mp1->addContributor(1.0);
			mp1->addContributor(2.0);
			MixtureProportionPtr mp2(new MixtureProportion());
			mp2->addContributor(0.25);
			mp2->addContributor(0.25);
			mp2->addContributor(0.50);

			// Expected values
			unsigned int eCN = 3;
			double eCS1 = 0.25;
			double eCS2 = 0.25;
			double eCS3 = 0.50;

			// Testing retrieving shared - valid indexes
			ASSERT_EQUALM("Mixture proportion 1: Contributors number check failed", eCN, mp1->getContributorsNumber());
			ASSERT_EQUALM("Mixture proportion 1: 1 contributor share error", eCS1, mp1->getContributorShare(0));
			ASSERT_EQUALM("Mixture proportion 1: 2 contributor share error", eCS2, mp1->getContributorShare(1));
			ASSERT_EQUALM("Mixture proportion 1: 3 contributor share error", eCS3, mp1->getContributorShare(2));
			ASSERT_EQUALM("Mixture proportion 2: Contributors number check failed", eCN, mp2->getContributorsNumber());
			ASSERT_EQUALM("Mixture proportion 2: 1 contributor share error", eCS1, mp2->getContributorShare(0));
			ASSERT_EQUALM("Mixture proportion 2: 2 contributor share error", eCS2, mp2->getContributorShare(1));
			ASSERT_EQUALM("Mixture proportion 2: 3 contributor share error", eCS3, mp2->getContributorShare(2));

			// Testing retrieving shared - invalid indexes
			ASSERT_THROWS( mp1->getContributorShare(5), OutOfBoundsException );

		}

		void TestMixtureProportion::testMultipleContributorShareCalculation()
		{

			// 0.25 0.5 0.25 //
			MixtureProportionPtr mp(new MixtureProportion());
			mp->addContributor(1.0);
			mp->addContributor(2.0);
			mp->addContributor(1.0);

			// Generating query indexes
			shared_ptr<std::vector<unsigned int> > indexes_1_3(new std::vector<unsigned int>());
			indexes_1_3->push_back(0);
			indexes_1_3->push_back(2);
			shared_ptr<std::vector<unsigned int> > indexes_2_3(new std::vector<unsigned int>());
			indexes_2_3->push_back(1);
			indexes_2_3->push_back(2);
			shared_ptr<std::vector<unsigned int> > indexes_1_2_3(new std::vector<unsigned int>());
			indexes_1_2_3->push_back(0);
			indexes_1_2_3->push_back(1);
			indexes_1_2_3->push_back(2);
			shared_ptr<std::vector<unsigned int> > indexes_2_3_1(new std::vector<unsigned int>());
			indexes_2_3_1->push_back(1);
			indexes_2_3_1->push_back(2);
			indexes_2_3_1->push_back(0);
			shared_ptr<std::vector<unsigned int> > indexes_3_1_2(new std::vector<unsigned int>());
			indexes_3_1_2->push_back(2);
			indexes_3_1_2->push_back(0);
			indexes_3_1_2->push_back(1);

			// Setting expected values and calculating actual values
			double e_mp_1_3_c1 = 0.50;
			double e_mp_1_3_c2 = 0.50;
			shared_ptr<std::vector<double> > a_mp_1_3 = mp->getContributorsShares(indexes_1_3);

			double e_mp_2_3_c2 = (2.0 / 3.0);
			double e_mp_2_3_c3 = (1.0 / 3.0);
			shared_ptr<std::vector<double> > a_mp_2_3 = mp->getContributorsShares(indexes_2_3);

			double e_mp_1_2_3_c1 = 0.25;
			double e_mp_1_2_3_c2 = 0.50;
			double e_mp_1_2_3_c3 = 0.25;
			shared_ptr<std::vector<double> > a_mp_1_2_3 = mp->getContributorsShares(indexes_1_2_3);

			double e_mp_2_3_1_c1 = 0.50;
			double e_mp_2_3_1_c2 = 0.25;
			double e_mp_2_3_1_c3 = 0.25;
			shared_ptr<std::vector<double> > a_mp_2_3_1 = mp->getContributorsShares(indexes_2_3_1);

			double e_mp_3_1_2_c1 = 0.25;
			double e_mp_3_1_2_c2 = 0.25;
			double e_mp_3_1_2_c3 = 0.50;
			shared_ptr<std::vector<double> > a_mp_3_1_2 = mp->getContributorsShares(indexes_3_1_2);

			// Testing for valid indexes
			ASSERT_EQUAL( 2, a_mp_1_3->size() );
			ASSERT_EQUAL( e_mp_1_3_c1, a_mp_1_3->at(0));
			ASSERT_EQUAL( e_mp_1_3_c2, a_mp_1_3->at(1));

			ASSERT_EQUAL( 2, a_mp_2_3->size() );
			ASSERT_EQUAL( e_mp_2_3_c2, a_mp_2_3->at(0));
			ASSERT_EQUAL( e_mp_2_3_c3, a_mp_2_3->at(1));

			ASSERT_EQUAL( 3, a_mp_1_2_3->size() );
			ASSERT_EQUAL( e_mp_1_2_3_c1, a_mp_1_2_3->at(0));
			ASSERT_EQUAL( e_mp_1_2_3_c2, a_mp_1_2_3->at(1));
			ASSERT_EQUAL( e_mp_1_2_3_c3, a_mp_1_2_3->at(2));

			ASSERT_EQUAL( 3, a_mp_2_3_1->size() );
			ASSERT_EQUAL( e_mp_2_3_1_c1, a_mp_2_3_1->at(0));
			ASSERT_EQUAL( e_mp_2_3_1_c2, a_mp_2_3_1->at(1));
			ASSERT_EQUAL( e_mp_2_3_1_c3, a_mp_2_3_1->at(2));

			ASSERT_EQUAL( 3, a_mp_3_1_2->size() );
			ASSERT_EQUAL( e_mp_3_1_2_c1, a_mp_3_1_2->at(0));
			ASSERT_EQUAL( e_mp_3_1_2_c2, a_mp_3_1_2->at(1));
			ASSERT_EQUAL( e_mp_3_1_2_c3, a_mp_3_1_2->at(2));

			// Testing against invalid indexes
			shared_ptr<std::vector<unsigned int> > indexes_1_4(new std::vector<unsigned int>());
			indexes_1_4->push_back(0);
			indexes_1_4->push_back(3);
			ASSERT_THROWS( mp->getContributorsShares(indexes_1_4), OutOfBoundsException );

		}

	}
}
