/*
 * TestProfile.cpp
 *
 *  Created on: Jul 7, 2009
 *      Author: mariusz
 */

#include "TestProfile.h"

namespace dma
{

	namespace test
	{

		TestProfile::TestProfile()
		{

		}

		TestProfile::~TestProfile()
		{

		}

		void TestProfile::testProfileCreation()
		{

			// Creating alleles and loci
			NumericAllelePtr nap10(new NumericAllele(10.0));
			NumericAllelePtr nap11(new NumericAllele(11.0));
			NumericAllelePtr nap20(new NumericAllele(20.0));
			NumericAllelePtr nap21(new NumericAllele(21.0));
			LocusPtr locus_10_11(new Locus(nap10, nap11));
			LocusPtr locus_20_21(new Locus(nap20, nap21));

			// Creating profiles
			ProfilePtr profile1(new Profile());

			// Test 1 - Expected number of marker - 0
			shared_ptr<std::vector<std::string> > markerNames = profile1->getMarkerNames();

			ASSERT_EQUAL( 0, markerNames->size() );
			ASSERT_THROWS( profile1->getMarker("TEST1"), MarkerNotFoundException );

			// Test 2 - Adding example markers
			profile1->addMarker("Marker1", locus_10_11);
			profile1->addMarker("Marker2", locus_20_21);
			markerNames = profile1->getMarkerNames();

			ASSERT_EQUAL( 2, markerNames->size() );
			ASSERT_EQUAL( 2, profile1->getMarker("Marker1")->getAlleleNumber() );
			ASSERT_EQUAL( 2, profile1->getMarker("Marker2")->getAlleleNumber() );
			ASSERT_EQUAL( 10, profile1->getMarker("Marker1")->getAllele(0)->getAlleleValue() );
			ASSERT_EQUAL( 11, profile1->getMarker("Marker1")->getAllele(1)->getAlleleValue() );
			ASSERT_EQUAL( 20, profile1->getMarker("Marker2")->getAllele(0)->getAlleleValue() );
			ASSERT_EQUAL( 21, profile1->getMarker("Marker2")->getAllele(1)->getAlleleValue() );
			ASSERT_EQUAL( "Marker1", profile1->getMarker("Marker1")->getMarkerName() );
			ASSERT_EQUAL( "Marker2", profile1->getMarker("Marker2")->getMarkerName() );
		}

		void TestProfile::testAssignment()
		{

			// Creating profile
			Profile profile1;

			// Creating alleles and loci
			NumericAllelePtr nap10(new NumericAllele(10.0));
			NumericAllelePtr nap11(new NumericAllele(11.0));
			NumericAllelePtr nap20(new NumericAllele(20.0));
			NumericAllelePtr nap21(new NumericAllele(21.0));
			LocusPtr locus_10_11(new Locus(nap10, nap11));
			LocusPtr locus_20_21(new Locus(nap20, nap21));
			profile1.addMarker("Marker1", locus_10_11);
			profile1.addMarker("Marker2", locus_20_21);

			// Copying
			Profile profile2 = profile1;

			// Testing
			ASSERT_EQUAL( 2, profile2.getMarkerNames()->size() );
			ASSERT_EQUAL( "Marker1", profile2.getMarkerNames()->at(0) );
			ASSERT_EQUAL( "Marker2", profile2.getMarkerNames()->at(1) );
			ASSERT_EQUAL( nap10->getAlleleValue(), profile2.getMarker("Marker1")->getAllele(0)->getAlleleValue() );
			ASSERT_EQUAL( nap21->getAlleleValue(), profile2.getMarker("Marker2")->getAllele(1)->getAlleleValue() );
		}

	}

}
