/*
 * TestSample.cpp
 *
 *  Created on: Jul 9, 2009
 *      Author: mariusz
 */

#include "TestSample.h"

namespace dma
{

	namespace test
	{

		TestSample::TestSample()
		{

		}

		TestSample::~TestSample()
		{

		}

		void TestSample::testEmptySample()
		{

			SamplePtr sample1(new Sample("Sample1", "Panel1"));
			ASSERT_EQUAL( "Sample1", sample1->getName() );
			ASSERT_EQUAL( "Panel1", sample1->getPanel() );
			ASSERT_EQUAL( 0, sample1->getMarkersNumber() );
			ASSERT_EQUAL( 0, sample1->getMarkerNames()->size() );
		}

		void TestSample::testNonEmptySample()
		{

			SamplePtr sample1(new Sample("Sample1", "Panel1"));

			// Creating markers and adding them to sample
			MarkerPtr marker1(new Marker("Marker1"));
			MarkerPtr marker2(new Marker("Marker2"));
			sample1->addMarker(marker1);
			sample1->addMarker(marker2);

			ASSERT_EQUAL( "Sample1", sample1->getName() );
			ASSERT_EQUAL( "Panel1", sample1->getPanel() );
			ASSERT_EQUAL( 2, sample1->getMarkersNumber() );

			ASSERT_EQUAL( "Marker1", sample1->getMarker("Marker1")->getName() );
			ASSERT_EQUAL( "Marker2", sample1->getMarker("Marker2")->getName() );
			ASSERT_THROWS( sample1->getMarker("MarkerNotExisting"), MarkerNotFoundException );

			ASSERT_EQUAL( 2, sample1->getMarkerNames()->size() );
			ASSERT_EQUAL( "Marker1", sample1->getMarkerNames()->at(0) );
			ASSERT_EQUAL( "Marker2", sample1->getMarkerNames()->at(1) );

			// Testing marker substitution when marker already exists within sample
			MarkerPtr marker1_2(new Marker("Marker1"));
			NumericAllelePtr na10(new NumericAllele(10.0));
			marker1_2->addAllele(na10);

			ASSERT_EQUAL( 0, sample1->getMarker("Marker1")->getAllelesNumber() );
			sample1->addMarker(marker1_2);
			ASSERT_EQUAL( 1, marker1_2->getAllelesNumber() );
			ASSERT_EQUAL( 1, sample1->getMarker("Marker1")->getAllelesNumber() );

		}

		void TestSample::testAssignment()
		{

			Sample sample1("Sample1", "Panel1");

			// Creating markers and adding them to sample
			MarkerPtr marker1(new Marker("Marker1"));
			MarkerPtr marker2(new Marker("Marker2"));
			sample1.addMarker(marker1);
			sample1.addMarker(marker2);

			// Performing assignement
			Sample sample2 = sample1;

			// Testing
			ASSERT_EQUAL( sample1.getName(), sample2.getName() );
			ASSERT_EQUAL( sample1.getPanel(), sample2.getPanel() );
			ASSERT_EQUAL( sample1.getMarkersNumber(), sample2.getMarkersNumber() );
			ASSERT_EQUAL( sample1.getMarkerNames()->at(0), sample2.getMarkerNames()->at(0) );
			ASSERT_EQUAL( sample1.getMarker(sample1.getMarkerNames()->at(0))->getName(), sample2.getMarker(sample2.getMarkerNames()->at(0))->getName() );

		}

	}

}
