/*
 * TestSampleConfiguration.cpp
 *
 *  Created on: Jul 16, 2009
 *      Author: mariusz
 */

#include "TestSampleConfiguration.h"

namespace dma
{

	namespace test
	{

		/*
		 NumericAllelePtr na_10;
		 NumericAllelePtr na_11;
		 NumericAllelePtr na_12;
		 NumericAllelePtr na_13;
		 NumericAllelePtr na_14;
		 NumericAllelePtr na_15;
		 LocusPtr locus_10_11;
		 LocusPtr locus_12_13;
		 LocusPtr locus_14_15;
		 */

		TestSampleConfiguration::TestSampleConfiguration() :
			na_10(new NumericAllele(10.0)),
			na_11(new NumericAllele(11.0)),
			na_12(new NumericAllele(12.0)),
			na_13(new NumericAllele(13.0)),
			na_14(new NumericAllele(14.0)),
			na_15(new NumericAllele(15.0)),
			na_16(new NumericAllele(16.0)),
			na_17(new NumericAllele(17.0)),
			locus_10_11(new Locus(na_10->cloneToPtr(), na_11->cloneToPtr())),
			locus_12_13(new Locus(na_12->cloneToPtr(), na_13->cloneToPtr())),
			locus_14_15(new Locus(na_14->cloneToPtr(), na_15->cloneToPtr())),
			locus_16_17(new Locus(na_16->cloneToPtr(), na_17->cloneToPtr()))
		{

		}

		TestSampleConfiguration::~TestSampleConfiguration()
		{

		}

		void TestSampleConfiguration::testAll()
		{

			// Creating marker configurations for marker A
			MarkerConfigurationPtr mcA_1(new MarkerConfiguration("MarkerA"));
			mcA_1->addLocus(locus_10_11->cloneToSmartPtr() );
			MarkerConfigurationPtr mcA_2(new MarkerConfiguration("MarkerA"));
			mcA_2->addLocus(locus_10_11->cloneToSmartPtr());
			mcA_2->addLocus(locus_12_13->cloneToSmartPtr());
			mcA_2->addLocus(locus_14_15->cloneToSmartPtr());
			MarkerConfigurationPtr mcA_3(new MarkerConfiguration("MarkerA"));
			mcA_3->addLocus(locus_10_11->cloneToSmartPtr());
			mcA_3->addLocus(locus_12_13->cloneToSmartPtr());
			mcA_3->addLocus(locus_14_15->cloneToSmartPtr());
			mcA_3->addLocus(locus_16_17->cloneToSmartPtr());

			// Creating marker configurations for marker B
			MarkerConfigurationPtr mcB_1(new MarkerConfiguration("MarkerB"));
			mcB_1->addLocus(locus_10_11->cloneToSmartPtr());
			mcB_1->addLocus(locus_12_13->cloneToSmartPtr());
			MarkerConfigurationPtr mcB_2(new MarkerConfiguration("MarkerB"));
			mcB_2->addLocus(locus_10_11->cloneToSmartPtr());
			mcB_2->addLocus(locus_12_13->cloneToSmartPtr());

			// Marking configuration with highest number of loci as invalid
			MCStatusExcMixtureProportionPtr statusA_3(new MCStatusExcMixtureProportion(locus_16_17->cloneToSmartPtr(), 0.2, 0.5));
			mcA_3->setStatus( statusA_3 );

			// Creating sample configurations
			SampleConfigurationPtr sample1( new SampleConfiguration( "Sample1") );
			sample1->addMarkerConfiguration( "MarkerA", mcA_1 );
			sample1->addMarkerConfiguration( "MarkerA", mcA_2 );
			sample1->addMarkerConfiguration( "MarkerA", mcA_3 );
			sample1->addMarkerConfiguration( "MarkerB", mcB_1 );
			sample1->addMarkerConfiguration( "MarkerB", mcB_2 );

			// Testing
			shared_ptr<std::vector<std::string> > markerNames = sample1->getMarkerNames();
			ASSERT_EQUAL( "Sample1", sample1->getName() );
			ASSERT_EQUAL( 2, markerNames->size() );
			ASSERT_EQUAL( "MarkerA", markerNames->at(0) );
			ASSERT_EQUAL( "MarkerB", markerNames->at(1) );
			ASSERT_EQUAL( 3, sample1->getMarkerConfigurations("MarkerA")->size() );
			ASSERT_EQUAL( 1, sample1->getMarkerConfigurations("MarkerA")->at(0)->getLociNumber() );
			ASSERT_EQUAL( 3, sample1->getMarkerConfigurations("MarkerA")->at(1)->getLociNumber() );
			ASSERT_EQUAL( 4, sample1->getMarkerConfigurations("MarkerA")->at(2)->getLociNumber() );
			ASSERT_EQUAL( 2, sample1->getMarkerConfigurations("MarkerB")->size() );
			ASSERT_EQUAL( 2, sample1->getMarkerConfigurations("MarkerB")->at(0)->getLociNumber() );
			ASSERT_EQUAL( 2, sample1->getMarkerConfigurations("MarkerB")->at(1)->getLociNumber() );
			ASSERT_EQUAL( 4, sample1->getMaximumLociNumber(false) );
			ASSERT_EQUAL( 3, sample1->getMaximumLociNumber(true) );

		}

		void TestSampleConfiguration::testAssignment() {

			// Creating configuration
			SampleConfiguration sc1("Sample1");
			MarkerConfigurationPtr mcA_1(new MarkerConfiguration("MarkerA"));
			MarkerConfigurationPtr mcA_2(new MarkerConfiguration("MarkerA"));
			MarkerConfigurationPtr mcB_1(new MarkerConfiguration("MarkerB"));
			sc1.addMarkerConfiguration(mcA_1->getMarkerName(), mcA_1 );
			sc1.addMarkerConfiguration(mcA_2->getMarkerName(), mcA_2 );
			sc1.addMarkerConfiguration(mcB_1->getMarkerName(), mcB_1 );

			// Perofrmign assignment
			SampleConfiguration sc2 = sc1;

			// Testing
			ASSERT_EQUAL( sc1.getName(), sc2.getName() );
			ASSERT_EQUAL( sc1.getMarkerNames()->size(), sc2.getMarkerNames()->size() );
			ASSERT_EQUAL( sc1.getMarkerNames()->at(0), sc2.getMarkerNames()->at(0) );
			ASSERT_EQUAL( sc1.getMarkerConfigurations("MarkerA")->size(), sc2.getMarkerConfigurations("MarkerA")->size() );


		}
	}

}
