/*
 * TestAlgebra.cpp
 *
 *  Created on: Dec 22, 2008
 *      Author: mariusz
 */

#include "TestAlgebra.h"

namespace dma
{

	namespace test
	{

		TestAlgebra::TestAlgebra()
		{

		}

		TestAlgebra::~TestAlgebra()
		{

		}

		void TestAlgebra::testAlgAdd()
		{
			AlgValuePtr av_A(new AlgValue("A", 10.0));
			AlgValuePtr av_B(new AlgValue("B", 20.0));
			AlgValuePtr av_C(new AlgValue("C", 30.0));

			AlgOperAddPtr ao(new AlgOperAdd(av_A, av_B));
			ao->addExp(av_C);

			ASSERT_EQUAL( 60.0, ao->getValue() );
			ASSERT_EQUAL( "( A + B + C )", ao->getSymbol() );

		}

		void TestAlgebra::testAlgSub()
		{

			AlgValuePtr av_A(new AlgValue("A", 100.0));
			AlgValuePtr av_B(new AlgValue("B", 20.0));
			AlgValuePtr av_C(new AlgValue("C", 10.0));

			AlgOperSubPtr ao(new AlgOperSub(av_A, av_B));
			ao->addExp(av_C);

			ASSERT_EQUAL( "( A - B - C )", ao->getSymbol() );
			ASSERT_EQUAL( 70.0, ao->getValue() );

		}

		void TestAlgebra::testAlgMulti()
		{

			AlgValuePtr av_A(new AlgValue("A", 10.0));
			AlgValuePtr av_B(new AlgValue("B", 20.0));
			AlgValuePtr av_C(new AlgValue("C", 30.0));

			AlgOperMultiPtr ao(new AlgOperMulti(av_A, av_B));
			ao->addExp(av_C);

			ASSERT_EQUAL( "( A * B * C )", ao->getSymbol() );
			ASSERT_EQUAL( 6000.0, ao->getValue() );

		}

		void TestAlgebra::testAlgDiv()
		{

			AlgValuePtr av_A(new AlgValue("A", 200.0));
			AlgValuePtr av_B(new AlgValue("B", 10.0));

			AlgOperDivPtr ao(new AlgOperDiv(av_A, av_B));

			ASSERT_EQUAL( "( A / B )", ao->getSymbol() );
			ASSERT_EQUAL( 20.0, ao->getValue() );

			AlgValuePtr av_C(new AlgValue("C", 200.0));
			AlgValuePtr av_D(new AlgValue("D", 0.0));

			AlgOperDivPtr ao2(new AlgOperDiv(av_C, av_D));

			ASSERT_EQUAL( "( C / D )", ao2->getSymbol() );
			ASSERT_THROWS( ao2->getValue(), DivisionByZeroException );
		}

		void TestAlgebra::testAlgComplex()
		{

			AlgValuePtr av_A(new AlgValue("A", 10.0));
			AlgValuePtr av_B(new AlgValue("B", 20.0));
			AlgValuePtr av_C(new AlgValue("C", 5.0));
			AlgValuePtr av_D(new AlgValue("D", 50.0));
			AlgValuePtr av_E(new AlgValue("E", 250.0));
			AlgValuePtr av_F(new AlgValue("F", 10.0));

			AlgOperPtr ao1(new AlgOperAdd(av_A, av_B));
			ASSERT_EQUAL( "( A + B )", ao1->getSymbol() );
			ASSERT_EQUAL( 30, ao1->getValue() );

			AlgOperPtr ao2(new AlgOperSub(ao1, av_C));
			ASSERT_EQUAL( "( ( A + B ) - C )", ao2->getSymbol());
			ASSERT_EQUAL( 25, ao2->getValue() );

			AlgOperPtr ao3(new AlgOperMulti(ao2, av_D));
			ASSERT_EQUAL( "( ( ( A + B ) - C ) * D )", ao3->getSymbol());
			ASSERT_EQUAL( 1250, ao3->getValue() );

			AlgOperPtr ao4(new AlgOperDiv(ao3, av_E));
			ASSERT_EQUAL( "( ( ( ( A + B ) - C ) * D ) / E )", ao4->getSymbol());
			ASSERT_EQUAL( 5, ao4->getValue() );

			AlgOperPtr ao5(new AlgOperAdd(ao4, av_F));
			ASSERT_EQUAL( "( ( ( ( ( A + B ) - C ) * D ) / E ) + F )", ao5->getSymbol());
			ASSERT_EQUAL( 15, ao5->getValue() );
		}

	}
}
