/*
 * TestHbDataParser.cpp
 *
 *  Created on: Nov 9, 2008
 *      Author: mariusz
 */

#include "TestHbDataParser.h"

namespace dma
{

	namespace test
	{

		TestHbDataParser::TestHbDataParser() :
			validFile_("data/parsers/test_hb_data_parser/test_hb_data_parser_data_valid.txt"),
					invalidFile_("data/parsers/test_hb_data_parser/test_hb_data_parser_data_invalid.txt")
		{

		}

		TestHbDataParser::~TestHbDataParser()
		{

		}

		void TestHbDataParser::testParseAndGetHb_valid()
		{

			// Creating parser and parsing
			HbDataParserPtr parser1(new HbDataParser(validFile_, ""));
			parser1->parse();

			// Parsing data
			HbMapPtr parsedHb = parser1->getHb();

			/*
			 * File data ( take concern that data given below may be outdated. Check source file )
			 *
			 * 	Marker name,HB
			 * 	D16S539,0.6
			 * 	D18S51,0.6
			 * 	D19S433,0.6
			 * 	D21S11,0.6
			 * 	D2S1338,0.6
			 * 	D3S1358,0.6
			 * 	D8S1179,0.6
			 * 	FGA,0.6
			 * 	TH01,0.6
			 * 	vWA,0.6
			 * 	AMEL,0.6
			 */

			// Expected results
			std::string markerName01 = "D16S539";
			std::string markerName02 = "D18S51";
			std::string markerName03 = "D19S433";
			std::string markerName04 = "D21S11";
			std::string markerName05 = "D2S1338";
			std::string markerName06 = "D3S1358";
			std::string markerName07 = "D8S1179";
			std::string markerName08 = "FGA";
			std::string markerName09 = "TH01";
			std::string markerName10 = "vWA";
			std::string markerName11 = "AMEL";
			double hbValue = 0.6;

			// Comparing expected data and parsing results
			ASSERT_EQUAL(11, parsedHb->size());
			ASSERT_EQUAL(1, parsedHb->count(markerName01));
			ASSERT_EQUAL(1, parsedHb->count(markerName02));
			ASSERT_EQUAL(1, parsedHb->count(markerName03));
			ASSERT_EQUAL(1, parsedHb->count(markerName04));
			ASSERT_EQUAL(1, parsedHb->count(markerName05));
			ASSERT_EQUAL(1, parsedHb->count(markerName06));
			ASSERT_EQUAL(1, parsedHb->count(markerName07));
			ASSERT_EQUAL(1, parsedHb->count(markerName08));
			ASSERT_EQUAL(1, parsedHb->count(markerName09));
			ASSERT_EQUAL(1, parsedHb->count(markerName10));
			ASSERT_EQUAL(1, parsedHb->count(markerName11));
			ASSERT_EQUAL( hbValue, parsedHb->find(markerName01)->second);
			ASSERT_EQUAL( hbValue, parsedHb->find(markerName02)->second);
			ASSERT_EQUAL( hbValue, parsedHb->find(markerName03)->second);
			ASSERT_EQUAL( hbValue, parsedHb->find(markerName04)->second);
			ASSERT_EQUAL( hbValue, parsedHb->find(markerName05)->second);
			ASSERT_EQUAL( hbValue, parsedHb->find(markerName06)->second);
			ASSERT_EQUAL( hbValue, parsedHb->find(markerName07)->second);
			ASSERT_EQUAL( hbValue, parsedHb->find(markerName08)->second);
			ASSERT_EQUAL( hbValue, parsedHb->find(markerName09)->second);
			ASSERT_EQUAL( hbValue, parsedHb->find(markerName10)->second);
			ASSERT_EQUAL( hbValue, parsedHb->find(markerName11)->second);
		}

		void TestHbDataParser::testParseAndGetHb_invalid()
		{

			// Creating parser and parsing
			HbDataParserPtr parser1(new HbDataParser(invalidFile_, ""));

			try {

				parser1->parse();
				ASSERTM( "Expected exception. Nothing throwned", 0 == 1 );
			} catch( ParserException e ) {

				ASSERT_EQUAL( parser1->getFileType(), e.getFileType() );
				ASSERT_EQUAL( invalidFile_, e.getFilename() );
				ASSERT_EQUAL( 2, e.getLineNumber() );
			}
		}
	}
}
