/*
 * TestMixtureDataParserColumnarFormat.cpp
 *
 *  Created on: Jul 25, 2009
 *      Author: mariusz
 */

#include "TestMixtureDataParserColumnarFormat.h"

namespace dma
{

	namespace test
	{

		TestMixtureDataParserColumnarFormat::TestMixtureDataParserColumnarFormat() :
					fileValid("data/parsers/test_mixture_data_parser_columnar_format/test_mixture_data_parser_columnar_format_valid.txt"),
					fileInvalidAlleleSize("data/parsers/test_mixture_data_parser_columnar_format/test_mixture_data_parser_columnar_format_invalid_allele_size.txt"),
					fileInvalidAlleleValue("data/parsers/test_mixture_data_parser_columnar_format/test_mixture_data_parser_columnar_format_invalid_allele_value.txt"),
					fileInvalidPeakHeight("data/parsers/test_mixture_data_parser_columnar_format/test_mixture_data_parser_columnar_format_invalid_peak_height.txt"),
					fileInvalidPeakArea("data/parsers/test_mixture_data_parser_columnar_format/test_mixture_data_parser_columnar_format_invalid_peak_area.txt"),
					fileInvalidTypeConflict("data/parsers/test_mixture_data_parser_columnar_format/test_mixture_data_parser_columnar_format_invalid_type_conflict.txt"),
					fileInvalidColumnsNumber("data/parsers/test_mixture_data_parser_columnar_format/test_mixture_data_parser_columnar_format_invalid_columns_number.txt")
		{

		}

		TestMixtureDataParserColumnarFormat::~TestMixtureDataParserColumnarFormat()
		{

		}

		void TestMixtureDataParserColumnarFormat::testParse_valid()
		{

			// Creating parser
			MixtureDataParserColumnarFormatPtr parser1(new MixtureDataParserColumnarFormat(fileValid, ""));

			// Parsing and fetching results
			parser1->parse();
			SamplePtrVectorPtr samples1 = parser1->getSamples();

			/*
			 * Test file contents:
			 *
			 * Sample Name,Panel,Marker,Allele 1,Allele 2,Size 1,Size 2,Height 1,Height 2,Peak Area 1,Peak Area 2
			 * 118_353_1a,SGM_Plus_v2,AMEL,X,Y,103.19,108.88,1946,2025,23796,25212
			 * 118_353_1a,SGM_Plus_v2,D8S1179,11,12,135.57,139.73,1200,357,16539,4706
			 */

			ASSERT_EQUAL( 1, samples1->size() );

			SamplePtr sample1 = samples1->at(0);
			ASSERT_EQUAL( "118_353_1a", sample1->getName() );
			ASSERT_EQUAL( "SGM_Plus_v2", sample1->getPanel() );

			ASSERT_EQUAL( 2, sample1->getMarkerNames()->size() );
			ASSERT_EQUAL( "AMEL", sample1->getMarkerNames()->at(0) );
			ASSERT_EQUAL( "D8S1179", sample1->getMarkerNames()->at(1) );

			ASSERT_EQUAL( 2, sample1->getMarker("AMEL")->getAllelesNumber() );
			ASSERT_EQUAL( 'X', sample1->getMarker("AMEL")->getAllele(0)->getAlleleValue() );
			ASSERT_EQUAL( 1946, sample1->getMarker("AMEL")->getAllele(0)->getPeakHeight() );
			ASSERT_EQUAL( 23796, sample1->getMarker("AMEL")->getAllele(0)->getPeakArea() );
			ASSERT_EQUAL( 'Y', sample1->getMarker("AMEL")->getAllele(1)->getAlleleValue() );
			ASSERT_EQUAL( 2025, sample1->getMarker("AMEL")->getAllele(1)->getPeakHeight() );
			ASSERT_EQUAL( 25212, sample1->getMarker("AMEL")->getAllele(1)->getPeakArea() );

			ASSERT_EQUAL( 2, sample1->getMarker("D8S1179")->getAllelesNumber() );
			ASSERT_EQUAL( 11, sample1->getMarker("D8S1179")->getAllele(0)->getAlleleValue() );
			ASSERT_EQUAL( 135.57, sample1->getMarker("D8S1179")->getAllele(0)->getAlleleSize() );
			ASSERT_EQUAL( 1200, sample1->getMarker("D8S1179")->getAllele(0)->getPeakHeight() );
			ASSERT_EQUAL( 16539, sample1->getMarker("D8S1179")->getAllele(0)->getPeakArea() );
			ASSERT_EQUAL( 12, sample1->getMarker("D8S1179")->getAllele(1)->getAlleleValue() );
			ASSERT_EQUAL( 139.73, sample1->getMarker("D8S1179")->getAllele(1)->getAlleleSize() );
			ASSERT_EQUAL( 357, sample1->getMarker("D8S1179")->getAllele(1)->getPeakHeight() );
			ASSERT_EQUAL( 4706, sample1->getMarker("D8S1179")->getAllele(1)->getPeakArea() );
		}

		void TestMixtureDataParserColumnarFormat::testParse_invalidAlleleSize()
		{

			// Creating parser
			MixtureDataParserColumnarFormatPtr parser1(new MixtureDataParserColumnarFormat(fileInvalidAlleleSize, ""));

			try
			{

				parser1->parse();
				ASSERTM("Expected exception. Nothing throwned", 0 == 1);
			} catch (ParserException e)
			{

				ASSERT_EQUAL(parser1->getFileType(), e.getFileType());
				ASSERT_EQUAL(fileInvalidAlleleSize, e.getFilename());
				ASSERT_EQUAL(3, e.getLineNumber());
			}
		}

		void TestMixtureDataParserColumnarFormat::testParse_invalidAlleleValue()
		{

			// Creating parser
			MixtureDataParserColumnarFormatPtr parser1(new MixtureDataParserColumnarFormat(fileInvalidAlleleValue, ""));

			try
			{

				parser1->parse();
				ASSERTM("Expected exception. Nothing throwned", 0 == 1);
			} catch (ParserException e)
			{

				ASSERT_EQUAL(parser1->getFileType(), e.getFileType());
				ASSERT_EQUAL(fileInvalidAlleleValue, e.getFilename());
				ASSERT_EQUAL(3, e.getLineNumber());
			}
		}

		void TestMixtureDataParserColumnarFormat::testParse_invalidPeakHeight()
		{

			// Creating parser
			MixtureDataParserColumnarFormatPtr parser1(new MixtureDataParserColumnarFormat(fileInvalidPeakHeight, ""));

			try
			{

				parser1->parse();
				ASSERTM("Expected exception. Nothing throwned", 0 == 1);
			} catch (ParserException e)
			{

				ASSERT_EQUAL(parser1->getFileType(), e.getFileType());
				ASSERT_EQUAL(fileInvalidPeakHeight, e.getFilename());
				ASSERT_EQUAL(3, e.getLineNumber());
			}
		}

		void TestMixtureDataParserColumnarFormat::testParse_invalidPeakArea()
		{

			// Creating parser
			MixtureDataParserColumnarFormatPtr parser1(new MixtureDataParserColumnarFormat(fileInvalidPeakArea, ""));

			try
			{

				parser1->parse();
				ASSERTM("Expected exception. Nothing throwned", 0 == 1);
			} catch (ParserException e)
			{

				ASSERT_EQUAL(parser1->getFileType(), e.getFileType());
				ASSERT_EQUAL(fileInvalidPeakArea, e.getFilename());
				ASSERT_EQUAL(3, e.getLineNumber());
			}
		}

		void TestMixtureDataParserColumnarFormat::testParse_invalidTypeConflict()
		{

			// Creating parser
			MixtureDataParserColumnarFormatPtr parser1(new MixtureDataParserColumnarFormat(fileInvalidTypeConflict, ""));

			try
			{

				parser1->parse();
				ASSERTM("Expected exception. Nothing throwned", 0 == 1);
			} catch (ParserException e)
			{

				ASSERT_EQUAL(parser1->getFileType(), e.getFileType());
				ASSERT_EQUAL(fileInvalidTypeConflict, e.getFilename());
				ASSERT_EQUAL(3, e.getLineNumber());
			}
		}

		void TestMixtureDataParserColumnarFormat::testParse_invalidColumnsNumber()
		{

			// Creating parser
			MixtureDataParserColumnarFormatPtr parser1(new MixtureDataParserColumnarFormat(fileInvalidColumnsNumber, ""));

			try
			{

				parser1->parse();
				ASSERTM("Expected exception. Nothing throwned", 0 == 1);
			} catch (ParserException e)
			{

				ASSERT_EQUAL(parser1->getFileType(), e.getFileType());
				ASSERT_EQUAL(fileInvalidColumnsNumber, e.getFilename());
				ASSERT_EQUAL(2, e.getLineNumber());
			}
		}
	}

}
