/*
 * TestMixtureDataParserColumnarFormat.cpp
 *
 *  Created on: Jul 25, 2009
 *      Author: mariusz
 */

#include "TestMixtureDataParserLinearFormat.h"

namespace dma
{

	namespace test
	{

		TestMixtureDataParserLinearFormat::TestMixtureDataParserLinearFormat() :
					fileValid("data/parsers/test_mixture_data_parser_linear_format/test_mixture_data_parser_linear_format_valid.txt"),
					fileInvalidAlleleSize("data/parsers/test_mixture_data_parser_linear_format/test_mixture_data_parser_linear_format_invalid_allele_size.txt"),
					fileInvalidAlleleValue("data/parsers/test_mixture_data_parser_linear_format/test_mixture_data_parser_linear_format_invalid_allele_value.txt"),
					fileInvalidPeakHeight("data/parsers/test_mixture_data_parser_linear_format/test_mixture_data_parser_linear_format_invalid_peak_height.txt"),
					fileInvalidPeakArea("data/parsers/test_mixture_data_parser_linear_format/test_mixture_data_parser_linear_format_invalid_peak_area.txt"),
					fileInvalidTypeConflict("data/parsers/test_mixture_data_parser_linear_format/test_mixture_data_parser_linear_format_invalid_type_conflict.txt"),
					fileInvalidColumnsNumber("data/parsers/test_mixture_data_parser_linear_format/test_mixture_data_parser_linear_format_invalid_columns_number.txt")
		{

		}

		TestMixtureDataParserLinearFormat::~TestMixtureDataParserLinearFormat()
		{

		}

		void TestMixtureDataParserLinearFormat::testParse_valid()
		{

			// Creating parser
			MixtureDataParserLinearFormatPtr parser1(new MixtureDataParserLinearFormat(fileValid, ""));

			// Parsing and fetching results
			parser1->parse();
			SamplePtrVectorPtr samples1 = parser1->getSamples();

			/*
			 * Test file contents:
			 *
			 * Sample File Name,Marker,Allele,Size,Height,Area
			 * frag_007_D01.fsa,D2S1338,24,328.94,1886,22776
			 * frag_007_D01.fsa,D2S1338,25,332.99,996,12007
			 * frag_007_D01.fsa,D2S1338,26,337.1,1008,12201
			 * frag_007_D01.fsa,,,75.07,70,781
			 * frag_007_D01.fsa,AMEL,X,103.34,4326,52215
			 */

			ASSERT_EQUAL( 1, samples1->size() );

			SamplePtr sample1 = samples1->at(0);
			ASSERT_EQUAL( "frag_007_D01.fsa", sample1->getName() );
			ASSERT_EQUAL( "", sample1->getPanel() );

			ASSERT_EQUAL( 2, sample1->getMarkerNames()->size() );
			ASSERT_EQUAL( "AMEL", sample1->getMarkerNames()->at(0) );
			ASSERT_EQUAL( "D2S1338", sample1->getMarkerNames()->at(1) );

			ASSERT_EQUAL( 1, sample1->getMarker("AMEL")->getAllelesNumber() );

			ASSERT_EQUAL( 'X', sample1->getMarker("AMEL")->getAllele(0)->getAlleleValue() );
			ASSERT_EQUAL( 4326, sample1->getMarker("AMEL")->getAllele(0)->getPeakHeight() );
			ASSERT_EQUAL( 52215, sample1->getMarker("AMEL")->getAllele(0)->getPeakArea() );

			ASSERT_EQUAL( 3, sample1->getMarker("D2S1338")->getAllelesNumber() );

			ASSERT_EQUAL( 24, sample1->getMarker("D2S1338")->getAllele(0)->getAlleleValue() );
			ASSERT_EQUAL( 328.94, sample1->getMarker("D2S1338")->getAllele(0)->getAlleleSize() );
			ASSERT_EQUAL( 1886, sample1->getMarker("D2S1338")->getAllele(0)->getPeakHeight() );
			ASSERT_EQUAL( 22776, sample1->getMarker("D2S1338")->getAllele(0)->getPeakArea() );

			ASSERT_EQUAL( 25, sample1->getMarker("D2S1338")->getAllele(1)->getAlleleValue() );
			ASSERT_EQUAL( 332.99, sample1->getMarker("D2S1338")->getAllele(1)->getAlleleSize() );
			ASSERT_EQUAL( 996, sample1->getMarker("D2S1338")->getAllele(1)->getPeakHeight() );
			ASSERT_EQUAL( 12007, sample1->getMarker("D2S1338")->getAllele(1)->getPeakArea() );

			ASSERT_EQUAL( 26, sample1->getMarker("D2S1338")->getAllele(2)->getAlleleValue() );
			ASSERT_EQUAL( 337.1, sample1->getMarker("D2S1338")->getAllele(2)->getAlleleSize() );
			ASSERT_EQUAL( 1008, sample1->getMarker("D2S1338")->getAllele(2)->getPeakHeight() );
			ASSERT_EQUAL( 12201, sample1->getMarker("D2S1338")->getAllele(2)->getPeakArea() );
		}

		void TestMixtureDataParserLinearFormat::testParse_invalidAlleleSize()
		{

			// Creating parser
			MixtureDataParserLinearFormatPtr parser1(new MixtureDataParserLinearFormat(fileInvalidAlleleSize, ""));

			try
			{

				parser1->parse();
				ASSERTM("Expected exception. Nothing throwned", 0 == 1);
			} catch (ParserException e)
			{

				ASSERT_EQUAL(parser1->getFileType(), e.getFileType());
				ASSERT_EQUAL(fileInvalidAlleleSize, e.getFilename());
				ASSERT_EQUAL(4, e.getLineNumber());
			}
		}

		void TestMixtureDataParserLinearFormat::testParse_invalidAlleleValue()
		{

			// Creating parser
			MixtureDataParserLinearFormatPtr parser1(new MixtureDataParserLinearFormat(fileInvalidAlleleValue, ""));

			try
			{

				parser1->parse();
				ASSERTM("Expected exception. Nothing throwned", 0 == 1);
			} catch (ParserException e)
			{

				ASSERT_EQUAL(parser1->getFileType(), e.getFileType());
				ASSERT_EQUAL(fileInvalidAlleleValue, e.getFilename());
				ASSERT_EQUAL(4, e.getLineNumber());
			}
		}

		void TestMixtureDataParserLinearFormat::testParse_invalidPeakHeight()
		{

			// Creating parser
			MixtureDataParserLinearFormatPtr parser1(new MixtureDataParserLinearFormat(fileInvalidPeakHeight, ""));

			try
			{

				parser1->parse();
				ASSERTM("Expected exception. Nothing throwned", 0 == 1);
			} catch (ParserException e)
			{

				ASSERT_EQUAL(parser1->getFileType(), e.getFileType());
				ASSERT_EQUAL(fileInvalidPeakHeight, e.getFilename());
				ASSERT_EQUAL(4, e.getLineNumber());
			}
		}

		void TestMixtureDataParserLinearFormat::testParse_invalidPeakArea()
		{

			// Creating parser
			MixtureDataParserLinearFormatPtr parser1(new MixtureDataParserLinearFormat(fileInvalidPeakArea, ""));

			try
			{

				parser1->parse();
				ASSERTM("Expected exception. Nothing throwned", 0 == 1);
			} catch (ParserException e)
			{

				ASSERT_EQUAL(parser1->getFileType(), e.getFileType());
				ASSERT_EQUAL(fileInvalidPeakArea, e.getFilename());
				ASSERT_EQUAL(4, e.getLineNumber());
			}
		}

		void TestMixtureDataParserLinearFormat::testParse_invalidTypeConflict()
		{

			// Creating parser
			MixtureDataParserLinearFormatPtr parser1(new MixtureDataParserLinearFormat(fileInvalidTypeConflict, ""));

			try
			{

				parser1->parse();
				ASSERTM("Expected exception. Nothing throwned", 0 == 1);
			} catch (ParserException e)
			{

				ASSERT_EQUAL(parser1->getFileType(), e.getFileType());
				ASSERT_EQUAL(fileInvalidTypeConflict, e.getFilename());
				ASSERT_EQUAL(4, e.getLineNumber());
			}
		}

		void TestMixtureDataParserLinearFormat::testParse_invalidColumnsNumber()
		{

			// Creating parser
			MixtureDataParserLinearFormatPtr parser1(new MixtureDataParserLinearFormat(fileInvalidColumnsNumber, ""));

			try
			{

				parser1->parse();
				ASSERTM("Expected exception. Nothing throwned", 0 == 1);
			} catch (ParserException e)
			{

				ASSERT_EQUAL(parser1->getFileType(), e.getFileType());
				ASSERT_EQUAL(fileInvalidColumnsNumber, e.getFilename());
				ASSERT_EQUAL(4, e.getLineNumber());
			}
		}
	}

}
