/*
 * TestOutputDataParser.cpp
 *
 *  Created on: Dec 13, 2008
 *      Author: mariusz
 */

#include "TestOutputDataParser.h"

namespace dma
{

	namespace test
	{

		TestOutputDataParser::TestOutputDataParser() :
					fileValid_("data/parsers/test_output_data_parser/test_output_data_parser_valid.txt"),
					fileInvalidSampleNameNotGiven("data/parsers/test_output_data_parser/test_output_data_parser_invalid_sample_name_not_given.txt"),
					fileInvalidMarkerNameNotGiven("data/parsers/test_output_data_parser/test_output_data_parser_invalid_marker_name_not_given.txt"),
					fileInvalidMarkerTypeIncorrect1("data/parsers/test_output_data_parser/test_output_data_parser_invalid_marker_type_incorrect_1.txt"),
					fileInvalidMarkerTypeIncorrect2("data/parsers/test_output_data_parser/test_output_data_parser_invalid_marker_type_incorrect_2.txt"),
					fileInvalidContributorNumberNotGiven("data/parsers/test_output_data_parser/test_output_data_parser_invalid_contributor_number_not_given.txt"),
					fileInvalidContributorNumberNotNumerical("data/parsers/test_output_data_parser/test_output_data_parser_invalid_contributor_number_not_numerical.txt"),
					fileInvalidContributorNumberZero("data/parsers/test_output_data_parser/test_output_data_parser_invalid_contributor_number_zero.txt"),
					fileInvalidStatusNotGiven("data/parsers/test_output_data_parser/test_output_data_parser_invalid_status_not_given.txt"),
					fileInvalidStatusIncorrect("data/parsers/test_output_data_parser/test_output_data_parser_invalid_status_incorrect.txt"),
					fileInvalidAlleleTypeConflict("data/parsers/test_output_data_parser/test_output_data_parser_invalid_allele_type_conflict.txt"),
					fileInvalidIncorrectColumnsNumber("data/parsers/test_output_data_parser/test_output_data_parser_invalid_columns_number.txt"),
					fileInvalidLocusData("data/parsers/test_output_data_parser/test_output_data_parser_invalid_locus_data.txt")
		{

		}

		TestOutputDataParser::~TestOutputDataParser()
		{

		}

		void TestOutputDataParser::testParse_valid()
		{

			// Creating parser and parsing file
			OutputDataParserPtr parser1(new OutputDataParser(fileValid_, ""));
			parser1->parse();

			// Testing
			SampleConfigurationPtr sample1 = parser1->getSampleConfiguration();

			ASSERT_EQUAL( "118_353_1a", sample1->getName() );
			ASSERT_EQUAL( 2, sample1->getMarkerNames()->size() );
			ASSERT_EQUAL( "AMEL", sample1->getMarkerNames()->at(0) );
			ASSERT_EQUAL( "D16S539", sample1->getMarkerNames()->at(1) );

			ASSERT_EQUAL( 2, sample1->getMarkerConfigurations("AMEL")->size() );

			ASSERT_EQUAL( 2, sample1->getMarkerConfigurations("AMEL")->at(0)->getLociNumber() );
			ASSERT_EQUAL( (double)'X', sample1->getMarkerConfigurations("AMEL")->at(0)->getLocus(0)->getAllele(0)->getAlleleValue() );
			ASSERT_EQUAL( 376, sample1->getMarkerConfigurations("AMEL")->at(0)->getLocus(0)->getAllele(0)->getPeakHeight() );
			ASSERT_EQUAL( 4605, sample1->getMarkerConfigurations("AMEL")->at(0)->getLocus(0)->getAllele(0)->getPeakArea() );
			ASSERT_EQUAL( (double)'X', sample1->getMarkerConfigurations("AMEL")->at(0)->getLocus(0)->getAllele(1)->getAlleleValue() );
			ASSERT_EQUAL( 376, sample1->getMarkerConfigurations("AMEL")->at(0)->getLocus(0)->getAllele(1)->getPeakHeight() );
			ASSERT_EQUAL( 4605, sample1->getMarkerConfigurations("AMEL")->at(0)->getLocus(0)->getAllele(1)->getPeakArea() );
			ASSERT_EQUAL( (double)'X', sample1->getMarkerConfigurations("AMEL")->at(0)->getLocus(1)->getAllele(0)->getAlleleValue() );
			ASSERT_EQUAL( 1192, sample1->getMarkerConfigurations("AMEL")->at(0)->getLocus(1)->getAllele(0)->getPeakHeight() );
			ASSERT_EQUAL( 14584, sample1->getMarkerConfigurations("AMEL")->at(0)->getLocus(1)->getAllele(0)->getPeakArea() );
			ASSERT_EQUAL( (double)'Y', sample1->getMarkerConfigurations("AMEL")->at(0)->getLocus(1)->getAllele(1)->getAlleleValue() );
			ASSERT_EQUAL( 2025, sample1->getMarkerConfigurations("AMEL")->at(0)->getLocus(1)->getAllele(1)->getPeakHeight() );
			ASSERT_EQUAL( 25212, sample1->getMarkerConfigurations("AMEL")->at(0)->getLocus(1)->getAllele(1)->getPeakArea() );
			ASSERT_EQUAL( MCStatusExcHeterozygoticImbalance::getStatusName(), sample1->getMarkerConfigurations("AMEL")->at(0)->getStatus()->getName() );

			ASSERT_EQUAL( 2, sample1->getMarkerConfigurations("AMEL")->at(1)->getLociNumber() );
			ASSERT_EQUAL( (double)'X', sample1->getMarkerConfigurations("AMEL")->at(1)->getLocus(0)->getAllele(0)->getAlleleValue() );
			ASSERT_EQUAL( 973, sample1->getMarkerConfigurations("AMEL")->at(1)->getLocus(0)->getAllele(0)->getPeakHeight() );
			ASSERT_EQUAL( 11898, sample1->getMarkerConfigurations("AMEL")->at(1)->getLocus(0)->getAllele(0)->getPeakArea() );
			ASSERT_EQUAL( (double)'X', sample1->getMarkerConfigurations("AMEL")->at(1)->getLocus(0)->getAllele(1)->getAlleleValue() );
			ASSERT_EQUAL( 973, sample1->getMarkerConfigurations("AMEL")->at(1)->getLocus(0)->getAllele(1)->getPeakHeight() );
			ASSERT_EQUAL( 11898, sample1->getMarkerConfigurations("AMEL")->at(1)->getLocus(0)->getAllele(1)->getPeakArea() );
			ASSERT_EQUAL( (double)'Y', sample1->getMarkerConfigurations("AMEL")->at(1)->getLocus(1)->getAllele(0)->getAlleleValue() );
			ASSERT_EQUAL( 1012, sample1->getMarkerConfigurations("AMEL")->at(1)->getLocus(1)->getAllele(0)->getPeakHeight() );
			ASSERT_EQUAL( 12606, sample1->getMarkerConfigurations("AMEL")->at(1)->getLocus(1)->getAllele(0)->getPeakArea() );
			ASSERT_EQUAL( (double)'Y', sample1->getMarkerConfigurations("AMEL")->at(1)->getLocus(1)->getAllele(1)->getAlleleValue() );
			ASSERT_EQUAL( 1012, sample1->getMarkerConfigurations("AMEL")->at(1)->getLocus(1)->getAllele(1)->getPeakHeight() );
			ASSERT_EQUAL( 12606, sample1->getMarkerConfigurations("AMEL")->at(1)->getLocus(1)->getAllele(1)->getPeakArea() );
			ASSERT_EQUAL( MCStatusValid::getStatusName(), sample1->getMarkerConfigurations("AMEL")->at(1)->getStatus()->getName() );

			ASSERT_EQUAL( 1, sample1->getMarkerConfigurations("D16S539")->at(0)->getLociNumber() );
			ASSERT_EQUAL( 11, sample1->getMarkerConfigurations("D16S539")->at(0)->getLocus(0)->getAllele(0)->getAlleleValue() );
			ASSERT_EQUAL( 56, sample1->getMarkerConfigurations("D16S539")->at(0)->getLocus(0)->getAllele(0)->getPeakHeight() );
			ASSERT_EQUAL( 827, sample1->getMarkerConfigurations("D16S539")->at(0)->getLocus(0)->getAllele(0)->getPeakArea() );
			ASSERT_EQUAL( 11, sample1->getMarkerConfigurations("D16S539")->at(0)->getLocus(0)->getAllele(1)->getAlleleValue() );
			ASSERT_EQUAL( 56, sample1->getMarkerConfigurations("D16S539")->at(0)->getLocus(0)->getAllele(1)->getPeakHeight() );
			ASSERT_EQUAL( 827, sample1->getMarkerConfigurations("D16S539")->at(0)->getLocus(0)->getAllele(1)->getPeakArea() );
			ASSERT_EQUAL( MCStatusExcAlleleMissing::getStatusName(), sample1->getMarkerConfigurations("D16S539")->at(0)->getStatus()->getName() );

			ASSERT_EQUAL( 2, sample1->getMarkerConfigurations("D16S539")->at(1)->getLociNumber() );
			ASSERT_EQUAL( 11, sample1->getMarkerConfigurations("D16S539")->at(1)->getLocus(0)->getAllele(0)->getAlleleValue() );
			ASSERT_EQUAL( 90, sample1->getMarkerConfigurations("D16S539")->at(1)->getLocus(0)->getAllele(0)->getPeakHeight() );
			ASSERT_EQUAL( 1335, sample1->getMarkerConfigurations("D16S539")->at(1)->getLocus(0)->getAllele(0)->getPeakArea() );
			ASSERT_EQUAL( 11, sample1->getMarkerConfigurations("D16S539")->at(1)->getLocus(0)->getAllele(1)->getAlleleValue() );
			ASSERT_EQUAL( 90, sample1->getMarkerConfigurations("D16S539")->at(1)->getLocus(0)->getAllele(1)->getPeakHeight() );
			ASSERT_EQUAL( 1335, sample1->getMarkerConfigurations("D16S539")->at(1)->getLocus(0)->getAllele(1)->getPeakArea() );
			ASSERT_EQUAL( 11, sample1->getMarkerConfigurations("D16S539")->at(1)->getLocus(1)->getAllele(0)->getAlleleValue() );
			ASSERT_EQUAL( 288, sample1->getMarkerConfigurations("D16S539")->at(1)->getLocus(1)->getAllele(0)->getPeakHeight() );
			ASSERT_EQUAL( 4227, sample1->getMarkerConfigurations("D16S539")->at(1)->getLocus(1)->getAllele(0)->getPeakArea() );
			ASSERT_EQUAL( 12, sample1->getMarkerConfigurations("D16S539")->at(1)->getLocus(1)->getAllele(1)->getAlleleValue() );
			ASSERT_EQUAL( 275, sample1->getMarkerConfigurations("D16S539")->at(1)->getLocus(1)->getAllele(1)->getPeakHeight() );
			ASSERT_EQUAL( 3983, sample1->getMarkerConfigurations("D16S539")->at(1)->getLocus(1)->getAllele(1)->getPeakArea() );
			ASSERT_EQUAL( MCStatusExcAlleleMissing::getStatusName(), sample1->getMarkerConfigurations("D16S539")->at(0)->getStatus()->getName() );

		}

		void TestOutputDataParser::testParse_invalidSampleNameNotGiven()
		{

			// Creating parser
			OutputDataParserPtr parser1(new OutputDataParser(fileInvalidSampleNameNotGiven, ""));

			try
			{

				parser1->parse();
				ASSERTM( "Expected exception. Nothing throwned", 0 == 1 );
			} catch (ParserException e)
			{

				ASSERT_EQUAL( parser1->getFileType(), e.getFileType() );
				ASSERT_EQUAL( fileInvalidSampleNameNotGiven, e.getFilename() );
				ASSERT_EQUAL( 2, e.getLineNumber() );
			}
		}
		void TestOutputDataParser::testParse_invalidMarkerNameNotGiven()
		{

			// Creating parser
			OutputDataParserPtr parser1(new OutputDataParser(fileInvalidMarkerNameNotGiven, ""));

			try
			{

				parser1->parse();
				ASSERTM( "Expected exception. Nothing throwned", 0 == 1 );
			} catch (ParserException e)
			{

				ASSERT_EQUAL( parser1->getFileType(), e.getFileType() );
				ASSERT_EQUAL( fileInvalidMarkerNameNotGiven, e.getFilename() );
				ASSERT_EQUAL( 2, e.getLineNumber() );
			}
		}
		void TestOutputDataParser::testParse_invalidMarkerTypeIncorrect1()
		{

			// Creating parser
			OutputDataParserPtr parser1(new OutputDataParser(fileInvalidMarkerTypeIncorrect1, ""));

			try
			{

				parser1->parse();
				ASSERTM( "Expected exception. Nothing throwned", 0 == 1 );
			} catch (ParserException e)
			{

				ASSERT_EQUAL( parser1->getFileType(), e.getFileType() );
				ASSERT_EQUAL( fileInvalidMarkerTypeIncorrect1, e.getFilename() );
				ASSERT_EQUAL( 2, e.getLineNumber() );
			}
		}
		void TestOutputDataParser::testParse_invalidMarkerTypeIncorrect2()
		{

			// Creating parser
			OutputDataParserPtr parser1(new OutputDataParser(fileInvalidMarkerTypeIncorrect2, ""));

			try
			{

				parser1->parse();
				ASSERTM( "Expected exception. Nothing throwned", 0 == 1 );
			} catch (ParserException e)
			{

				ASSERT_EQUAL( parser1->getFileType(), e.getFileType() );
				ASSERT_EQUAL( fileInvalidMarkerTypeIncorrect2, e.getFilename() );
				ASSERT_EQUAL( 2, e.getLineNumber() );
			}
		}
		void TestOutputDataParser::testParse_invalidContributorNumberNotGiven()
		{

			// Creating parser
			OutputDataParserPtr parser1(new OutputDataParser(fileInvalidContributorNumberNotGiven, ""));

			try
			{

				parser1->parse();
				ASSERTM( "Expected exception. Nothing throwned", 0 == 1 );
			} catch (ParserException e)
			{

				ASSERT_EQUAL( parser1->getFileType(), e.getFileType() );
				ASSERT_EQUAL( fileInvalidContributorNumberNotGiven, e.getFilename() );
				ASSERT_EQUAL( 2, e.getLineNumber() );
			}
		}
		void TestOutputDataParser::testParse_invalidContributorNumberNotNumerical()
		{

			// Creating parser
			OutputDataParserPtr parser1(new OutputDataParser(fileInvalidContributorNumberNotNumerical, ""));

			try
			{

				parser1->parse();
				ASSERTM( "Expected exception. Nothing throwned", 0 == 1 );
			} catch (ParserException e)
			{

				ASSERT_EQUAL( parser1->getFileType(), e.getFileType() );
				ASSERT_EQUAL( fileInvalidContributorNumberNotNumerical, e.getFilename() );
				ASSERT_EQUAL( 2, e.getLineNumber() );
			}
		}
		void TestOutputDataParser::testParse_invalidContributorNumberZero()
		{

			// Creating parser
			OutputDataParserPtr parser1(new OutputDataParser(fileInvalidContributorNumberZero, ""));

			try
			{

				parser1->parse();
				ASSERTM( "Expected exception. Nothing throwned", 0 == 1 );
			} catch (ParserException e)
			{

				ASSERT_EQUAL( parser1->getFileType(), e.getFileType() );
				ASSERT_EQUAL( fileInvalidContributorNumberZero, e.getFilename() );
				ASSERT_EQUAL( 2, e.getLineNumber() );
			}
		}
		void TestOutputDataParser::testParse_invalidStatusNotGiven()
		{

			// Creating parser
			OutputDataParserPtr parser1(new OutputDataParser(fileInvalidStatusNotGiven, ""));

			try
			{

				parser1->parse();
				ASSERTM( "Expected exception. Nothing throwned", 0 == 1 );
			} catch (ParserException e)
			{

				ASSERT_EQUAL( parser1->getFileType(), e.getFileType() );
				ASSERT_EQUAL( fileInvalidStatusNotGiven, e.getFilename() );
				ASSERT_EQUAL( 2, e.getLineNumber() );
			}
		}
		void TestOutputDataParser::testParse_invalidStatusIncorrect()
		{

			// Creating parser
			OutputDataParserPtr parser1(new OutputDataParser(fileInvalidStatusIncorrect, ""));

			try
			{

				parser1->parse();
				ASSERTM( "Expected exception. Nothing throwned", 0 == 1 );
			} catch (ParserException e)
			{

				ASSERT_EQUAL( parser1->getFileType(), e.getFileType() );
				ASSERT_EQUAL( fileInvalidStatusIncorrect, e.getFilename() );
				ASSERT_EQUAL( 2, e.getLineNumber() );
			}
		}
		void TestOutputDataParser::testParse_invalidAlleleTypeConflict()
		{

			// Creating parser
			OutputDataParserPtr parser1(new OutputDataParser(fileInvalidAlleleTypeConflict, ""));

			try
			{

				parser1->parse();
				ASSERTM( "Expected exception. Nothing throwned", 0 == 1 );
			} catch (ParserException e)
			{

				ASSERT_EQUAL( parser1->getFileType(), e.getFileType() );
				ASSERT_EQUAL( fileInvalidAlleleTypeConflict, e.getFilename() );
				ASSERT_EQUAL( 2, e.getLineNumber() );
			}
		}
		void TestOutputDataParser::testParse_invalidIncorrectColumnsNumber()
		{

			// Creating parser
			OutputDataParserPtr parser1(new OutputDataParser(fileInvalidIncorrectColumnsNumber, ""));

			try
			{

				parser1->parse();
				ASSERTM( "Expected exception. Nothing throwned", 0 == 1 );
			} catch (ParserException e)
			{

				ASSERT_EQUAL( parser1->getFileType(), e.getFileType() );
				ASSERT_EQUAL( fileInvalidIncorrectColumnsNumber, e.getFilename() );
				ASSERT_EQUAL( 2, e.getLineNumber() );
			}
		}
		void TestOutputDataParser::testParse_invalidLocusData()
		{

			// Creating parser
			OutputDataParserPtr parser1(new OutputDataParser(fileInvalidLocusData, ""));

			try
			{

				parser1->parse();
				ASSERTM( "Expected exception. Nothing throwned", 0 == 1 );
			} catch (ParserException e)
			{

				ASSERT_EQUAL( parser1->getFileType(), e.getFileType() );
				ASSERT_EQUAL( fileInvalidLocusData, e.getFilename() );
				ASSERT_EQUAL( 2, e.getLineNumber() );
			}
		}
	}
}
