/*
 * TestOutputDataParser.h
 *
 *  Created on: Dec 13, 2008
 *      Author: mariusz
 */

#ifndef TESTOUTPUTDATAPARSER_H_
#define TESTOUTPUTDATAPARSER_H_

#include <string>
#include <iostream>
#include <fstream>
#include "cute.h"
#include "parsers/OutputDataParser.h"
#include "models/statuses/MCStatusValid.h"
#include "models/statuses/MCStatusExcAlleleMissing.h"
#include "models/statuses/MCStatusExcDuplication.h"
#include "models/statuses/MCStatusExcEquivalence.h"
#include "models/statuses/MCStatusExcHeterozygoticImbalance.h"
#include "models/statuses/MCStatusExcKnownProfile.h"
#include "models/statuses/MCStatusExcMixtureProportion.h"

using namespace dma::external;

namespace dma
{

	namespace test
	{

		class TestOutputDataParser
		{
		private:

			std::string fileValid_;
			std::string fileInvalidSampleNameNotGiven;
			std::string fileInvalidMarkerNameNotGiven;
			std::string fileInvalidMarkerTypeIncorrect1;
			std::string fileInvalidMarkerTypeIncorrect2;
			std::string fileInvalidContributorNumberNotGiven;
			std::string fileInvalidContributorNumberNotNumerical;
			std::string fileInvalidContributorNumberZero;
			std::string fileInvalidStatusNotGiven;
			std::string fileInvalidStatusIncorrect;
			std::string fileInvalidAlleleTypeConflict;
			std::string fileInvalidIncorrectColumnsNumber;
			std::string fileInvalidLocusData;

		public:
			TestOutputDataParser();
			virtual ~TestOutputDataParser();

			void testParse();

			/**
			 * Test parser with following data in file:
			 *
			 * Sample name,Marker name,Marker type,Configuration number,Contributors number,C1 A1 Allele value,C1 A1 Peak height,C1 A1 Peak area,C1 A2 Allele value,C1 A2 Peak height,C1 A2 Peak area,Configuration status
			 * 118_353_1a	AMEL	C	1	2	X	376	4605	X	376	4605	X	1192	14584	Y	2025	25212	EXCLUDED_HETEROZYGOTIC_IMBALANCE	Configuration is excluded since heterozygotic balance is not met for locus X Y. Heterozygotic balance: 0.57848029356220543
			 * 118_353_1a	AMEL	C	2	2	X	973	11898	X	973	11898	Y	1012	12606	Y	1012	12606	VALID	Configuration is VALID
			 * 118_353_1a	D16S539	N	1	1	11	56	827	11	56	827							EXCLUDED_ALLELE_MISSING	Configuration is excluded since allele 12 from mixture sample sample is not given in configuration
			 * 118_353_1a	D16S539	N	2	2	11	90	1335	11	90	1335	11	288	4227	12	275	3983	EXCLUDED_ALLELE_MISSING	Configuration is excluded since allele 13 from mixture sample sample is not given in configuration
			 */
			void testParse_valid();

			void testParse_invalidSampleNameNotGiven();
			void testParse_invalidMarkerNameNotGiven();
			void testParse_invalidMarkerTypeIncorrect1();
			void testParse_invalidMarkerTypeIncorrect2();
			void testParse_invalidContributorNumberNotGiven();
			void testParse_invalidContributorNumberNotNumerical();
			void testParse_invalidContributorNumberZero();
			void testParse_invalidStatusNotGiven();
			void testParse_invalidStatusIncorrect();
			void testParse_invalidAlleleTypeConflict();
			void testParse_invalidIncorrectColumnsNumber();
			void testParse_invalidLocusData();
		};

	}
}
#endif /* TESTOUTPUTDATAPARSER_H_ */
