/*
 * TestPopulationDataParser.cpp
 *
 *  Created on: Dec 21, 2008
 *      Author: mariusz
 */

#include "TestPopulationDataParser.h"

namespace dma
{

	namespace test
	{

		TestPopulationDataParser::TestPopulationDataParser() :
					fileValid_("data/parsers/test_population_data_parser/test_population_data_parser_valid.txt"),
					fileInvalidAlleleAlleleValue_("data/parsers/test_population_data_parser/test_population_data_parser_invalid_allele_value.txt"),
					fileInvalidAlleleFrequency_("data/parsers/test_population_data_parser/test_population_data_parser_invalid_frequency.txt"),
					fileInvalidUnknownMarker_("data/parsers/test_population_data_parser/test_population_data_parser_invalid_unknown_marker.txt")
		{

		}

		TestPopulationDataParser::~TestPopulationDataParser()
		{

		}

		void TestPopulationDataParser::testParse_valid()
		{

			// Creating parser
			PopulationDataParserPtr parser1(new PopulationDataParser(fileValid_, ""));
			parser1->parse();

			/*
			 * Tested file contents:
			 *
			 * Allel	D3S1358
			 * 11	0.0007
			 * 13	0.0034
			 * 14	0.1352
			 * 15	0.2515
			 * 16	0.2376
			 * 17	0.2136
			 * 18	0.1458
			 * 19	0.0116
			 * 20	0.0004
			 * 21	0.0002
			 *
			 * Allel	vWA
			 * 11	0.0007
			 * 13	0.0040
			 * 14	0.1021
			 * 15	0.1055
			 * 16	0.1845
			 * 17	0.2788
			 * 18	0.2351
			 * 19	0.0775
			 * 20	0.0103
			 * 21	0.0016
			 *
			 * Allele	D16S539
			 * 7	0.0002
			 * 8	0.0085
			 * 9	0.0898
			 * 10	0.0403
			 * 11	0.2833
			 * 12	0.3453
			 * 13	0.2022
			 * 14	0.0291
			 * 15	0.0013
			 */

			// Fetching parse data
			MarkerPtrVectorPtr markers = parser1->getMarkers();

			// Testing
			ASSERT_EQUAL( 3, markers->size() );
			ASSERT_EQUAL( "D3S1358", markers->at(0)->getName() );
			ASSERT_EQUAL( "vWA", markers->at(1)->getName() );
			ASSERT_EQUAL( "D16S539", markers->at(2)->getName() );
			ASSERT_EQUAL( 10, markers->at(0)->getAllelesNumber() );
			ASSERT_EQUAL( 10, markers->at(1)->getAllelesNumber() );
			ASSERT_EQUAL( 9, markers->at(2)->getAllelesNumber() );

			ASSERT_EQUAL( 11, markers->at(0)->getAllele(0)->getAlleleValue() );
			ASSERT_EQUAL( 13, markers->at(0)->getAllele(1)->getAlleleValue() );
			ASSERT_EQUAL( 14, markers->at(0)->getAllele(2)->getAlleleValue() );
			ASSERT_EQUAL( 15, markers->at(0)->getAllele(3)->getAlleleValue() );
			ASSERT_EQUAL( 16, markers->at(0)->getAllele(4)->getAlleleValue() );
			ASSERT_EQUAL( 17, markers->at(0)->getAllele(5)->getAlleleValue() );
			ASSERT_EQUAL( 18, markers->at(0)->getAllele(6)->getAlleleValue() );
			ASSERT_EQUAL( 19, markers->at(0)->getAllele(7)->getAlleleValue() );
			ASSERT_EQUAL( 20, markers->at(0)->getAllele(8)->getAlleleValue() );
			ASSERT_EQUAL( 21, markers->at(0)->getAllele(9)->getAlleleValue() );

			ASSERT_EQUAL( 0.0007, markers->at(0)->getAllele(0)->getFrequency() );
			ASSERT_EQUAL( 0.0034, markers->at(0)->getAllele(1)->getFrequency() );
			ASSERT_EQUAL( 0.1352, markers->at(0)->getAllele(2)->getFrequency() );
			ASSERT_EQUAL( 0.2515, markers->at(0)->getAllele(3)->getFrequency() );
			ASSERT_EQUAL( 0.2376, markers->at(0)->getAllele(4)->getFrequency() );
			ASSERT_EQUAL( 0.2136, markers->at(0)->getAllele(5)->getFrequency() );
			ASSERT_EQUAL( 0.1458, markers->at(0)->getAllele(6)->getFrequency() );
			ASSERT_EQUAL( 0.0116, markers->at(0)->getAllele(7)->getFrequency() );
			ASSERT_EQUAL( 0.0004, markers->at(0)->getAllele(8)->getFrequency() );
			ASSERT_EQUAL( 0.0002, markers->at(0)->getAllele(9)->getFrequency() );

		}

		void TestPopulationDataParser::testParse_invalidAlleleAlleleValue()
		{

			// Creating parser
			PopulationDataParserPtr parser1(new PopulationDataParser(fileInvalidAlleleAlleleValue_, ""));

			try
			{

				parser1->parse();
				ASSERTM( "Expected exception. Nothing throwned", 0 == 1 );
			} catch (ParserException e)
			{

				ASSERT_EQUAL( parser1->getFileType(), e.getFileType() );
				ASSERT_EQUAL( fileInvalidAlleleAlleleValue_, e.getFilename() );
				ASSERT_EQUAL( 2, e.getLineNumber() );
			}

		}

		void TestPopulationDataParser::testParse_invalidAlleleFrequency()
		{

			// Creating parser
			PopulationDataParserPtr parser1(new PopulationDataParser(fileInvalidAlleleFrequency_, ""));

			try
			{

				parser1->parse();
				ASSERTM( "Expected exception. Nothing throwned", 0 == 1 );
			} catch (ParserException e)
			{

				ASSERT_EQUAL( parser1->getFileType(), e.getFileType() );
				ASSERT_EQUAL( fileInvalidAlleleFrequency_, e.getFilename() );
				ASSERT_EQUAL( 2, e.getLineNumber() );
			}

		}

		void TestPopulationDataParser::testParse_invalidUnknownMarker()
		{

			// Creating parser
			PopulationDataParserPtr parser1(new PopulationDataParser(fileInvalidUnknownMarker_, ""));

			try
			{

				parser1->parse();
				ASSERTM( "Expected exception. Nothing throwned", 0 == 1 );
			} catch (ParserException e)
			{

				ASSERT_EQUAL( parser1->getFileType(), e.getFileType() );
				ASSERT_EQUAL( fileInvalidUnknownMarker_, e.getFilename() );
				ASSERT_EQUAL( 2, e.getLineNumber() );
			}
		}
	}
}
