/*
 * TestProfileDataParser.cpp
 *
 *  Created on: Dec 6, 2008
 *      Author: mariusz
 */

#include "TestProfileDataParser.h"

namespace dma
{

	namespace test
	{

		TestProfileDataParser::TestProfileDataParser() :
			fileValid_("data/parsers/test_profile_data_parser/test_profile_data_parser_valid.txt"),
			fileInvalidColumnNumber_("data/parsers/test_profile_data_parser/test_profile_data_parser_invalid_column_number.txt"),
			fileInvalidNumericalAllele_("data/parsers/test_profile_data_parser/test_profile_data_parser_invalid_numerical_allele.txt"),
			fileInvalidTypeConflict_("data/parsers/test_profile_data_parser/test_profile_data_parser_invalid_type_conflict.txt"),
			fileInvalidUndefiedAlleleType_("data/parsers/test_profile_data_parser/test_profile_data_parser_invalid_undefined_allele_type.txt")
		{


		}

		TestProfileDataParser::~TestProfileDataParser()
		{

		}

		void TestProfileDataParser::testParse_valid()
		{

			// Creating parser
			ProfileDataParserPtr parser1(new ProfileDataParser(fileValid_, ""));

			// Parsing
			parser1->parse();
			ProfilePtrMapPtr profiles = parser1->getProfiles();

			/*
			 * Tested file contents.
			 *
			 * Sample name / person id,Marker name,Allele 1,Allele 2
			 * PROFILE1,AMEL,X,X
			 * PROFILE1,D16S539,12,13
			 * PROFILE1,D18S51,12,14
			 * PROFILE2,AMEL,X,Y
			 * PROFILE2,D16S539,13,14
			 * PROFILE2,D18S51,11,10
			 */

			// Testing
			ASSERT_EQUAL( 2, profiles->size() );
			ASSERT_EQUAL( 1, profiles->count("PROFILE1") );
			ASSERT_EQUAL( 1, profiles->count("PROFILE2") );

			ASSERT_EQUAL( 3, (*profiles->find("PROFILE1")).second->getMarkerNames()->size() );
			ASSERT_EQUAL( "AMEL", (*profiles->find("PROFILE1")).second->getMarkerNames()->at(0) );
			ASSERT_EQUAL( "D16S539", (*profiles->find("PROFILE1")).second->getMarkerNames()->at(1) );
			ASSERT_EQUAL( "D18S51", (*profiles->find("PROFILE1")).second->getMarkerNames()->at(2) );
			ASSERT_EQUAL( 'X', (*profiles->find("PROFILE1")).second->getMarker("AMEL")->getAllele(0)->getAlleleValue() );
			ASSERT_EQUAL( 'X', (*profiles->find("PROFILE1")).second->getMarker("AMEL")->getAllele(1)->getAlleleValue() );
			ASSERT_EQUAL( 12, (*profiles->find("PROFILE1")).second->getMarker("D16S539")->getAllele(0)->getAlleleValue() );
			ASSERT_EQUAL( 13, (*profiles->find("PROFILE1")).second->getMarker("D16S539")->getAllele(1)->getAlleleValue() );
			ASSERT_EQUAL( 12, (*profiles->find("PROFILE1")).second->getMarker("D18S51")->getAllele(0)->getAlleleValue() );
			ASSERT_EQUAL( 14, (*profiles->find("PROFILE1")).second->getMarker("D18S51")->getAllele(1)->getAlleleValue() );

			ASSERT_EQUAL( 3, (*profiles->find("PROFILE2")).second->getMarkerNames()->size() );
			ASSERT_EQUAL( "AMEL", (*profiles->find("PROFILE2")).second->getMarkerNames()->at(0) );
			ASSERT_EQUAL( "D16S539", (*profiles->find("PROFILE2")).second->getMarkerNames()->at(1) );
			ASSERT_EQUAL( "D18S51", (*profiles->find("PROFILE2")).second->getMarkerNames()->at(2) );
			ASSERT_EQUAL( 'X', (*profiles->find("PROFILE2")).second->getMarker("AMEL")->getAllele(0)->getAlleleValue() );
			ASSERT_EQUAL( 'Y', (*profiles->find("PROFILE2")).second->getMarker("AMEL")->getAllele(1)->getAlleleValue() );
			ASSERT_EQUAL( 13, (*profiles->find("PROFILE2")).second->getMarker("D16S539")->getAllele(0)->getAlleleValue() );
			ASSERT_EQUAL( 14, (*profiles->find("PROFILE2")).second->getMarker("D16S539")->getAllele(1)->getAlleleValue() );
			ASSERT_EQUAL( 10, (*profiles->find("PROFILE2")).second->getMarker("D18S51")->getAllele(0)->getAlleleValue() );
			ASSERT_EQUAL( 11, (*profiles->find("PROFILE2")).second->getMarker("D18S51")->getAllele(1)->getAlleleValue() );
		}

		void TestProfileDataParser::testParse_invalidColumnNumber()
		{

			// Creating parser
			ProfileDataParserPtr parser1(new ProfileDataParser(fileInvalidColumnNumber_, ""));

			try
			{

				parser1->parse();
				ASSERTM( "Expected exception. Nothing throwned", 0 == 1 );
			} catch (ParserException e)
			{

				ASSERT_EQUAL( parser1->getFileType(), e.getFileType() );
				ASSERT_EQUAL( fileInvalidColumnNumber_, e.getFilename() );
				ASSERT_EQUAL( 2, e.getLineNumber() );
			}
		}

		void TestProfileDataParser::testParse_invalidNumericalAllele()
		{

			// Creating parser
			ProfileDataParserPtr parser1(new ProfileDataParser(fileInvalidNumericalAllele_, ""));

			try
			{

				parser1->parse();
				ASSERTM( "Expected exception. Nothing throwned", 0 == 1 );
			} catch (ParserException e)
			{

				ASSERT_EQUAL( parser1->getFileType(), e.getFileType() );
				ASSERT_EQUAL( fileInvalidNumericalAllele_, e.getFilename() );
				ASSERT_EQUAL( 3, e.getLineNumber() );
			}
		}

		void TestProfileDataParser::testParse_invalidTypeConflict()
		{

			// Creating parser
			ProfileDataParserPtr parser1(new ProfileDataParser(fileInvalidTypeConflict_, ""));

			try
			{

				parser1->parse();
				ASSERTM( "Expected exception. Nothing throwned", 0 == 1 );
			} catch (ParserException e)
			{

				ASSERT_EQUAL( parser1->getFileType(), e.getFileType() );
				ASSERT_EQUAL( fileInvalidTypeConflict_, e.getFilename() );
				ASSERT_EQUAL( 2, e.getLineNumber() );
			}
		}

		void TestProfileDataParser::testParse_invalidUndefiedAlleleType()
		{

			// Creating parser
			ProfileDataParserPtr parser1(new ProfileDataParser(fileInvalidUndefiedAlleleType_, ""));

			try
			{

				parser1->parse();
				ASSERTM( "Expected exception. Nothing throwned", 0 == 1 );
			} catch (ParserException e)
			{

				ASSERT_EQUAL( parser1->getFileType(), e.getFileType() );
				ASSERT_EQUAL( fileInvalidUndefiedAlleleType_, e.getFilename() );
				ASSERT_EQUAL( 2, e.getLineNumber() );
			}
		}
	}
}
