/*
 * TestLikelihoodRatioPython.cpp
 *
 *  Created on: Jan 26, 2009
 *      Author: mariusz
 */

#include "TestLikelihoodRatioPython.h"

namespace dma
{

	namespace test
	{

		TestLikelihoodRatioPython::TestLikelihoodRatioPython()
		{

		}

		TestLikelihoodRatioPython::~TestLikelihoodRatioPython()
		{

		}

		void TestLikelihoodRatioPython::testConstruction()
		{

			// Creating algebraic expression for marker A
			AlgValuePtr av10(new AlgValue("p10", 10));
			AlgValuePtr av11(new AlgValue("p11", 11));
			AlgValuePtr av12(new AlgValue("p12", 12));
			AlgValuePtr av13(new AlgValue("p13", 13));
			AlgOperMultiPtr am10_11(new AlgOperMulti(av10, av11));
			AlgOperMultiPtr am12_13(new AlgOperMulti(av12, av13));
			AlgOperAddPtr am_10_11_a_am_12_13(new AlgOperAdd(am10_11, am12_13));

			// Creating algebraic expression for marker B
			AlgValuePtr av20(new AlgValue("p20", 20));
			AlgValuePtr av21(new AlgValue("p21", 21));
			AlgOperMultiPtr am_20_21(new AlgOperMulti(av20, av21));

			// Creating likelihood ration with given algebraic expressions
			LikelihoodRatioPtr lr(new LikelihoodRatio());
			lr->addLRForMarker("MARKER_A", am_10_11_a_am_12_13);
			lr->addLRForMarker("MARKER_B", am_20_21);

			// Creating python representation of likelihood ratio
			LikelihoodRatioPython lrp(lr);

			// Testing marker names retrieval
			std::string markerNames = lrp.getMarkerNames();
			ASSERT_EQUAL( "[ \"MARKER_A\", \"MARKER_B\"]", markerNames);

			std::string formulaMarkerA = lrp.getLrFormula("MARKER_A");
			std::string formulaMarkerB = lrp.getLrFormula("MARKER_B");

			ASSERT_EQUAL( "( ( p10 * p11 ) + ( p12 * p13 ) )", formulaMarkerA );
			ASSERT_EQUAL( "( p20 * p21 )", formulaMarkerB );

			double valueMarkerA = lrp.getLrValue("MARKER_A");
			double valueMarkerB = lrp.getLrValue("MARKER_B");

			ASSERT_EQUAL( 266.0, valueMarkerA );
			ASSERT_EQUAL( 420.0, valueMarkerB );

			double totalLr = lrp.getTotalLrValue();

			ASSERT_EQUAL( 111720.0, totalLr );
		}
	}
}
