import re

#
# Class used for parsing file with profile from single person
# File format:
# [Marker name][;|,|\t][Allele 1 value][;|,|\t][Allele 2 value]
#
# First line may contain column headers or data
class KnownProfileParser:
    
    #
    # Method responsible for parsing data file. Method raises RuntimeError if parsing error is detected
    # 
    # @param filetoBeParsed: Object representing file to be parsed
    # @return: Data in form of array
    def parse(self, fileToBeParsed):
        
        # Creating output container - list
        output = [];
        
        # Getting data from file
        data = fileToBeParsed.read();
        
        # Checking type of delimiter used
        delimiter = '';
        countCF = data.count('\r');
        countLF = data.count('\n');
        
        # Checking whether file is not empty
        if countLF == 0 and countCF == 0:
            
            # TODO Exception should be generated
            return 0;

        if countCF > 0:
            delimiter = '\r';
        
        if countLF > 0:
            delimiter = '\n';

        # Removing CR if it is not and delimeter
        if delimiter == '\n':
            data = data.replace('\r', '');
        
        # Converting data into array of lines
        lines = data.split(delimiter);
        
        # Iterating through lines
        i = 0;
        for line in lines:
           
            # Removing LF
            line = line.replace(delimiter,'');
                            
            # Parsing and validating row
            try:
            
                validatedData = self.parseRow(line, (i+1));
                output.append(validatedData);
            except RuntimeError, re:
                
                # Omitting exception if it has been found in first row
                if i > 0:
                    raise re;
            
            # Incrementing lines counter
            i += 1;
            
        return output;
        
        
    #
    # Parsing single data row. Method returns data in form of array
    # Method raises RuntimeError if parsing error is detected
    #
    # @param line: Line to be parsed
    # @param lineNumber: Line number
    def parseRow(self, line, lineNumber):
      
        # Output container
        output = [];
        
        # Determining which separator has been used
        delimiter = '';
        countTab = line.count('\t');
        countSemiColon = line.count(';');
        countComma = line.count(',');

        if countTab == 0 and countSemiColon == 0 and countComma == 0:
            return output;
        
        if countTab > 0:
            delimiter = '\t';
        if countSemiColon > 0:
            delimiter = ';';
        if countComma > 0:
            delimiter = ',';
            
        # Splitting data
        data = line.split(delimiter);
        
        # Checking whether minimum number of columns has been given
        if  len(data) >= 3:

            # Checking whether allele value are fields or floating point numbers
            flagAllele1 = self.isAlleleValueValid(data[1]);
            flagAllele2 = self.isAlleleValueValid(data[2]);

            if flagAllele1 == True and flagAllele2 == True:
                
                # Assigning data to output variable
                return data;
            else:
                errorMsg = "";
                if flagAllele1 == False:
                    errorMsg += 'Value of allele 1 in row ' + str(lineNumber) + ' is not valid.';    
                if flagAllele2 == False:
                    errorMsg += 'Value of allele 2 in row ' + str(lineNumber) + ' is not valid.';
                
                raise RuntimeError, errorMsg;
    
    #
    # Method checks whether given allele value is valid - it is a character or floating point number
    #
    # @param value Text value of cell
    # @return: Boolean value
    def isAlleleValueValid(self, value):
        
        # Testing for float
        try:
            i = float(value)
        except ValueError:
            
            # Not numeric - checking for character 
            if len(value) == 1:
                return True;
            else:
                return False;
        else:
            # Numeric
            return True;
            
                             