# Create your views here.

import simplejson as json

from django.http import HttpResponse
from django import forms
from django.shortcuts import render_to_response

from dma.frontend.models import *;
from dma.frontend.handlersDeconvolution import *;
from dma.frontend.handlersLrCalculation import *;

##
# Class representing form for starting new mixture analysis - deconvolution
##
class DeconvolutionForm(forms.Form):

    # Mixture sample data fiel selection and type selection
    fileMdf  = forms.FileField(label='Mixture sample: ')
    CHOICES = [(0, 'Columnar Format'), (1, 'Linear Format') ]
    mdfType = forms.ChoiceField(label='Mixture data file format:', choices=CHOICES)    
    fileHbdf  = forms.FileField(label='Heterozygotic Balance: ', required=False);
    useStandardHb = forms.CharField(widget=forms.HiddenInput);

###
# Class representing form for setting up hypothesis for LR calculation
###
class LrCalculationForm(forms.Form):
    
    reportGiven = forms.CharField(widget=forms.HiddenInput);
    reportFilename = forms.CharField(widget=forms.HiddenInput, required=False);
    fileOdf  = forms.FileField(label='Report file from step 1: ', required=False);
    filePdf  = forms.FileField(label='Population data:: ', required=False);
    useStandardPd = forms.CharField(widget=forms.HiddenInput);

def deconvolution(request):
    
    # Checking whether form 
    if request.method == 'POST':
        
        # FORM HAS BEEN ISSUED
        
        # Creating form
        form = DeconvolutionForm(request.POST, request.FILES)
        
        # Checking whether form is valid
        if form.is_valid():
           
            # Handling mixture deconvolution
            result = handleMixtureDeconvolutionRequest(request.POST, request.FILES);
            
            return render_to_response('deconvolutionResult.html', {'result': result });
    else:
        
        # FORM HAS NOT BEEN ISSUED
        data = { 'useStandardHb' : 'True' };
        form = DeconvolutionForm(data);
        

    return render_to_response('deconvolutionForm.html', {'form': form} )

def lrCalculation(request, reportFilename = ''):
    
    if request.method == 'POST':
        
        # FORM HAS BEEN ISSUED
        
        # Handling LR calculation request by usage of library
        result = handleLrCalculationRequest(request.POST, request.FILES);
        
        return render_to_response('lrCalculationResult.html', { 'result' : result} )
    else:
    
        # FORM HAS NOT BEEN ISSUED 
        
        # Checking whether report file has been given
        data = {};
        
        if ( len(reportFilename) > 0 ):
           
            # REPORT FILENAME HAS BEEN GIVEN
            reportPath = settings.REPORT_DIR + '/' + reportFilename;
            data = {
                'reportGiven' : 'True',
                'reportFilename' : reportFilename
            };
            
            flagReportGiven = "True";
        else:
            
            # REPORT FILENAME HAS NOT GIVEN - FILE INPUT FORM SHOULD BE DISPLAYED
            data = {
                'reportGiven' : 'False',
            };
            
            flagReportGiven = "False";
        
        # Creating form
        data['useStandardPd'] = 'True'
        form = LrCalculationForm(data);
         
        return render_to_response('lrCalculationForm.html', {'form': form, 'flagReportGiven': flagReportGiven } )

#
# Function responsible for handling upload of known profiles data file
#
def uploadKpDataFile(request):
    
    response = handleUploadedKpDataFile(request.POST, request.FILES);
    
    # Encoding response in JSON
    response = ( 
        ('responseCode', response.responseCode),
        ('responseBody', response.responseBody),
        ('contributorNumber', response.contributorNumber),
        ('hypothesis', response.hypothesis ) 
        );
    responseJSON = json.dumps( response );
    
    return HttpResponse( responseJSON , mimetype='text/html')
