
/**
 * Method display file upload field for heterozygotic balance minimas
 * 
 * @return
 */
function displayHbFileInput() {

	// Clearing cells
	var td1 = document.getElementById('td_form_fields_hbdf_data1');
	var td2 = document.getElementById('td_form_fields_hbdf_data2');
	
	var i;
	var children = td1.childNodes;
	for (i = 0; i < children.length; i++) {

		td1.removeChild(children[i]);
	}
	td1.innerHTML = '';
	var children = td2.childNodes;
	for (i = 0; i < children.length; i++) {

		td2.removeChild(children[i]);
	}
	td2.innerHTML = '';
	
	// Adding field for selecting data file
	var inputFile = document.createElement('input');
	inputFile.id = 'id_fileHbdf';
	inputFile.name = 'fileHbdf';
	inputFile.type = 'file';
	td1.appendChild(inputFile);
	
	// Displaying un-hide button
	var inputButton = document.createElement('input');
	inputButton.type = 'button';
	inputButton.value = 'Use standard HB min values';
	inputButton.setAttribute('onClick','hideHbFileInput();');
	td2.appendChild(inputButton);
	
	// Setting hidden input indicating whether standard hb is used
	var inputHidden = document.getElementById('id_useStandardHb');
	inputHidden.value = 'False';
	
	return true;
}

/**
 * Method display file upload field for heterozygotic balance minimas
 * 
 * @return
 */
function hideHbFileInput() {

	
	// Clearing cells
	var td1 = document.getElementById('td_form_fields_hbdf_data1');
	var td2 = document.getElementById('td_form_fields_hbdf_data2');
	
	var i;
	var children = td1.childNodes;
	for (i = 0; i < children.length; i++) {

		td1.removeChild(children[i]);
	}
	td1.innerHTML = '';
	var children = td2.childNodes;
	for (i = 0; i < children.length; i++) {

		td2.removeChild(children[i]);
	}
	td2.innerHTML = '';
	
	// Setting information label
	var spanInfo = document.createElement('span');
	spanInfo.innerHTML = 'Using standard HB min values [ <a href="/dma/site_media/datarep/standard_hbmin.csv" target="_blank" >Download file</a> ]';
	td1.appendChild(spanInfo);
	
	// Displaying un-hide button
	var inputButton = document.createElement('input');
	inputButton.type = 'button';
	inputButton.value = 'Upload file with non-standard HB min values';
	inputButton.setAttribute('onClick','displayHbFileInput();');
	td2.appendChild(inputButton);
	
	// Setting hidden input indicating whether standard hb is used
	var inputHidden = document.getElementById('id_useStandardHb');
	inputHidden.value = 'True';
	
	return true;
}

/**
 * Function responsible for validation of given group of fields for setting
 * mixture proportion
 * 
 * @param personNumber
 * @param flagFirstValidation
 * @return
 */
function validateMPData(personNumber) {

	var regexFP = '/^\d+(\.\d+)?$/';

	// Creating debug field
	var debug = document.getElementById('debug_debug');

	// Flag indicating whether all fields are valid
	var flagAllValid = true;

	// Iterating through fields
	var i;
	for (i = 1; i <= personNumber; i++) {

		// Getting value for single field
		var field = document.getElementById('id_mp_' + personNumber + '_' + i);
		field.value = field.value.replace(',', '.');
		var fv = field.value;

		// Checking whether date has been given
		if (fv.length > 0) {

			// Converting to polish notation to ISO
			fv = fv.replace(',', '.');

			// Checking whether whether field is floating-point number
			var flagValid = /^[-+]?[0-9]+(\.[0-9]+)?$/.test(fv);
			if (flagValid == false) {

				field.setAttribute('class', 'mp_textfield_invalid');
				flagAllValid = false;
			} else {

				field.setAttribute('class', 'mp_textfield');
			}
		} else {

			// Data has not been given for a field. Whole group is invalid
			flagAllValid = false;
		}

	}

	// Setting status of given mixture proportion
	var status = document.getElementById('mp_status_' + personNumber);

	if (flagAllValid == true) {

		// Checking whether all parts sum up to 1.0
		var sum = 0;
		for (i = 1; i <= personNumber; i++) {

			var field = document.getElementById('id_mp_' + personNumber + '_' + i);
			var fvFP = parseFloat(field.value);

			sum = sum + fvFP;
		}

		if (sum == 1.0) {

			status.innerHTML = " VALID";
			status.setAttribute('class', 'status_valid');
		} else {

			status.innerHTML = " INVALID - Sum is not equal 1.0";
			status.setAttribute('class', 'status_invalid');
		}
	} else {

		// Some of field are empty or have incorrect value
		status.innerHTML = " INVALID";
		status.setAttribute('class', 'status_invalid');
	}
	
	return flagAllValid;
}

/**
 * Some data has entered to field. If field is the last in the row whole row validation is done
 * 
 * @param element_id ID of element in which key release has been detected
 * @return
 */
function dataFieldKeyUp(element_id) {

	if ( element_id.length == 9 && element_id.substr(0,6) == 'id_mp_' ) {
	
		personNumber = element_id.charAt(6);
		personIndex = element_id.charAt(8);
		
		if ( personNumber == personIndex ) {
		
			validateMPData(personNumber);
		}
	}
}

/**
 * Function responsible for handling event in which one of fields has been
 * changed
 * 
 * @param element_id
 *            ID of an field which has been changed
 * @return
 */
function dataFieldChanged(element_id) {

	// Getting number of persons for which validation should be performed
	var personNumber = element_id.charAt(6);

	// Validating data
	validateMPData(personNumber);
}

/**
 * Add fields responsible for setting mixture proportion for given person number
 * 
 * @param personNumber
 *            Number of persons
 * @return
 */
function addFields(personNumber) {

	// Getting container
	var ulContainer = document.getElementById('mp');

	// Creating container for fields
	var li = document.createElement("li");
	li.id = 'mp_' + personNumber;

	// Creating caption
	var spanCaption = document.createElement("span");
	spanCaption.innerHTML = personNumber + ": ";
	li.appendChild(spanCaption);

	// Creating input for data
	var i;
	for (i = 1; i <= personNumber; i++) {

		// Creating input field
		var input = document.createElement("input");
		input.type = 'text';
		input.name = 'mp_' + personNumber + '_' + i;
		input.id = 'id_mp_' + personNumber + '_' + i;
		input.setAttribute('class', 'mp_textfield');
		input.setAttribute('onchange', 'dataFieldChanged(id)');
		input.setAttribute('onkeyup', 'dataFieldKeyUp(id)');
		

		// Adding input field
		li.appendChild(input);

		// Adding delimiter
		if (i < personNumber) {

			var spanDelimiter = document.createElement("span");
			spanDelimiter.innerHTML = " : ";
			li.appendChild(spanDelimiter);
		}
	}

	// Adding status
	var spanStatus = document.createElement("span");
	spanStatus.id = 'mp_status_' + personNumber;
	spanStatus.setAttribute('class', 'status_invalid');
	spanStatus.innerHTML = ' INVALID';
	li.appendChild(spanStatus);

	// Adding to proportions container
	ulContainer.appendChild(li);

}

/**
 * Subtract given group of fields responsible for mixture proportion for given
 * mixture
 * 
 * @param personNumber
 * @return
 */
function subtractFields(personNumber) {

	// Getting container
	var ulContainer = document.getElementById('mp');

	// Deleting appropriate li
	var i;
	for (i = personNumber + 1; i <= currentMaxPersonNumber; i++) {

		var li = document.getElementById('mp_' + i);
		ulContainer.removeChild(li);
	}
}

/**
 * Function executed when maximum number of persons is changed using drop-down
 * list. Function is responsible for addition or subtraction of appropriate
 * fields for setting mixture proportion for given person number
 * 
 * @return
 */
function maxPersonNumberChanged() {

	// Creating debug field
	// var debug = document.getElementById('debug_debug');

	// Getting new max person number
	var maxPersonNumberField = document.getElementById('id_maxPersonNumber');
	var newMaxPersonNumber = parseInt(maxPersonNumberField.value);

	if (newMaxPersonNumber > currentMaxPersonNumber) {

		// Adding appropriate number of fields for each person number
		var i = currentMaxPersonNumber + 1;
		for (i; i <= newMaxPersonNumber; i++) {

			addFields(i);
		}
	} else {

		// New person number is smaller than last one. Changes may delete data
		subtractFields(newMaxPersonNumber);
	}

	// Updating current number of max persons
	currentMaxPersonNumber = newMaxPersonNumber;
}

/**
 * Method validates form and returns validation result as boolean value
 * 
 * @param form
 * @return
 */
function validateForm(form) {
	
	var flagIsValid = true;
	var errorMessage = 'Errors has been reported:';
	
	// Checking whether mixuture data file has been given
	var inputFileMdf = document.getElementById('id_fileMdf');
	if ( inputFileMdf.value.length == 0 ) {
		flagIsValid = false;
		errorMessage = errorMessage + '\n- ' + 'Mixture data file not given';
	}
	
	// Checking whether hb data file has been ginve if non-standard is used
	var inputUseStandardHb = document.getElementById('id_useStandardHb');
	if ( inputUseStandardHb.value == 'False' ) {
		
		var inputFileHbdf = document.getElementById('id_fileHbdf');
		if ( inputFileHbdf.value.length == 0 ) {
			flagIsValid = false;
			errorMessage = errorMessage + '\n- ' + 'Heterozygotic balance data file not given';
		}
	}
	
	// Checking whether all mixture proportions have been given
	var inputMaxPersonNumberField = document.getElementById('id_maxPersonNumber');
	var inputMaxPersonNumber = parseInt( inputMaxPersonNumberField.value );
	
	var i;
	for(i=2;i<=inputMaxPersonNumber; i++) {
		
		if ( validateMPData(i) == false ) {
			flagIsValid = false;
			errorMessage = errorMessage + '\n- ' + 'Mixture proportion for configuration with '+i+ ' contributors is not valid';
		}
	}
	
	if ( flagIsValid == false ) {
		
		alert(errorMessage);
	}
	return flagIsValid;
}

function log(msg) {

	var debug = document.getElementById('debug');
	debug.innerHTML = debug.innerHTML + msg + '\n';
}