/**
 * Variables used "globally":
 * - Hypothesis type:	ht - 'hp' | 'hd'
 * - Contributor number: cn
 *  
 * ID naming reference:
 * 
 * Container for hypothesis:			main_container_[ht]
 * Contributor profiles as input:		profile_[ht]_[cn]
 * Tables with contributors data:		table_[ht]
 * Rows with contributors data:			tr_[ht]
 * Cells with contributors data: 		td_[ht]_[cn]
 * DIVs for contributor profile:		data_container_[ht]_[cn]
 * Fields for contributor names:		contributor_name_[ht]_[cn]
 * DIVs with uploading form:			form_container_[ht]_[cn]
 * Forms for uploading profile:			upload_form_[ht]_[cn]
 * Tables with contributors profile:	data_table_[ht]_[cn]
 * 
 * 
 * Uploading form fields 
 * kpuf[hypothesis]			Type of hypothesis, hidden
 * kpuf[contributorNumber]	Contributur number, hidden
 * kpuf[file]
 */


/**
 * Method display file upload field for population data file
 * 
 * @return
 */
function displayPdFileInput() {

	// Clearing cells
	var td1 = document.getElementById('td_form_fields_pdf_data1');
	var td2 = document.getElementById('td_form_fields_pdf_data2');
	
	var i;
	var children = td1.childNodes;
	for (i = 0; i < children.length; i++) {

		td1.removeChild(children[i]);
	}
	td1.innerHTML = '';
	var children = td2.childNodes;
	for (i = 0; i < children.length; i++) {

		td2.removeChild(children[i]);
	}
	td2.innerHTML = '';
	
	// Adding field for selecting data file
	var inputFile = document.createElement('input');
	inputFile.id = 'id_filePdf';
	inputFile.name = 'filePdf';
	inputFile.type = 'file';
	td1.appendChild(inputFile);
	
	// Displaying un-hide button
	var inputButton = document.createElement('input');
	inputButton.type = 'button';
	inputButton.value = 'Use standard population data';
	inputButton.setAttribute('onClick','hidePdFileInput();');
	td2.appendChild(inputButton);
	
	// Setting hidden input indicating whether standard hb is used
	var inputHidden = document.getElementById('id_useStandardPd');
	inputHidden.value = 'False';
	
	return true;
}

/**
 * Method display file upload field for population data
 * 
 * @return
 */
function hidePdFileInput() {

	
	// Clearing cells
	var td1 = document.getElementById('td_form_fields_pdf_data1');
	var td2 = document.getElementById('td_form_fields_pdf_data2');
	
	var i;
	var children = td1.childNodes;
	for (i = 0; i < children.length; i++) {

		td1.removeChild(children[i]);
	}
	td1.innerHTML = '';
	var children = td2.childNodes;
	for (i = 0; i < children.length; i++) {

		td2.removeChild(children[i]);
	}
	td2.innerHTML = '';
	
	// Setting information label
	var spanInfo = document.createElement('span');
	spanInfo.innerHTML = 'Using standard population data [ <a href="/dma/site_media/datarep/allele_freq_pl.csv" target="_blank" >Download file</a> ]';
	td1.appendChild(spanInfo);
	
	// Displaying un-hide button
	var inputButton = document.createElement('input');
	inputButton.type = 'button';
	inputButton.value = 'Upload file with non-standard population data';
	inputButton.setAttribute('onClick','displayPdFileInput();');
	td2.appendChild(inputButton);
	
	// Setting hidden input indicating whether standard hb is used
	var inputHidden = document.getElementById('id_useStandardPd');
	inputHidden.value = 'True';
	
	return true;
}

/**
 * Function creates form for uploading profile data
 * 
 * @param ht Hypothesis type - 'hp' | 'hd'
 * @param cn Contributor number
 * @return DOM element of type FORM
 */
function buildFormForUpload(ht, cn) {

	var uploadForm = document.createElement('form');
	uploadForm.id = 'upload_form_' + ht + '_' + cn;
	uploadForm.setAttribute('action', '/dma/frontend/uploadKpDataFile');
	uploadForm.setAttribute('method', 'post');
	uploadForm.setAttribute('enctype', 'multipart/form-data');
	uploadForm.setAttribute('onSubmit',
			'return AIM.submit(this, {\'onStart\' : startCallback, \'onComplete\' : completeCallback})');

	var uploadFormFieldHypothesis = document.createElement('input');
	uploadFormFieldHypothesis.type = 'hidden';
	uploadFormFieldHypothesis.name = 'kpuf[hypothesis]';
	uploadFormFieldHypothesis.value = ht;

	var uploadFormFieldContributorN = document.createElement('input');
	uploadFormFieldContributorN.type = 'hidden';
	uploadFormFieldContributorN.name = 'kpuf[contributorNumber]';
	uploadFormFieldContributorN.value = cn;

	var uploadFormFieldFile = document.createElement('input');
	uploadFormFieldFile.type = 'file';
	uploadFormFieldFile.name = 'kpuf[file]';

	var uploadFormFieldSubmit = document.createElement('input');
	uploadFormFieldSubmit.type = 'submit';
	uploadFormFieldSubmit.value = 'Upload';

	uploadForm.appendChild(uploadFormFieldHypothesis);
	uploadForm.appendChild(uploadFormFieldContributorN);
	uploadForm.appendChild(uploadFormFieldFile);
	uploadForm.appendChild(uploadFormFieldSubmit);

	return uploadForm;
}

/**
 * Function displays form for uploading profile data
 * 
 * @param ht Hypothesis type - 'hp' | 'hd'
 * @param cn Contributor number
 * @return DOM element of type FORM
 */
function displayUploadForm(ht, cn) {
	
	// Getting DIV containing form
	var divUploadForm = document.getElementById('form_container_'+ht+'_'+cn);
	
	// Reseting container
	divUploadForm.innerHTML = "";
	divUploadForm.setAttribute('class','lrc_left');
	
	// Generating form and adding it to DIV
	var uploadForm = buildFormForUpload(ht, cn);
	divUploadForm.appendChild(uploadForm);
}

/**
 * Function hides form for uploading profile data
 * 
 * @param ht Hypothesis type - 'hp' | 'hd'
 * @param cn Contributor number
 * @return DOM element of type FORM
 */
function hideUploadForm(ht, cn) {
	
	// Getting DIV containing form
	var divUploadForm = document.getElementById('form_container_'+ht+'_'+cn);

	// Reseting container
	divUploadForm.innerHTML = "";
	divUploadForm.setAttribute('class','lrc_center');
	
	// Creating button for displaying form
	var displayFormButton = document.createElement('input');
	displayFormButton.type = 'button';
	displayFormButton.value = 'Upload profile data file';
	displayFormButton.setAttribute('onClick','return displayUploadForm(\''+ht+'\','+cn+')');
	
	divUploadForm.appendChild( displayFormButton );
}

/**
 * Function creates table cell in which contributors data will be presented
 * 
 *  @param ht
 * 	          Hypothesis type
 * 
 * @param cn
 *            Contributor number
 * @return DOM Element of type TD 
 */
function createTableCellForContributor(ht,cn) {

	// Creating cell
	var td = document.createElement('td');
	td.id = 'td_' + ht + '_' + cn;
	td.innerHTML = '';

	// Title
	var divTitle = document.createElement('div');
	divTitle.setAttribute('class', 'lrc_center');
	divTitle.innerHTML = '<b>Contibutor #' + cn + '</b>';
	td.appendChild(divTitle);

	// Vertical separator
	var hr1 = document.createElement('hr');
	td.appendChild(hr1);
	
	// Field for contributor name
	var divContributorName = document.createElement('div');
	divContributorName.setAttribute('class', 'lrc_left');
	divContributorName.innerHTML = 'Name: ';
	td.appendChild(divContributorName);
	
	var inputContributorName = document.createElement('input');
	inputContributorName.type = 'text';
	inputContributorName.name = 'contributor_name_' + ht + '_' + cn;
	inputContributorName.id = 'contributor_name_' + ht + '_' + cn;
	inputContributorName.value = 'UNKNOWN_' + ht.toUpperCase() + '_' + cn;
	divContributorName.appendChild(inputContributorName);
	
	// Vertical separator
	var hr1a = document.createElement('hr');
	td.appendChild(hr1a);
	
	// Container for storing contributor's profile
	var divData = document.createElement('div');
	divData.setAttribute('class', 'lrc_left');
	divData.innerHTML = 'No profile given';
	divData.id = 'data_container_' + ht + '_' + cn;
	td.appendChild(divData);

	// Vertical separator
	var hr2 = document.createElement('hr');
	td.appendChild(hr2);

	// Generating container for upload form
	var divUploadForm = document.createElement('div');
	divUploadForm.setAttribute('class', 'lrc_left');
	divUploadForm.id = 'form_container_' + ht + '_' + cn;
	td.appendChild(divUploadForm);

	// Vertical separator
	var hr3 = document.createElement('hr');
	td.appendChild(hr3);

	// Remove button for contributor
	var divRemoveButton = document.createElement('div');
	divRemoveButton.setAttribute('class', 'lrc_center');

	var inputRemoveButton = document.createElement('input');
	inputRemoveButton.setAttribute('onClick', 'removeContributor(' + cn + ',\'' + ht + '\')');
	inputRemoveButton.type = 'button';
	inputRemoveButton.value = 'Remove contributor';

	divRemoveButton.appendChild(inputRemoveButton);
	td.appendChild(divRemoveButton);

	td.setAttribute('class', 'lrc_contributor');
	
	return td;
}

/**
 * Add contributor to selected hypothesis
 * 
 * @param ht Hypothesis type - 'hp' | 'hd'
 * @return
 */
function addContributor(ht) {

	var i;
	var cn = 0;

	// Finding appropriate row
	var tr = document.getElementById('tr_' + ht);

	// Finding current contributor number
	var children = tr.childNodes;
	for (i = 0; i < children.length; i++) {

		// Filtering DOM elements
		if (children[i].nodeType == 1) {

			// Filtering TD elements
			if (children[i].nodeName.toLowerCase() == 'td') {

				// Filtering elements basing on ID prefix
				if (children[i].id.length >= 7 && children[i].id.substring(0, 4) == 'td_h') {

					cn = parseInt(children[i].id.charAt(6));
				}
			}
		}
	}

	// Calculating next cell id
	var nextCn = cn + 1;

	// Creating next cell
	var td = createTableCellForContributor(ht, nextCn);
	tr.appendChild(td);

	// Hiding upload form
	hideUploadForm(ht, nextCn);
	
	return true;
}

/**
 * Remove data of contributor with given number
 * 
 * @param cn
 *            Contributor number
 * @param ht
 *            Hypothesis type
 * @return
 */
function removeContributor(cn, ht) {

	// Finding appropriate row
	var tr = document.getElementById('tr_' + ht);

	// Finding current contributor number
	var children = tr.childNodes;
	for (i = 0; i < children.length; i++) {

		// Filtering DOM elements
		if (children[i].nodeType == 1) {

			// Filtering TD elements
			if (children[i].nodeName.toLowerCase() == 'td') {

				// Filtering elements basing on ID prefix
				if (children[i].id.length >= 7 && children[i].id.substring(0, 4) == 'td_h') {

					var currentCn = parseInt(children[i].id.charAt(6));
					if (currentCn == cn) {
						
						tr.removeChild(children[i]);
					}
				}
			}
		}
	}
	
	// Deleting input with profile
	var lrForm = document.getElementById('lrForm');
	var inputProfile = document.getElementById('profile_' + ht + '_' + cn);
	
	if ( inputProfile != NULL ) {
		
		lrForm.removeChild(inputProfile);
	}
	
	// Deleting contributor name input
	var inputContriburName = document.getElementById('contributor_name_' + ht + '_' + cn);
	if ( inputContriburName != NULL ) {
		
		inputContriburName.parent.removeChild( inputContriburName );
	}

	return true;
}



/**
 * Method creates table with data regarding profile
 * 
 * @param data - Data given in form of 2-dimensional array [n,3], where n is number of data rows 
 * @param ht - Hypothesis type - 'hp' | 'hd'
 * @param cn - Contributor number
 * @return DOM element of type TABLE
 */
function buildTableWithProfileData(data, ht, cn) {

	// Creating table
	var table = document.createElement('table');
	table.id = 'data_table_' + ht + '_' + cn;

	// Creating table header
	var trHeader = document.createElement('tr');
	var thHeader1 = document.createElement('th');
	var thHeader2 = document.createElement('th');
	var thHeader3 = document.createElement('th');
	thHeader1.innerHTML = 'Marker';
	thHeader2.innerHTML = 'Allele 1';
	thHeader3.innerHTML = 'Allele 2';
	trHeader.appendChild(thHeader1);
	trHeader.appendChild(thHeader2);
	trHeader.appendChild(thHeader3);
	table.appendChild(trHeader);

	// Inserting data
	var i, j;
	var sizeI = data.length;
	for (i = 0; i < sizeI; i++) {

		// Fetching single data row
		var row = data[i];

		// Creating cells
		var tr = document.createElement('tr');
		for (j = 0; j < row.length; j++) {

			var td = document.createElement('td');
			td.innerHTML = row[j];
			tr.appendChild(td);
		}

		// Adding row to table
		table.appendChild(tr);
	}

	return table;
}

/**
 * Function called when before file upload is started
 * 
 * @return
 */
function startCallback() {

	return true;
}

/**
 * Function called when uploading process ends and server generated response
 * 
 * @param response
 * @return
 */
function completeCallback(response) {

	try {

		// Parsing response given in JSON
		var responseObj = JSON.parse(response);

		// Checking whether JSON response has proper lenght
		if (responseObj.length == 4) {

			// Fetching data
			var responseCode = responseObj[0][1];
			var responseBody = responseObj[1][1];
			var cn = responseObj[2][1];
			var ht = responseObj[3][1];

			// Finding container for data
			var divData = document.getElementById('data_container_' + ht + '_' + cn);

			// Finding target output element
			if (responseCode == 200) {

				// Displaying profile
				var profileTable = buildTableWithProfileData(responseBody, ht, cn);
				divData.innerHTML = "";
				divData.appendChild(profileTable);
				
				// Adding profile to hidden input
				var lrForm = document.getElementById('lrc_form');
				var inputProfile = document.createElement('input');
				inputProfile.type = 'hidden';
				inputProfile.id = 'profile_' + ht + '_' + cn;
				inputProfile.name = 'profile_' + ht + '_' + cn;
				inputProfile.value = JSON.stringify(responseBody);
				lrForm.appendChild(inputProfile);
				
				// Hiding form
				hideUploadForm(ht, cn);

			} else if (responseCode == 500) {

				divData.innerHTML = "<b>Error has been reported:</b><br/>";
				divData.innerHTML += responseBody;
			} else {

				
			}
		} else {

			divData.innerHTML = "<b>Error has been reported:</b><br/>";
			divData.innerHTML = "Response incomplete. Number of elements in response: " + responseObj.length;
		}
	} catch (e) {

		var errorMsg = "";
		if (e instanceof SyntaxError) {

			// Error has been reported while parsing response
			errorMsg += "Error has been reported: ";
			errorMsg += "Response from server couldn't be parsed" + response;
		} else {

			errorMsg += "Error has been reported, while executing upload: " + e;
		}

		alert(errorMsg);
	}

	return true
}

function log(msg) {

	var debug = document.getElementById('debug');
	debug.innerHTML = debug.innerHTML + msg + '\n';
}